@@ -1,4 +1,5 @@
 from numbers import Integral, Real
+from collections.abc import Iterable
 
 import numpy as np
 
@@ -10,7 +11,7 @@ from ._help import array_fill_repeat
 from .shape import Sphere
 from .orbital import Orbital
 
-__all__ = ['PeriodicTable', 'Atom', 'Atoms']
+__all__ = ['PeriodicTable', 'Atom', 'AtomUnknown', 'AtomGhost', 'Atoms']
 
 
 @set_module("sisl")
@@ -23,7 +24,7 @@ class PeriodicTable:
     Several quantities available to the atomic species are available
     from <https://en.wikipedia.org/wiki/Atomic_radii_of_the_elements_(data_page)>.
 
-    The following values are accesible:
+    The following values are accessible:
 
     * atomic mass (in atomic units)
     * empirical atomic radii (in Ang)
@@ -816,7 +817,7 @@ class PeriodicTable:
         get = self._Z_int.get
         if len(key) == 1:
             return get(key[0], key[0])
-        return _a.asarrayi([get(ia, ia) for ia in key])
+        return _a.fromiteri(map(get, key, key))
 
     Z_int = Z
 
@@ -840,8 +841,8 @@ class PeriodicTable:
         ak = np.asarray(key).ravel()
         get = self._Z_short.get
         if len(ak) == 1:
-            return get(ak[0], 'fa')
-        return [get(ia, 'fa') for ia in ak]
+            return get(ak[0], "fa")
+        return [get(ia, "fa") for ia in ak]
 
     Z_short = Z_label
 
@@ -866,7 +867,7 @@ class PeriodicTable:
         get = self._atomic_mass.get
         if isinstance(Z, (Integral, Real)):
             return get(Z, 0.)
-        return _a.arrayd([get(z, 0.) for z in Z])
+        return _a.fromiterd(map(get, Z, iter(float, 1)))
 
     def radius(self, key, method='calc'):
         """ Atomic radii using different methods
@@ -891,20 +892,10 @@ class PeriodicTable:
             atomic radius in `Ang`
         """
         Z = self.Z_int(key)
-        if method == 'calc':
-            if isinstance(Z, Integral):
-                return self._radius_calc[Z] / 100
-            return _a.arrayd([self._radius_calc[i] for i in Z]) / 100
-        elif method == 'empirical':
-            if isinstance(Z, Integral):
-                return self._radius_empirical[Z] / 100
-            return _a.arrayd([self._radius_empirical[i] for i in Z]) / 100
-        elif method == 'vdw':
-            if isinstance(Z, Integral):
-                return self._radius_vdw[Z] / 100
-            return _a.arrayd([self._radius_vdw[i] for i in Z]) / 100
-        raise ValueError(
-            'Method option could not be deciphered [calc|empirical|vdw].')
+        func = getattr(self, f"_radius_{method}").get
+        if isinstance(Z, Integral):
+            return func(Z) / 100
+        return _a.fromiterd(map(func, Z)) / 100
     radii = radius
 
 # Create a local instance of the periodic table to
@@ -953,19 +944,6 @@ class Atom(metaclass=AtomMeta):
 
     The `Atom` object is `pickle`-able.
 
-    Attributes
-    ----------
-    Z : int
-        atomic number
-    no : int
-        number of orbitals belonging to the `Atom`
-    R : numpy.ndarray
-        the range of each orbital associated with this `Atom` (see `Orbital.R` for details)
-    q0 : numpy.ndarray
-        the charge of each orbital associated with this `Atom` (see `Orbital.q0` for details)
-    mass : float
-        mass of `Atom`
-
     Parameters
     ----------
     Z : int or str
@@ -978,58 +956,90 @@ class Atom(metaclass=AtomMeta):
         arbitrary designation for user handling similar atoms with
         different settings (defaults to the label of the atom)
     """
+    def __new__(cls, *args, **kwargs):
+        """ Figure out which class to actually use """
+        # Handle the case where no arguments are passed (e.g. for serializing stuff)
+        if len(args) == 0 and "Z" not in kwargs:
+            return super().__new__(cls)
+
+        # direct call
+        if len(args) > 0:
+            Z = args[0]
+        else:
+            Z = None
+        Z = kwargs.get("Z", Z)
+        if isinstance(Z, Atom):
+            return super().__new__(Z.__class__)
+        elif isinstance(Z, Integral) and not issubclass(cls, AtomGhost) and Z < 0:
+            cls = AtomGhost
+        elif Z not in _ptbl._Z_int and not issubclass(cls, AtomUnknown):
+            cls = AtomUnknown
+        return super().__new__(cls)
 
     def __init__(self, Z, orbitals=None, mass=None, tag=None, **kwargs):
         if isinstance(Z, Atom):
-            Z = Z.Z
-        self.Z = _ptbl.Z_int(Z)
+            self._Z = Z.Z
+        elif isinstance(Z, Integral):
+            self._Z = Z
+        else:
+            self._Z = _ptbl.Z_int(Z)
 
-        self.orbitals = None
+        self._orbitals = None
         if isinstance(orbitals, (tuple, list, np.ndarray)):
             if isinstance(orbitals[0], Orbital):
                 # all is good
-                self.orbitals = orbitals
+                self._orbitals = orbitals
             elif isinstance(orbitals[0], Real):
                 # radius has been given
-                self.orbitals = [Orbital(R) for R in orbitals]
+                self._orbitals = [Orbital(R) for R in orbitals]
         elif isinstance(orbitals, Orbital):
-            self.orbitals = [orbitals]
+            self._orbitals = [orbitals]
         elif isinstance(orbitals, Real):
-            self.orbitals = [Orbital(orbitals)]
+            self._orbitals = [Orbital(orbitals)]
 
-        if self.orbitals is None:
+        if self._orbitals is None:
             if 'R' in kwargs:
                 # backwards compatibility (possibly remove this in the future)
                 R = _a.asarrayd(kwargs['R']).ravel()
-                self.orbitals = [Orbital(r) for r in R]
+                self._orbitals = [Orbital(r) for r in R]
             else:
-                self.orbitals = [Orbital(-1.)]
+                self._orbitals = [Orbital(-1.)]
 
         if mass is None:
-            self.mass = _ptbl.atomic_mass(self.Z)
+            self._mass = _ptbl.atomic_mass(self.Z)
         else:
-            self.mass = mass
+            self._mass = mass
 
         if tag is None:
-            self.tag = self.symbol
+            self._tag = self.symbol
         else:
-            self.tag = tag
+            self._tag = tag
+
+    @property
+    def Z(self):
+        """ Atomic number """
+        return self._Z
+
+    @property
+    def orbitals(self):
+        """ List of orbitals """
+        return self._orbitals
+
+    @property
+    def mass(self):
+        """ Atomic mass """
+        return self._mass
+
+    @property
+    def tag(self):
+        """ Tag for atom """
+        return self._tag
 
     @property
     def no(self):
         """ Number of orbitals on this atom """
         return len(self.orbitals)
 
-    @property
-    def R(self):
-        """ Orbital radius """
-        return _a.arrayd([o.R for o in self.orbitals])
-
-    @property
-    def q0(self):
-        """ Orbital initial charges """
-        return _a.arrayd([o.q0 for o in self.orbitals])
-
     def index(self, orbital):
         """ Return the index of the orbital in the atom object """
         if not isinstance(orbital, Orbital):
@@ -1135,7 +1145,7 @@ class Atom(metaclass=AtomMeta):
            the scale factor for the atomic radii
         """
         new = self.copy()
-        new.orbitals = [o.scale(scale) for o in self.orbitals]
+        new._orbitals = [o.scale(scale) for o in self.orbitals]
         return new
 
     def __iter__(self):
@@ -1185,6 +1195,62 @@ class Atom(metaclass=AtomMeta):
         """ Return number of orbitals in this atom """
         return self.no
 
+    def __getattr__(self, attr):
+        """ Pass attribute calls to the orbital classes and return lists/array
+
+        Parameters
+        ----------
+        attr : str
+        """
+
+        # First we create a list of values that the orbitals have
+        # Some may have it, others may not
+        vals = [None] * len(self.orbitals)
+        found = False
+        is_Integral = is_Real = is_callable = True
+        for io, orb in enumerate(self.orbitals):
+            try:
+                vals[io] = getattr(orb, attr)
+                found = True
+                is_callable &= callable(vals[io])
+                is_Integral &= isinstance(vals[io], Integral)
+                is_Real &= isinstance(vals[io], Real)
+            except AttributeError:
+                pass
+
+        if found == 0:
+            # we never got any values, reraise the AttributeError
+            raise AttributeError(f"'{self.__class__.__name__}.orbitals' objects has no attribute '{attr}'")
+
+        # Now parse the data, currently we'll only allow Integral, Real, Complex
+        if is_Integral:
+            for io in range(len(vals)):
+                if vals[io] is None:
+                    vals[io] = 0
+            return _a.arrayi(vals)
+        elif is_Real:
+            for io in range(len(vals)):
+                if vals[io] is None:
+                    vals[io] = 0.
+            return _a.arrayd(vals)
+        elif is_callable:
+            def _ret_none(*args, **kwargs):
+                return None
+            for io in range(len(vals)):
+                if vals[io] is None:
+                    vals[io] = _ret_none
+
+            # Now subclass the content and return values per method
+            class ArrayCall:
+                def __init__(self, methods):
+                    self.methods = methods
+                def __call__(self, *args, **kwargs):
+                    return [m(*args, **kwargs) for m in self.methods]
+            return ArrayCall(vals)
+
+        # We don't know how to handle this, simply return...
+        return vals
+
     def toSphere(self, center=None):
         """ Return a sphere with the maximum orbital radius equal
 
@@ -1235,6 +1301,29 @@ class Atom(metaclass=AtomMeta):
         self.__init__(d['Z'], d['orbitals'], d['mass'], d['tag'])
 
 
+@set_module("sisl")
+class AtomUnknown(Atom):
+    def __init__(self, Z, *args, **kwargs):
+        """ Instantiate with overridden tag """
+        if len(args) < 3 and "tag" not in kwargs:
+            kwargs["tag"] = "unknown"
+        super().__init__(Z, *args, **kwargs)
+
+
+@set_module("sisl")
+class AtomGhost(AtomUnknown):
+    def __init__(self, Z, *args, **kwargs):
+        """ Instantiate with overridden tag and taking the absolute value of Z """
+        if Z < 0:
+            Z = - Z
+        if len(args) < 3 and "tag" not in kwargs:
+            kwargs["tag"] = "ghost"
+        if len(args) < 2 and "mass" not in kwargs:
+            # A very high default mass (unless specified)
+            kwargs["mass"] = 1e40
+        super().__init__(Z, *args, **kwargs)
+
+
 @set_module("sisl")
 class Atoms:
     """ A list-like object to contain a list of different atoms with minimum
@@ -1252,26 +1341,6 @@ class Atoms:
     na : int or None
        total number of atoms, if ``len(atom)`` is smaller than `na` it will
        be repeated to match `na`.
-
-    Attributes
-    ----------
-    atom : list of Atom
-        a list of unique atoms in this object
-    specie : (na, )
-        a list of unique specie indices
-    no : int
-        total number of orbitals
-    q0 : (no, )
-        initial charge on each orbital
-    mass : (na, )
-        mass for each atom
-    firsto : (no + 1,)
-        a list of orbital indices for each atom, this corresponds to the first
-        orbital on each of the atoms. The last element is the total number of
-        orbitals and is equivalent to `no`.
-    lasto : (no, )
-        a list of orbital indices for each atom, this corresponds to the last
-        orbital on each of the atoms.
     """
 
     # Using the slots should make this class slightly faster.
@@ -1284,43 +1353,7 @@ class Atoms:
             atoms = Atom('H')
 
         # Correct the atoms input to Atom
-        if isinstance(atoms, (np.ndarray, list, tuple)):
-            # Convert to a list of unique elements
-            # We can not use set because that is unordered
-            # And we want the same order, always...
-            uatoms = []
-            specie = [0] * len(atoms)
-            if isinstance(atoms[0], Atom):
-                for i, a in enumerate(atoms):
-                    try:
-                        s = uatoms.index(a)
-                    except:
-                        s = -1
-                    if s < 0:
-                        s = len(uatoms)
-                        uatoms.append(a)
-                    specie[i] = s
-
-            elif isinstance(atoms[0], (str, Integral)):
-                for i, a in enumerate(atoms):
-                    a = Atom(a)
-                    try:
-                        s = uatoms.index(a)
-                    except:
-                        s = -1
-                    if s < 0:
-                        s = len(uatoms)
-                        uatoms.append(a)
-                    specie[i] = s
-
-            else:
-                raise ValueError('atoms keyword was wrong input')
-
-        elif isinstance(atoms, (str, Integral)):
-            uatoms = [Atom(atoms)]
-            specie = [0]
-
-        elif isinstance(atoms, Atom):
+        if isinstance(atoms, Atom):
             uatoms = [atoms]
             specie = [0]
 
@@ -1331,6 +1364,23 @@ class Atoms:
             uatoms = catoms.atom[:]
             specie = catoms.specie[:]
 
+        elif isinstance(atoms, (str, Integral)):
+            uatoms = [Atom(atoms)]
+            specie = [0]
+
+        elif isinstance(atoms, Iterable):
+            uatoms = []
+            specie = []
+            for a in atoms:
+                if not isinstance(a, Atom):
+                    a = Atom(a)
+                try:
+                    s = uatoms.index(a)
+                except:
+                    s = len(uatoms)
+                    uatoms.append(a)
+                specie.append(s)
+
         else:
             raise ValueError('atoms keyword was wrong input')
 
@@ -1371,28 +1421,28 @@ class Atoms:
 
     @property
     def specie(self):
-        """ Atomic specie list """
+        """ List of atomic species """
         return self._specie
 
     @property
     def no(self):
-        """ Return the total number of orbitals in this list of atoms """
+        """ Total number of orbitals in this list of atoms """
         uorbs = _a.arrayi([a.no for a in self.atom])
         return uorbs[self.specie].sum()
 
     @property
     def orbitals(self):
-        """ Return an array of orbitals of the contained objects """
+        """ Array of orbitals of the contained objects """
         return np.diff(self.firsto)
 
     @property
     def firsto(self):
-        """ The first orbital of the corresponding atom in the consecutive list of orbitals """
+        """ First orbital of the corresponding atom in the consecutive list of orbitals """
         return self._firsto
 
     @property
     def lasto(self):
-        """ The lasto orbital of the corresponding atom in the consecutive list of orbitals """
+        """ Last orbital of the corresponding atom in the consecutive list of orbitals """
         return self._firsto[1:] - 1
 
     @property
@@ -1431,13 +1481,13 @@ class Atoms:
 
     @property
     def mass(self):
-        """ Return an array of masses of the contained objects """
+        """ Array of masses of the contained objects """
         umass = _a.arrayd([a.mass for a in self.atom])
         return umass[self.specie[:]]
 
     @property
     def Z(self):
-        """ Return an array of atomic numbers (integers) """
+        """ Array of atomic numbers """
         uZ = _a.arrayi([a.Z for a in self.atom])
         return uZ[self.specie[:]]
 
@@ -1463,6 +1513,33 @@ class Atoms:
                 return s
         raise KeyError('Could not find `atom` in the list of atoms.')
 
+    def group_atom_data(self, data, axis=0):
+        r""" Group data for each atom based on number of orbitals
+
+        This is useful for grouping data that is orbitally resolved.
+        This will return a list of length ``len(self)`` and with each item
+        having the sub-slice of the data corresponding to the orbitals on the given
+        atom.
+
+        Examples
+        --------
+        >>> atoms = Atoms([Atom(4, [0.1, 0.2]), Atom(6, [0.2, 0.3, 0.5])])
+        >>> orb_data = np.arange(10).reshape(2, 5)
+        >>> atoms.group_data(orb_data, axis=1)
+        [
+         [[0, 1], [2, 3]],
+         [[4, 5, 6], [7, 8, 9]]
+        ]
+
+        Parameters
+        ----------
+        data : numpy.ndarray
+           data to be grouped
+        axis : int, optional
+           along which axis one should split the data
+        """
+        return np.split(data, self.lasto[:-1] + 1, axis=axis)
+
     def reorder(self, in_place=False):
         """ Reorders the atoms and species index so that they are ascending (starting with a specie that exists)
 
