@@ -5,9 +5,10 @@ from math import pi
 import numpy as np
 from numpy import int32
 from numpy import floor, dot, add, cos, sin
-from numpy import ogrid, take
+from numpy import ogrid, take, asarray
 from scipy.sparse import diags as sp_diags
 from scipy.sparse import SparseEfficiencyWarning
+from scipy.ndimage import zoom as ndimage_zoom
 
 from ._internal import set_module
 from . import _array as _a
@@ -17,6 +18,7 @@ from .shape import Shape
 from .utils import default_ArgumentParser, default_namespace
 from .utils import cmd, strseq, direction, str_spec
 from .utils import array_arange
+from .utils import import_attr
 from .utils.mathematics import fnorm
 
 from .supercell import SuperCellChild
@@ -139,59 +141,168 @@ class Grid(SuperCellChild):
         """
         self.grid.fill(val)
 
-    def interp(self, shape, method='linear', **kwargs):
-        """ Interpolate grid values to a new grid
+    def interp(self, shape, order=1, mode="wrap", **kwargs):
+        """ Interpolate grid values to a new grid of a different shape
+
+        It uses the `scipy.ndimage.zoom`, which creates a finer or
+        more spaced grid using spline interpolation.
 
         Parameters
         ----------
-        shape : int, array_like
-            the new shape of the grid
-        method : str
-            the method used to perform the interpolation,
-            see `scipy.interpolate.interpn` for further details.
-        **kwargs : dict
+        shape : int, array_like of len 3
+            the new shape of the grid.
+        order : int 0-5, optional
+            the order of the spline interpolation.
+            1 means linear, 2 quadratic, etc...
+        mode: {'wrap', 'mirror', 'constant', 'reflect', 'nearest'}
+            determines how to compute the borders of the grid.
+            The default is ``'wrap'``, which accounts for periodic conditions.
+        **kwargs :
             optional arguments passed to the interpolation algorithm
-            The interpolation routine is `scipy.interpolate.interpn`
+            The interpolation routine is `scipy.ndimage.zoom`
+
+        See Also
+        --------
+        scipy.ndimage.zoom : method used for interpolation
         """
-        # Interpolate function
-        from scipy.interpolate import RegularGridInterpolator
+        # For backwards compatibility
+        method = kwargs.pop("method", None)
+        # Maybe the method was passed as a positional argument
+        if isinstance(order, str):
+            method = order
+        if method is not None:
+            order = {'linear': 1}.get(method, 3)
 
-        # Current dimensions of size 1
-        idx = tuple(i for i in range(3) if self.shape[i] > 1)
-        idx_1 = tuple(i for i in range(3) if not i in idx)
+        # And now we do the actual interpolation
+        # Calculate the zoom_factors
+        zoom_factors = _a.arrayd(shape) / self.shape
 
-        # Get current grid spacing
-        dold = tuple(_a.linspacef(0, 1, self.shape[i]) for i in idx)
+        # Apply the scipy.ndimage.zoom function and return a new grid
+        return self.apply(ndimage_zoom, zoom_factors, mode=mode, order=order, **kwargs)
 
-        # Create new grid and clean-up to reduce memory
+    def isosurface(self, level, step_size=1, **kwargs):
+        """Calculates the isosurface for a given value.
+
+        It uses `skimage.measure.marching_cubes`, so you need to have scikit-image installed.
+
+        Parameters
+        ----------
+        level: float
+            contour value to search for isosurfaces in the grid. 
+            If not given or None, the average of the min and max of the grid is used.
+        step_size: int, optional
+            step size in voxels. Larger steps yield faster but coarser results. 
+            The result will always be topologically correct though.
+        **kwargs:
+            optional arguments passed directly to `skimage.measure.marching_cubes`
+            for the calculation of isosurfaces.
+
+        Returns
+        ----------
+        numpy array of shape (V, 3)
+            Verts. Spatial coordinates for V unique mesh vertices.
+
+        numpy array of shape (n_faces, 3)
+            Faces. Define triangular faces via referencing vertex indices from verts. 
+            This algorithm specifically outputs triangles, so each face has exactly three indices.
+
+        numpy array of shape (V, 3)
+            Normals. The normal direction at each vertex, as calculated from the data.
+
+        numpy array of shape (V, 3)
+            Values. Gives a measure for the maximum value of the data in the local region near each vertex. 
+            This can be used by visualization tools to apply a colormap to the mesh.
+
+        See Also
+        --------
+        skimage.measure.marching_cubes : method used to calculate the isosurface.
+        """
+        try:
+            import skimage.measure
+        except ModuleNotFoundError:
+            raise ModuleNotFoundError(
+                f"{self.__class__.__name__}.isosurface requires scikit-image to be installed"
+            )
+
+        # Run the marching cubes algorithm to calculate the vertices and faces
+        # of the requested isosurface.
+        verts, *returns = skimage.measure.marching_cubes(
+            self.grid, level=level, step_size=step_size, **kwargs
+        )
+
+        # The verts cordinates are in fractional coordinates of unit-length.
+        verts = self.index2xyz(verts)
+
+        return (verts, *returns)
+
+    def smooth(self, r=0.7, method="gaussian", mode="wrap", **kwargs):
+        """ Make a smoother grid by applying a filter.
+
+        Parameters
+        -----------
+        r: float or array-like of len 3, optional
+            the radius of the filter in Angstrom for each axis.
+            If the method is ``"gaussian"``, this is the standard deviation!
+
+            If a single float is provided, then the same distance will be used for all axes.
+        method: {'gaussian', 'uniform'}
+            the type of filter to apply to smoothen the grid.
+        mode: {'wrap', 'mirror', 'constant', 'reflect', 'nearest'}
+            determines how to compute the borders of the grid.
+            The default is wrap, which accounts for periodic conditions.
+
+        See Also
+        --------
+        scipy.ndimage.gaussian_filter
+        """
+
+        # Normalize the radius input to a list of radius
+        if isinstance(r, Real):
+            r = [r, r, r]
+
+        # Calculate the size of the kernel in pixels (in case the
+        # gaussian filter is used, this is the standard deviation)
+        pixels_r = np.round(r / fnorm(self.dcell)).astype(np.int32)
+
+        # Update the kwargs accordingly
+        if method == "gaussian":
+            kwargs['sigma'] = pixels_r
+        elif method == "uniform":
+            kwargs['size'] = pixels_r * 2
+
+        # This should raise an import error if the method does not exist
+        func = import_attr(f"scipy.ndimage.{method}_filter")
+        return self.apply(func, mode=mode, **kwargs)
+
+    def apply(self, function_, *args, **kwargs):
+        """ Applies a function to the grid and returns a new grid.
+
+        You can also apply a function that does not return a grid (maybe you want to do
+        some measurement). In that case, you will get the result instead of a `Grid`.
+
+        Parameters
+        -----------
+        function_: str or function
+            for a string the full module path to the function should be given.
+            The function that will be called should have the grid as the first argument in its
+            interface.
+        args and kwargs:
+            arguments that go directly to the function call
+        """
+        if isinstance(function_, str):
+            function_ = import_attr(function_)
+
+        result = function_(self.grid, *args, **kwargs)
+
+        # Maybe the result is not a grid, because there are methods that actually
+        # do measurements of the grid
+        # TODO what to do about functions that squeeze shape == 1 dimensions?
+        if not isinstance(result, np.ndarray) or result.ndim != 3:
+            return result
+
+        # If the result is a grid, we will generate a copy of this one with the new grid values
         grid = self.copy()
-        del grid.grid
-
-        # Create the interpolator!
-        # And remove 1 dimensions
-        f = RegularGridInterpolator(dold, np.squeeze(self.grid), method=method)
-        del dold
-
-        # Create new interpolation points
-        dnew = tuple(_a.linspacef(0, 1, shape[i]) for i in idx)
-
-        grid.grid = np.empty(shape, dtype=self.dtype)
-
-        # Special case where grid has different dimensionality
-        if len(dnew) == 0:
-            # all is copied
-            grid.grid[...] = self.grid.ravel()
-        elif len(dnew) < 3:
-            reshape = tuple(shape[i] if i in idx else 1 for i in range(3))
-            sl = tuple(slice(0, 1, shape[i] * 1j) for i in idx)
-            dnew = np.mgrid[sl].reshape(len(sl), -1).T
-            grid.grid[:, :, :] = f(dnew).reshape(reshape)
-        else:
-            for iz, dz in enumerate(dnew[2]):
-                dnew = np.mgrid[0:1:shape[0] * 1j, 0:1:shape[1] * 1j, dz:dz+0.1:1j].reshape(3, -1).T
-                grid.grid[:, :, iz] = f(dnew).reshape((shape[0], shape[1]))
-
-        del dnew
+        grid.grid = result
 
         return grid
 
@@ -509,6 +620,33 @@ class Grid(SuperCellChild):
         ret_idx = np.delete(_a.arangei(self.shape[axis]), _a.asarrayi(idx))
         return self.sub(ret_idx, axis)
 
+    def tile(self, reps, axis):
+        """ Tile grid to create a bigger one
+
+        The atomic indices for the base Geometry will be retained.
+
+        Parameters
+        ----------
+        reps : int
+           number of tiles (repetitions)
+        axis : int
+           direction of tiling, 0, 1, 2 according to the cell-direction
+
+        See Also
+        --------
+        Geometry.tile : equivalent method for Geometry class
+        """
+        grid = self.copy()
+        grid.grid = None
+        reps_all = [1, 1, 1]
+        reps_all[axis] = reps
+        grid.grid = np.tile(self.grid, reps_all)
+        if self.geometry is None:
+            grid.set_sc(self.sc.tile(reps, axis))
+        else:
+            grid.set_geometry(self.geometry.tile(reps, axis))
+        return grid
+
     def index2xyz(self, index):
         """ Real-space coordinates of indices related to the grid
 
@@ -522,7 +660,7 @@ class Grid(SuperCellChild):
         numpy.ndarray
            coordinates of the indices with respect to this grid spacing
         """
-        return dot(np.asarray(index), self.dcell)
+        return asarray(index).dot(self.dcell)
 
     def index_fold(self, index, unique=True):
         """ Converts indices from *any* placement to only exist in the "primary" grid
@@ -1261,17 +1399,8 @@ class Grid(SuperCellChild):
         p.add_argument(*opts('--geometry', '-G'), action=SetGeometry,
                        help='Define the geometry attached to the Grid.')
 
-        # Define size of grid
-        class InterpGrid(argparse.Action):
-
-            def __call__(self, parser, ns, values, option_string=None):
-                ns._grid = ns._grid.interp([int(x) for x in values])
-        p.add_argument(*opts('--interp'), nargs=3, metavar=('NX', 'NY', 'NZ'),
-                       action=InterpGrid,
-                       help='Interpolate the grid.')
-
-        # substract another grid
-        # They *MUST* be conmensurate.
+        # subtract another grid
+        # They *MUST* be comensurate.
         class DiffGrid(argparse.Action):
 
             def __call__(self, parser, ns, value, option_string=None):
@@ -1373,6 +1502,16 @@ class Grid(SuperCellChild):
                        action=RemoveDirectionGrid,
                        help='Reduce the grid by removing a subset of the grid (along DIR).')
 
+        class Tile(argparse.Action):
+
+            def __call__(self, parser, ns, values, option_string=None):
+                r = int(values[0])
+                d = direction(values[1])
+                ns._grid = ns._grid.tile(r, d)
+        p.add_argument(*opts('--tile'), nargs=2, metavar=('TIMES', 'DIR'),
+                       action=Tile,
+                       help='Tiles the grid in the specified direction.')
+
         # Scale the grid with this value
         class ScaleGrid(argparse.Action):
 
@@ -1380,9 +1519,47 @@ class Grid(SuperCellChild):
                 ns._grid.grid *= value
         p.add_argument(*opts('--scale', '-S'), type=float,
                        action=ScaleGrid,
-                       help='Scale grid values.')
+                       help='Scale grid values with a factor')
 
         # Define size of grid
+        class InterpGrid(argparse.Action):
+
+            def __call__(self, parser, ns, values, option_string=None):
+                def _conv_shape(length, value):
+                    if "." in value:
+                        return int(round(length / float(value)))
+                    return int(value)
+                shape = list(map(_conv_shape, ns._grid.sc.length, values[:3]))
+                # shorten list for easier arguments
+                values = values[3:]
+                if len(values) > 0:
+                    values[0] = int(values[0])
+                ns._grid = ns._grid.interp(shape, *values)
+        p.add_argument(*opts('--interp'), nargs='*', metavar='NX NY NZ *ORDER *MODE',
+                       action=InterpGrid,
+                       help="""Interpolate grid for higher or lower density (minimum 3 arguments)
+Requires at least 3 arguments, number of points along 1st, 2nd and 3rd lattice vector. These may contain a "." to signal a distance in angstrom of each voxel.
+For instance --interp 0.1 10 100 will result in an interpolated shape of [nint(grid.sc.length / 0.1), 10, 100].
+
+The 4th optional argument is the order of interpolation; an integer 0<=i<=5 (default 1)
+The 5th optional argument is the mode to interpolate; wrap/mirror/constant/reflect/nearest
+""")
+
+        # Smoothen the grid
+        class SmoothGrid(argparse.Action):
+
+            def __call__(self, parser, ns, values, option_string=None):
+                if len(values) > 0:
+                    values[0] = float(values[0])
+                ns._grid = ns._grid.smooth(*values)
+        p.add_argument(*opts('--smooth'), nargs='*', metavar='*R *METHOD *MODE',
+                       action=SmoothGrid,
+                       help="""Smoothen grid values according to methods by applying a filter, all arguments are optional.
+The 1st argument is the radius of the filter for smoothening, a larger value means a larger volume which is agglomerated
+The 2nd argument is the method to use; gaussian/uniform
+The 3rd argument is the mode to use; wrap/mirror/constant/reflect/nearest
+""")
+
         class PrintInfo(argparse.Action):
 
             def __call__(self, parser, ns, values, option_string=None):
@@ -1392,6 +1569,39 @@ class Grid(SuperCellChild):
                        action=PrintInfo,
                        help='Print, to stdout, some regular information about the grid.')
 
+        class Plot(argparse.Action):
+            def __call__(self, parser, ns, values, option_string=None):
+                ns._stored_grid = True
+                import matplotlib.pyplot as plt
+
+                grid = ns._grid
+
+                axs = []
+                idx = []
+                for ax in (0, 1, 2):
+                    shape = grid.shape[ax]
+                    if shape > 1:
+                        axs.append(np.linspace(0, grid.sc.length[ax], shape, endpoint=False))
+                        idx.append(ax)
+
+                # Now plot data
+                if len(idx) == 3:
+                    raise ValueError("Cannot plot a 3D grid (yet!)")
+                elif len(idx) == 2:
+                    X, Y = np.meshgrid(*axs)
+                    plt.contourf(X, Y, np.squeeze(grid.grid).T)
+                    plt.xlabel(f"Distance along {'ABC'[idx[0]]} [Ang]")
+                    plt.ylabel(f"Distance along {'ABC'[idx[1]]} [Ang]")
+                elif len(idx) == 1:
+                    plt.plot(axs[0], grid.grid.ravel())
+                    plt.xlabel(f"Distance along {'ABC'[idx[0]]} [Ang]")
+                    plt.ylabel(f"Arbitrary unit")
+                plt.show()
+
+        p.add_argument(*opts('--plot', '-P'), nargs=0,
+                       action=Plot,
+                       help='Plot the grid (currently only enabled if at least one dimension has been averaged out')
+
         class Out(argparse.Action):
 
             def __call__(self, parser, ns, value, option_string=None):
@@ -1399,7 +1609,30 @@ class Grid(SuperCellChild):
                     return
                 if len(value) == 0:
                     return
-                ns._grid.write(value[0])
+                from sisl.io import get_sile
+                grid = ns._grid
+
+                # determine whether the write-out file has *write_grid* as a methd
+                # if not, and the grid only have 1 dimension, we allow it to be
+                # written to a datafile
+                sile = get_sile(value[0], 'w')
+                if hasattr(sile, "write_grid"):
+                    grid.write(sile)
+                elif np.prod(grid.shape) == np.amax(grid.shape):
+                    # this means that 2 dimensions have a length of 1
+                    # figure out which dimensions it is and add calculate
+                    # the distance along the lattice vector
+                    idx = np.argmax(grid.shape)
+
+                    dx = np.linspace(0, grid.sc.length[idx], grid.shape[idx], endpoint=False)
+                    sile.write_data(dx, grid.grid.ravel())
+                else:
+                    raise ValueError(f"""Either of these two cases are not fullfilled:
+
+1. {sile} do not have the `write_grid` method
+
+2. The grid is not 1D data; averaged or summed along 2 directions.""")
+
                 # Issue to the namespace that the grid has been written, at least once.
                 ns._stored_grid = True
         p.add_argument(*opts('--out', '-o'), nargs=1, action=Out,
@@ -1432,13 +1665,14 @@ def sgrid(grid=None, argv=None, ret_grid=False):
        whether the function should return the grid
     """
     import sys
-    import os.path as osp
     import argparse
+    from pathlib import Path
 
     from sisl.io import get_sile, BaseSile
 
     # The file *MUST* be the first argument
     # (except --help|-h)
+    exe = Path(sys.argv[0]).name
 
     # We cannot create a separate ArgumentParser to retrieve a positional arguments
     # as that will grab the first argument for an option!
@@ -1448,14 +1682,14 @@ def sgrid(grid=None, argv=None, ret_grid=False):
 This manipulation utility is highly advanced and one should note that the ORDER of
 options is determining the final structure. For instance:
 
-   $> {0} Reference.grid.nc --diff Other.grid.nc --sub 0.:0.2f z
+   {exe} Reference.grid.nc --diff Other.grid.nc --sub 0.:0.2f z
 
 is NOT equivalent to:
 
-   $> {0} Reference.grid.nc --sub 0.:0.2f z --diff Other.grid.nc
+   {exe} Reference.grid.nc --sub 0.:0.2f z --diff Other.grid.nc
 
 This may be unexpected but enables one to do advanced manipulations.
-    """.format(osp.basename(sys.argv[0]))
+    """
 
     if argv is not None:
         if len(argv) == 0:
@@ -1470,9 +1704,9 @@ This may be unexpected but enables one to do advanced manipulations.
     # Ensure that the arguments have pre-pended spaces
     argv = cmd.argv_negative_fix(argv)
 
-    p = argparse.ArgumentParser('Manipulates real-space grids.',
-                           formatter_class=argparse.RawDescriptionHelpFormatter,
-                           description=description)
+    p = argparse.ArgumentParser(exe,
+                                formatter_class=argparse.RawDescriptionHelpFormatter,
+                                description=description)
 
     # Add default sisl version stuff
     cmd.add_sisl_version_cite_arg(p)
