@@ -1,26 +1,16 @@
 """
-================================
-ScaleUp (:mod:`sisl.io.scaleup`)
-================================
-
-.. module:: sisl.io.scaleup
-   :noindex:
+ScaleUp
+=======
 
 The interaction between sisl and `ScaleUp`_ allows constructing large TB models
 to be post-processed in the NEGF code `TBtrans`_.
 
-.. autosummary::
-   :toctree:
-
    orboccSileScaleUp - orbital information
    refSileScaleUp - reference coordinates
    rhamSileScaleUp - Hamiltonian file
 
 """
 from .sile import *
-
 from .orbocc import *
 from .ref import *
 from .rham import *
-
-__all__ = [s for s in dir() if not s.startswith('_')]
