@@ -5,7 +5,7 @@ import scipy as sp
 from os.path import isfile
 import itertools as itools
 
-from ..sile import add_sile, sile_fh_open, sile_raise_write, SileError
+from ..sile import add_sile, get_sile_class, sile_fh_open, sile_raise_write, SileError
 from .sile import SileSiesta
 from .._help import *
 
@@ -28,7 +28,7 @@ from .siesta_nc import ncSileSiesta
 from .basis import ionxmlSileSiesta, ionncSileSiesta
 from .orb_indx import orbindxSileSiesta
 from .xv import xvSileSiesta
-from sisl import Geometry, Orbital, Atom, Atoms, SuperCell, DynamicalMatrix
+from sisl import Geometry, Orbital, Atom, AtomGhost, Atoms, SuperCell, DynamicalMatrix
 
 from sisl.utils.cmd import default_ArgumentParser, default_namespace
 from sisl.utils.misc import merge_instances
@@ -50,6 +50,21 @@ def _listify_str(arg):
     return arg
 
 
+def _track(method, msg):
+    if method.__self__.track:
+        info(f"{method.__self__.__class__.__name__}.{method.__name__}: {msg}")
+
+
+def _track_file(method, f, msg=None):
+    if msg is None:
+        if f.is_file():
+            msg = f"reading file {f}"
+        else:
+            msg = f"could not find file {f}"
+    if method.__self__.track:
+        info(f"{method.__self__.__class__.__name__}.{method.__name__}: {msg}")
+
+
 @set_module("sisl.io.siesta")
 class fdfSileSiesta(SileSiesta):
     """ FDF-input file
@@ -81,7 +96,7 @@ class fdfSileSiesta(SileSiesta):
         self._parent_fh = []
 
         # Public key for printing information about where stuff comes from
-        self.track = False
+        self.track = kwargs.get("track", False)
 
     def _pushfile(self, f):
         if self.dir_file(f).is_file():
@@ -416,12 +431,9 @@ class fdfSileSiesta(SileSiesta):
         def write(fh, value):
             if value is None:
                 return
-            if isinstance(value, str):
-                fh.write(self.print(key, value))
-                if '\n' not in value:
-                    fh.write('\n')
-            else:
-                raise NotImplementedError('Currently blocks are not implemented in set!')
+            fh.write(self.print(key, value))
+            if isinstance(value, str) and '\n' not in value:
+                fh.write('\n')
 
         # Now loop, write and edit
         do_write = True
@@ -436,6 +448,8 @@ class fdfSileSiesta(SileSiesta):
                     do_write = False
                 else:
                     fh.write(line)
+            if do_write:
+                write(fh, value)
 
     @staticmethod
     def print(key, value):
@@ -449,11 +463,13 @@ class fdfSileSiesta(SileSiesta):
                     has_nl = True
                     break
             if has_nl:
+                # copy list, we are going to change it
+                value = value[:]
                 # do not skip to next line in next segment
-                value[-1].replace('\n', '')
-                s += '\n{}'.format(''.join(value))
+                value[-1] = value[-1].replace('\n', '')
+                s += '\n{}\n'.format(''.join(value))
             else:
-                s += '\n{} {}'.format(value[0], '\n'.join(value[1:]))
+                s += '\n{}\n'.format('\n'.join(value))
             s += f'%endblock {key}'
         else:
             s = f'{key} {value}'
@@ -524,7 +540,10 @@ class fdfSileSiesta(SileSiesta):
         self._write(f'NumberOfSpecies {n_species}\n')
         self._write('%block ChemicalSpeciesLabel\n')
         for i, a in enumerate(geometry.atoms.atom):
-            self._write(' {} {} {}\n'.format(i + 1, a.Z, a.tag))
+            if isinstance(a, AtomGhost):
+                self._write(' {} {} {}\n'.format(i + 1, -a.Z, a.tag))
+            else:
+                self._write(' {} {} {}\n'.format(i + 1, a.Z, a.tag))
         self._write('%endblock ChemicalSpeciesLabel\n')
 
         _write_block = True
@@ -541,7 +560,7 @@ class fdfSileSiesta(SileSiesta):
             if n in geometry.names:
                 idx = list2str(geometry.names[n] + 1).replace('-', ' -- ')
                 if len(idx) > 200:
-                    info(str(self) + f'.write_geometry will not write the constraints for {n} (too long line).')
+                    info(f"{str(self)}.write_geometry will not write the constraints for {n} (too long line).")
                 else:
                     _write_block = write_block(idx, append, _write_block)
 
@@ -584,7 +603,7 @@ class fdfSileSiesta(SileSiesta):
             if Sa.no != a.no:
                 # Make sure the atom we replace with retains the same information
                 # *except* the number of orbitals.
-                a = Atom(a.Z, Sa.orbital, mass=a.mass, tag=a.tag)
+                a = a.__class__(a.Z, Sa.orbital, mass=a.mass, tag=a.tag)
             spgeom.geometry.atoms.replace(idx, a)
             spgeom.geometry.reduce()
         return no_no
@@ -600,8 +619,7 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             v = getattr(self, '_r_supercell_nsc_{}'.format(f.lower()))(*args, **kwargs)
             if v is not None:
-                if self.track:
-                    info(f"{self.file}(read_supercell_nsc) found in file={f}")
+                _track(self.read_supercell_nsc, f"found file {f}")
                 return v
         warn('number of supercells could not be read from output files. Assuming molecule cell '
              '(no supercell connections)')
@@ -609,12 +627,14 @@ class fdfSileSiesta(SileSiesta):
 
     def _r_supercell_nsc_nc(self, *args, **kwargs):
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_supercell_nsc_nc, f)
         if f.is_file():
             return ncSileSiesta(f).read_supercell_nsc()
         return None
 
     def _r_supercell_nsc_orb_indx(self, *args, **kwargs):
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.ORB_INDX')
+        _track_file(self._r_supercell_nsc_orb_indx, f)
         if f.is_file():
             return orbindxSileSiesta(f).read_supercell_nsc()
         return None
@@ -650,8 +670,7 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             v = getattr(self, '_r_supercell_{}'.format(f.lower()))(*args, **kwargs)
             if v is not None:
-                if self.track:
-                    info(f"{self.file}(read_supercell) found in file={f}")
+                _track(self.read_supercell, f"found file {f}")
                 return v
         return None
 
@@ -688,6 +707,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_supercell_nc(self):
         # Read supercell from <>.nc file
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_supercell_nc, f)
         if f.is_file():
             return ncSileSiesta(f).read_supercell()
         return None
@@ -695,6 +715,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_supercell_xv(self, *args, **kwargs):
         """ Returns `SuperCell` object from the FDF file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.XV')
+        _track_file(self._r_supercell_xv, f)
         if f.is_file():
             nsc = self.read_supercell_nsc()
             sc = xvSileSiesta(f).read_supercell()
@@ -702,6 +723,20 @@ class fdfSileSiesta(SileSiesta):
             return sc
         return None
 
+    def _r_supercell_tshs(self, *args, **kwargs):
+        f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSHS')
+        _track_file(self._r_supercell_tshs, f)
+        if f.is_file():
+            return tshsSileSiesta(f).read_supercell()
+        return None
+
+    def _r_supercell_onlys(self, *args, **kwargs):
+        f = self.dir_file(self.get('SystemLabel', default='siesta') + '.onlyS')
+        _track_file(self._r_supercell_onlys, f)
+        if f.is_file():
+            return onlysSileSiesta(f).read_supercell()
+        return None
+
     def read_force(self, *args, **kwargs):
         """ Read forces from the output of the calculation (forces are not defined in the input)
 
@@ -712,7 +747,7 @@ class fdfSileSiesta(SileSiesta):
 
         Returns
         -------
-        (*, 3) : vector with forces for each of the atoms
+        numpy.ndarray : vector with forces for each of the atoms, along each Cartesian direction
         """
         order = _listify_str(kwargs.pop('order', ['FA', 'nc']))
         for f in order:
@@ -737,6 +772,20 @@ class fdfSileSiesta(SileSiesta):
             return faSileSiesta(f).read_force()
         return None
 
+    def _r_force_tsfa(self, *args, **kwargs):
+        """ Read forces from the TSFA file """
+        f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSFA')
+        if f.is_file():
+            return faSileSiesta(f).read_force()
+        return None
+
+    def _r_force_tsfac(self, *args, **kwargs):
+        """ Read forces from the TSFAC file """
+        f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSFAC')
+        if f.is_file():
+            return faSileSiesta(f).read_force()
+        return None
+
     def _r_force_nc(self, *args, **kwargs):
         """ Read forces from the nc file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
@@ -750,7 +799,7 @@ class fdfSileSiesta(SileSiesta):
         Returns
         -------
         force_constant : numpy.ndarray
-            vector [*, 3, 2, *, 3]  with force constant element for each of the atomic displacements
+            vector ``[*, 3, 2, *, 3]``  with force constant element for each of the atomic displacements
         """
         order = _listify_str(kwargs.pop('order', ['nc', 'FC']))
         for f in order:
@@ -902,6 +951,7 @@ class fdfSileSiesta(SileSiesta):
         #  FC(NEW) = (n_displ, 3, na, 3)
         # In fact, after averaging this becomes the Hessian
         FC = FC.sum(axis=2) * 0.5
+        na_full = FC.shape[2]
         hermitian = kwargs.get("hermitian", True)
 
         # Figure out the "original" periodic directions
@@ -913,7 +963,7 @@ class fdfSileSiesta(SileSiesta):
 
         # Cut-off too small values
         fc_cut = kwargs.get('cutoff', 0.)
-        FC = np.where(np.abs(FC) > fc_cut, FC, 0.)
+        FC = np.where(np.fabs(FC) > fc_cut, FC, 0.)
 
         # Convert the force constant such that a diagonalization returns eV ^ 2
         # FC is in [eV / Ang^2]
@@ -924,17 +974,19 @@ class fdfSileSiesta(SileSiesta):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             for atom, _ in geom.atoms.iter(True):
-                new_atom = Atom(atom.Z, orbs, mass=atom.mass, tag=atom.tag)
+                new_atom = atom.__class__(atom.Z, orbs, mass=atom.mass, tag=atom.tag)
                 geom.atoms.replace(atom, new_atom)
 
         # Figure out the supercell indices
-        supercell = kwargs.get('supercell', [1, 1, 1])
+        # if the displaced atoms equals the length of the geometry
+        # it means we are not using a supercell.
+        supercell = kwargs.get('supercell', len(geom) != len(FC_atoms))
         if supercell is False:
             supercell = [1] * 3
         elif supercell is True:
             _, supercell = geom.as_primary(FC.shape[0], ret_super=True)
-            info(str(self) + ".read_dynamical_matrix(FC) guessed on a [{}, {}, {}] "
-                 "supercell calculation.".format(*supercell))
+            info("{}.read_dynamical_matrix(FC) guessed on a [{}, {}, {}] "
+                 "supercell calculation.".format(str(self), *supercell))
         # Convert to integer array
         supercell = _a.asarrayi(supercell)
 
@@ -987,13 +1039,12 @@ class fdfSileSiesta(SileSiesta):
             for _ in (supercell > 1).nonzero()[0]:
                 first_isc = (np.around(isc_xyz[FC_atoms + offset, :]) == 1.).sum(0)
                 axis_tiling.append(np.argmax(first_isc))
-                # Fix the offset
-                offset *= supercell[axis_tiling[-1]]
+                # Fix the offset and wrap-around
+                offset = (offset * supercell[axis_tiling[-1]]) % na_full
 
-            while len(axis_tiling) < 3:
-                for i in range(3):
-                    if not i in axis_tiling:
-                        axis_tiling.append(i)
+            for i in range(3):
+                if not i in axis_tiling:
+                    axis_tiling.append(i)
 
             # Now we have the tiling operation, check it sort of matches
             geom_tile = geom_small.copy()
@@ -1001,9 +1052,11 @@ class fdfSileSiesta(SileSiesta):
                 geom_tile = geom_tile.tile(supercell[axis], axis)
 
             # Proximity check of 0.01 Ang (TODO add this as an argument)
-            if not np.allclose(geom_tile.xyz, geom.xyz, rtol=0, atol=0.01):
-                raise SislError(f"{str(self)}.read_dynamical_matrix(FC) could "
-                                "not figure out the tiling method for the supercell")
+            for ax in range(3):
+                daxis = geom_tile.xyz[:, ax] - geom.xyz[:, ax]
+                if not np.allclose(daxis, daxis[0], rtol=0., atol=0.01):
+                    raise SislError(f"{str(self)}.read_dynamical_matrix(FC) could "
+                                    "not figure out the tiling method for the supercell")
 
             # Convert the FC matrix to a "rollable" matrix
             # This will make it easier to symmetrize
@@ -1014,12 +1067,28 @@ class fdfSileSiesta(SileSiesta):
             #  4. tile-axis_tiling[2]
             #  5. na
             #  6. x, y, z (force components)
-            FC.shape = (na_fc, 3, *supercell[axis_tiling], na_fc, 3)
+            # order of FC is reversed of the axis_tiling (because of contiguous arrays)
+            # so reverse
+            axis_tiling.reverse()
+            FC.shape = (na_fc, 3, *supercell[axis_tiling], -1, 3)
+
+            # now ensure we have the correct order of the supercell
+            # If the input supercell is
+            # [-2] [-1] [0] [1] [2]
+            # we need to convert it to
+            #  [0] [1] [2] [3] [4] [5]
+            isc_xyz.shape = (*supercell[axis_tiling], na_fc, 3)
+            for axis in axis_tiling:
+                nroll = isc_xyz[..., axis].min()
+                inroll = int(round(nroll))
+                if inroll != 0:
+                    # offset axis by 2 due to (na_fc, 3, ...)
+                    FC = np.roll(FC, inroll, axis=axis + 2)
+            FC_atoms -= FC_atoms.min()
 
             # Now swap the [2, 3, 4] dimensions so that we get in order of lattice vectors
-            swap = np.array([2, 3, 4])[axis_tiling]
-            swap = (0, 1, *swap, 5, 6)
-            FC = np.transpose(FC, swap)
+            #  x, y, z
+            FC = np.transpose(FC, (0, 1, *(axis_tiling.index(i)+2 for i in range(3)), 5, 6))
             del axis_tiling
             # Now FC is sorted according to the supercell tiling
 
@@ -1135,7 +1204,7 @@ class fdfSileSiesta(SileSiesta):
                         iter_j_FC_atoms = iter_FC_atoms[dist(ia, aoff + iter_FC_atoms) <= R]
 
                     for ja in iter_j_FC_atoms:
-                        D[ia*3:(ia+1)*3, joff+ja*3:joff+(ja+1)*3] = FC[ia, :, x, y, z, ja, :]
+                        D[ia*3:(ia+1)*3, joff+ja*3:joff+(ja+1)*3] += FC[ia, :, x, y, z, ja, :]
 
         D = D.tocsr()
         # Remove all zeros
@@ -1240,7 +1309,7 @@ class fdfSileSiesta(SileSiesta):
             s = Bohr2Ang
         elif 'scaledcartesian' in lc:
             # the same scaling as the lattice-vectors
-            s = self.get('LatticeConstant', 'Ang')
+            s = self.get('LatticeConstant', unit='Ang')
         elif 'fractional' in lc or 'scaledbylatticevectors' in lc:
             # no scaling of coordinates as that is entirely
             # done by the latticevectors
@@ -1293,7 +1362,7 @@ class fdfSileSiesta(SileSiesta):
         # Read the block (not strictly needed, if so we simply set all atoms to H)
         atoms = self.read_basis()
         if atoms is None:
-            warn(SileWarning('Block ChemicalSpeciesLabel does not exist, cannot determine the basis (all Hydrogen).'))
+            warn('Block ChemicalSpeciesLabel does not exist, cannot determine the basis (all Hydrogen).')
 
             # Default atom (hydrogen)
             atoms = Atom(1)
@@ -1367,16 +1436,20 @@ class fdfSileSiesta(SileSiesta):
             name = {'rho': 'Rho',
                     'rhoinit': 'RhoInit',
                     'vna': 'Vna',
+                    'ioch': 'Chlocal',
                     'chlocal': 'Chlocal',
-                    'rhotot': 'RhoTot',
+                    'toch': 'RhoTot',
                     'totalcharge': 'RhoTot',
+                    'rhotot': 'RhoTot',
+                    'drho': 'RhoDelta',
                     'deltarho': 'RhoDelta',
                     'rhodelta': 'RhoDelta',
-                    'electrostaticpotential': 'Vh',
                     'vh': 'Vh',
+                    'electrostaticpotential': 'Vh',
                     'rhoxc': 'RhoXC',
-                    'totalpotential': 'Vt',
                     'vt': 'Vt',
+                    'totalpotential': 'Vt',
+                    'bader': 'RhoBader',
                     'baderrho': 'RhoBader',
                     'rhobader': 'RhoBader'
             }.get(name.lower())
@@ -1388,16 +1461,20 @@ class fdfSileSiesta(SileSiesta):
         name = {'rho': 'Rho',
                 'rhoinit': 'RhoInit',
                 'vna': 'Vna',
+                'ioch': 'Chlocal',
                 'chlocal': 'Chlocal',
-                'rhotot': 'TotalCharge',
+                'toch': 'TotalCharge',
                 'totalcharge': 'TotalCharge',
+                'rhotot': 'TotalCharge',
+                'drho': 'DeltaRho',
                 'deltarho': 'DeltaRho',
                 'rhodelta': 'DeltaRho',
-                'electrostaticpotential': 'ElectrostaticPotential',
                 'vh': 'ElectrostaticPotential',
+                'electrostaticpotential': 'ElectrostaticPotential',
                 'rhoxc': 'RhoXC',
-                'totalpotential': 'TotalPotential',
                 'vt': 'TotalPotential',
+                'totalpotential': 'TotalPotential',
+                'bader': 'BaderCharge',
                 'baderrho': 'BaderCharge',
                 'rhobader': 'BaderCharge'
         }.get(name.lower()) + '.grid.nc'
@@ -1414,16 +1491,20 @@ class fdfSileSiesta(SileSiesta):
         name = {'rho': '.RHO',
                 'rhoinit': '.RHOINIT',
                 'vna': '.VNA',
+                'ioch': '.IOCH',
                 'chlocal': '.IOCH',
-                'rhotot': '.TOCH',
+                'toch': '.TOCH',
                 'totalcharge': '.TOCH',
+                'rhotot': '.TOCH',
+                'drho': '.DRHO',
                 'deltarho': '.DRHO',
                 'rhodelta': '.DRHO',
-                'electrostaticpotential': '.VH',
                 'vh': '.VH',
+                'electrostaticpotential': '.VH',
                 'rhoxc': '.RHOXC',
-                'totalpotential': '.VT',
                 'vt': '.VT',
+                'totalpotential': '.VT',
+                'bader': '.BADER',
                 'baderrho': '.BADER',
                 'rhobader': '.BADER'
         }.get(name.lower())
@@ -1497,8 +1578,8 @@ class fdfSileSiesta(SileSiesta):
                 found_all = False
 
         if found_one and not found_all:
-            warn(SileWarning('Siesta basis information could not read all ion.nc/ion.xml files. '
-                             'Only a subset of the basis information is accessible.'))
+            warn("Siesta basis information could not read all ion.nc/ion.xml files. "
+                 "Only a subset of the basis information is accessible.")
         elif not found_one:
             return None
         return atoms
@@ -1570,14 +1651,14 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             DM = getattr(self, '_r_density_matrix_{}'.format(f.lower()))(*args, **kwargs)
             if DM is not None:
-                if self.track:
-                    info(f"{self.file}(read_density_matrix) found in file={f}")
+                _track(self.read_density_matrix, f"found file {f}")
                 return DM
         return None
 
     def _r_density_matrix_nc(self, *args, **kwargs):
         """ Try and read the density matrix by reading the <>.nc """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_density_matrix_nc, f)
         DM = None
         if f.is_file():
             # this *should* also contain the overlap matrix
@@ -1587,6 +1668,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_density_matrix_tsde(self, *args, **kwargs):
         """ Read density matrix from the TSDE file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSDE')
+        _track_file(self._r_density_matrix_tsde, f)
         DM = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1599,6 +1681,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_density_matrix_dm(self, *args, **kwargs):
         """ Read density matrix from the DM file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.DM')
+        _track_file(self._r_density_matrix_dm, f)
         DM = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1624,14 +1707,14 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             EDM = getattr(self, '_r_energy_density_matrix_{}'.format(f.lower()))(*args, **kwargs)
             if EDM is not None:
-                if self.track:
-                    info(f"{self.file}(read_energy_density_matrix) found in file={f}")
+                _track(self.read_energy_density_matrix, f"found file {f}")
                 return EDM
         return None
 
     def _r_energy_density_matrix_nc(self, *args, **kwargs):
         """ Read energy density matrix by reading the <>.nc """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_energy_density_matrix_nc, f)
         if f.is_file():
             return ncSileSiesta(f).read_energy_density_matrix(*args, **kwargs)
         return None
@@ -1639,6 +1722,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_energy_density_matrix_tsde(self, *args, **kwargs):
         """ Read energy density matrix from the TSDE file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSDE')
+        _track_file(self._r_energy_density_matrix_tsde, f)
         EDM = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1664,14 +1748,14 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             v = getattr(self, '_r_overlap_{}'.format(f.lower()))(*args, **kwargs)
             if v is not None:
-                if self.track:
-                    info(f"{self.file}(read_overlap) found in file={f}")
+                _track(self.read_overlap, f"found file {f}")
                 return v
         return None
 
     def _r_overlap_nc(self, *args, **kwargs):
         """ Read overlap from the nc file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_overlap_nc, f)
         if f.is_file():
             return ncSileSiesta(f).read_overlap(*args, **kwargs)
         return None
@@ -1679,6 +1763,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_overlap_tshs(self, *args, **kwargs):
         """ Read overlap from the TSHS file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSHS')
+        _track_file(self._r_overlap_tshs, f)
         S = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1690,6 +1775,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_overlap_hsx(self, *args, **kwargs):
         """ Read overlap from the HSX file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.HSX')
+        _track_file(self._r_overlap_hsx, f)
         S = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1701,6 +1787,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_overlap_onlys(self, *args, **kwargs):
         """ Read overlap from the onlyS file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.onlyS')
+        _track_file(self._r_overlap_onlys, f)
         S = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1725,14 +1812,14 @@ class fdfSileSiesta(SileSiesta):
         for f in order:
             H = getattr(self, '_r_hamiltonian_{}'.format(f.lower()))(*args, **kwargs)
             if H is not None:
-                if self.track:
-                    info(f"{self.file}(read_hamiltonian) found in file={f}")
+                _track(self.read_hamiltonian, f"found file {f}")
                 return H
         return None
 
     def _r_hamiltonian_nc(self, *args, **kwargs):
         """ Read Hamiltonian from the nc file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.nc')
+        _track_file(self._r_hamiltonian_nc, f)
         if f.is_file():
             return ncSileSiesta(f).read_hamiltonian(*args, **kwargs)
         return None
@@ -1740,6 +1827,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_hamiltonian_tshs(self, *args, **kwargs):
         """ Read Hamiltonian from the TSHS file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.TSHS')
+        _track_file(self._r_hamiltonian_tshs, f)
         H = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1751,6 +1839,7 @@ class fdfSileSiesta(SileSiesta):
     def _r_hamiltonian_hsx(self, *args, **kwargs):
         """ Read Hamiltonian from the HSX file """
         f = self.dir_file(self.get('SystemLabel', default='siesta') + '.HSX')
+        _track_file(self._r_hamiltonian_hsx, f)
         H = None
         if f.is_file():
             if 'geometry' not in kwargs:
@@ -1759,7 +1848,7 @@ class fdfSileSiesta(SileSiesta):
             H = hsxSileSiesta(f).read_hamiltonian(*args, **kwargs)
             Ef = self.read_fermi_level()
             if Ef is None:
-                info(str(self) + '.read_hamiltonian from HSX file failed shifting to the Fermi-level.')
+                info(f"{str(self)}.read_hamiltonian from HSX file failed shifting to the Fermi-level.")
             else:
                 H.shift(-Ef)
         return H
