@@ -7,6 +7,7 @@ from ..sile import add_sile, sile_fh_open
 from sisl._internal import set_module
 import sisl._array as _a
 from sisl import Geometry, Atom, SuperCell
+from sisl.utils import PropertyDict
 from sisl.utils.cmd import *
 from sisl.unit.siesta import unit_convert
 
@@ -29,20 +30,26 @@ class outSileSiesta(SileSiesta):
 
     This enables reading the output quantities from the Siesta output.
     """
-    _job_completed = False
+    _completed = None
 
     def readline(self):
         line = super().readline()
         if 'Job completed' in line:
-            self._job_completed = True
+            self._completed = True
         return line
 
     readline.__doc__ = SileSiesta.readline.__doc__
 
-    @property
-    def job_completed(self):
+    @sile_fh_open()
+    def completed(self):
         """ True if the full file has been read and "Job completed" was found. """
-        return self._job_completed
+        if self._completed is None:
+            completed = self.step_to("Job completed")[0]
+        else:
+            completed = self._completed
+        if completed:
+            self._completed = True
+        return completed
 
     @sile_fh_open()
     def read_species(self):
@@ -71,13 +78,27 @@ class outSileSiesta(SileSiesta):
 
         return atom
 
+    @sile_fh_open(True)
+    def read_basis_block(self):
+        """ Reads the PAO.Basis block that Siesta writes """
+        found, line = self.step_to("%block PAO.Basis")
+        if not found:
+            raise ValueError(f"{self.__class__.__name__}.read_basis_block could not find PAO.Basis in output")
+
+        basis = []
+        while not line.startswith("%endblock PAO.Basis"):
+            line = self.readline()
+            basis.append(line)
+
+        return basis
+
     def _read_supercell_outcell(self):
         """ Wrapper for reading the unit-cell from the outcoor block """
 
         # Read until outcell is found
-        line = self.readline()
-        while not 'outcell: Unit cell vectors' in line:
-            line = self.readline()
+        found, line = self.step_to("outcell: Unit cell vectors")
+        if not found:
+            raise ValueError(f"{self.__class__.__name__}._r_supercell_outcell did not find outcell key")
 
         Ang = 'Ang' in line
 
@@ -405,7 +426,7 @@ class outSileSiesta(SileSiesta):
 
             if last:
                 return Ss[-1]
-            if self.job_completed and key == 'static':
+            if self.completed() and key == 'static':
                 return Ss[:-1]
             return Ss
 
@@ -414,27 +435,30 @@ class outSileSiesta(SileSiesta):
     @sile_fh_open()
     def read_moment(self, orbitals=False, quantity='S', last=True, all=False):
         """ Reads the moments from the Siesta output file
+
         These will only be present in case of spin-orbit coupling.
 
         Parameters
         ----------
-        orbitals: bool, False
+        orbitals: bool, optional
            return a table with orbitally resolved
            moments.
-        quantity: str, 'S'
+        quantity: {'S', 'L'}, optional
            return the spin-moments or the L moments
-        last: bool, True
+        last: bool, optional
            only read the last force
-        all: bool, False
+        all: bool, optional
            return a list of all forces (like an MD)
            If `True` `last` is ignored
         """
 
         # Read until outcoor is found
         itt = iter(self)
-        while not 'moments: Atomic' in next(itt):
-            if next(itt) == '':
-                return None
+        for line in itt:
+            if 'moments: Atomic' in line:
+                break
+        if not 'moments: Atomic' in line:
+            return None
 
         # The moments are printed in SPECIES list
         next(itt) # empty
@@ -490,6 +514,95 @@ class outSileSiesta(SileSiesta):
             return _a.arrayd(moments)
         return moments
 
+    @sile_fh_open()
+    def read_energy(self):
+        """ Reads the final energy distribution
+
+        Currently the energies translated are:
+
+        ``band``
+             band structure energy
+        ``kinetic``
+             electronic kinetic energy
+        ``hartree``
+             electronic electrostatic Hartree energy
+        ``dftu``
+             DFT+U energy
+        ``spin_orbit``
+             spin-orbit energy
+        ``extE``
+             external field energy
+        ``xc``
+             exchange-correlation energy
+        ``bulkV``
+             bulk-bias correction energy
+        ``total``
+             total energy
+        ``negf``
+             NEGF energy
+        ``fermi``
+             Fermi energy
+        ``ion.electron``
+             ion-electron interaction energy
+        ``ion.ion``
+             ion-ion interaction energy
+        ``ion.kinetic``
+             kinetic ion energy
+
+
+        Any unrecognized key gets added *as is*.
+
+        Examples
+        --------
+        >>> energies = sisl.get_sile("RUN.out").read_energy()
+        >>> ion_energies = energies.ion
+        >>> ion_energies.ion # ion-ion interaction energy
+        >>> ion_energies.kinetic # ion kinetic energy
+        >>> energies.fermi # fermi energy
+
+        Returns
+        -------
+        PropertyDict : dictionary like lookup table ionic energies are stored in a nested `PropertyDict` at the key ``ion`` (all energies in eV)
+        """
+        found = self.step_to("siesta: Final energy", reread=False)[0]
+        out = PropertyDict()
+        out.ion = PropertyDict()
+        if not found:
+            return out
+        itt = iter(self)
+
+        # Read data
+        line = next(itt)
+        name_conv = {
+            "Band Struct.": "band",
+            "Kinetic": "kinetic",
+            "Hartree": "hartree",
+            "Edftu": "dftu",
+            "Eldau": "dftu",
+            "Eso": "spin_orbit",
+            "Ext. field": "extE",
+            "Exch.-corr.": "xc",
+            "Ekinion": "ion.kinetic",
+            "Ion-electron": "ion.electron",
+            "Ion-ion": "ion.ion",
+            "Bulk bias": "bulkV",
+            "Total": "total",
+            "Fermi": "fermi",
+            "Enegf": "negf",
+        }
+        while len(line.strip()) > 0:
+            key, val = line.split("=")
+            key = key.split(":")[1].strip()
+            key = name_conv.get(key, key)
+            if key.startswith("ion."):
+                # sub-nest
+                out.ion[key[4:]] = float(val)
+            else:
+                out[key] = float(val)
+            line = next(itt)
+
+        return out
+
     def read_data(self, *args, **kwargs):
         """ Read specific content in the Siesta out file
 
@@ -513,12 +626,14 @@ class outSileSiesta(SileSiesta):
            read stress, args are passed to `read_stress`
         moment: bool, optional
            read moment, args are passed to `read_moment` (only for spin-orbit calculations)
+        energy: bool, optional
+           read final energies, args are passed to `read_energy`
         """
         run = []
         # This loops ensures that we preserve the order of arguments
         # From Py3.6 and onwards the **kwargs is an OrderedDictionary
         for kw in kwargs.keys():
-            if kw in ['geometry', 'force', 'moment', 'stress']:
+            if kw in ['geometry', 'force', 'moment', 'stress', 'energy']:
                 if kwargs[kw]:
                     run.append(kw)
 
@@ -573,12 +688,27 @@ class outSileSiesta(SileSiesta):
             elif line.startswith('SCF cycle continued'):
                 d['_final_iscf'] = 0
 
+        def common_parse(line, d):
+            if line.startswith('ts-Vha:'):
+                d['ts-Vha'] = float(line.split()[1])
+            elif line.startswith('bulk-bias: |v'):
+                d['bb-v'] = list(map(float, line.split()[-3:]))
+                if 'bb-vx' not in props:
+                    props.extend(['BB-vx', 'BB-vy', 'BB-vz'])
+            elif line.startswith('bulk-bias: {q'):
+                d['bb-q'] = list(map(float, line.split()[-3:]))
+                if 'bb-q+' not in props:
+                    props.extend(['BB-q+', 'BB-q-', 'BB-q0'])
+            else:
+                return False
+            return True
+
         if key.lower() == 'scf':
             def parse_next(line, d):
                 line = line.strip().replace('*', '0')
                 reset_d(d, line)
-                if line.startswith('ts-Vha:'):
-                    d['ts-Vha'] = float(line.split()[1])
+                if common_parse(line, d):
+                    pass
                 elif line.startswith('scf:'):
                     d['_found_iscf'] = True
                     if len(line) == 97:
@@ -602,8 +732,8 @@ class outSileSiesta(SileSiesta):
             def parse_next(line, d):
                 line = line.strip().replace('*', '0')
                 reset_d(d, line)
-                if line.startswith('ts-Vha:'):
-                    d['ts-Vha'] = float(line.split()[1])
+                if common_parse(line, d):
+                    pass
                 elif line.startswith('ts-q:'):
                     data = line.split()[1:]
                     try:
@@ -611,7 +741,7 @@ class outSileSiesta(SileSiesta):
                     except:
                         # We are probably reading a device list
                         # ensure that props are appended
-                        if props[-1] != data[-1]:
+                        if data[-1] not in props:
                             props.extend(data)
                         pass
                 elif line.startswith('ts-scf:'):
