@@ -9,7 +9,8 @@ from ..sile import add_sile, sile_fh_open, sile_raise_write
 from sisl._internal import set_module
 from sisl._array import aranged
 from sisl.unit.siesta import unit_convert
-from sisl import Geometry, Atom, Atoms, SuperCell, Grid, SphericalOrbital
+from sisl import Geometry, Atom, AtomGhost, Atoms, SuperCell, Grid, SphericalOrbital
+from sisl.sparse import _ncol_to_indptr
 from sisl.physics import SparseOrbitalBZ
 from sisl.physics import DensityMatrix, EnergyDensityMatrix
 from sisl.physics import DynamicalMatrix
@@ -116,7 +117,7 @@ class ncSileSiesta(SileCDFSiesta):
                 sorb = SphericalOrbital(l, (r * Bohr2Ang, psi), orb_q0[io])
 
                 # This will be -l:l (this is the way siesta does it)
-                orbital.extend(sorb.toAtomicOrbital(n=n, Z=z, P=P))
+                orbital.extend(sorb.toAtomicOrbital(n=n, zeta=z, P=P))
 
             # Get number of orbitals
             label = a.Label.strip()
@@ -174,7 +175,7 @@ class ncSileSiesta(SileCDFSiesta):
 
         C._csr.ncol = np.array(sp.variables['n_col'][:], np.int32)
         # Update maximum number of connections (in case future stuff happens)
-        C._csr.ptr = np.insert(np.cumsum(C._csr.ncol, dtype=np.int32), 0, 0)
+        C._csr.ptr = _ncol_to_indptr(C._csr.ncol)
         C._csr.col = np.array(sp.variables['list_col'][:], np.int32) - 1
 
         # Copy information over
@@ -208,7 +209,7 @@ class ncSileSiesta(SileCDFSiesta):
 
         C._csr.ncol = np.array(sp.variables['n_col'][:], np.int32)
         # Update maximum number of connections (in case future stuff happens)
-        C._csr.ptr = np.insert(np.cumsum(C._csr.ncol, dtype=np.int32), 0, 0)
+        C._csr.ptr = _ncol_to_indptr(C._csr.ncol)
         C._csr.col = np.array(sp.variables['list_col'][:], np.int32) - 1
 
         # Copy information over
@@ -231,7 +232,7 @@ class ncSileSiesta(SileCDFSiesta):
         sp = self.groups['SPARSE']
         S._csr._D[:, 0] = sp.variables['S'][:]
 
-        return S
+        return S.transpose(sort=kwargs.get("sort", True))
 
     def read_hamiltonian(self, **kwargs):
         """ Returns a Hamiltonian from the underlying NetCDF file """
@@ -251,7 +252,7 @@ class ncSileSiesta(SileCDFSiesta):
         Ef = - self._value('Ef')[:] * Ry2eV
         H.shift(Ef)
 
-        return H
+        return H.transpose(spin=False, sort=kwargs.get("sort", True))
 
     def read_dynamical_matrix(self, **kwargs):
         """ Returns a dynamical matrix from the underlying NetCDF file
@@ -266,7 +267,7 @@ class ncSileSiesta(SileCDFSiesta):
             raise SileError(self.__class__.__name__ + '.read_dynamical_matrix requires the stored matrix to be in Ry**2!')
         D._csr._D[:, 0] = sp.variables['H'][0, :] * Ry2eV ** 2
 
-        return D
+        return D.transpose(sort=kwargs.get("sort", True))
 
     def read_density_matrix(self, **kwargs):
         """ Returns a density matrix from the underlying NetCDF file """
@@ -280,7 +281,7 @@ class ncSileSiesta(SileCDFSiesta):
         # fix siesta specific notation
         _mat_spin_convert(DM)
 
-        return DM
+        return DM.transpose(spin=False, sort=kwargs.get("sort", True))
 
     def read_energy_density_matrix(self, **kwargs):
         """ Returns energy density matrix from the underlying NetCDF file """
@@ -300,7 +301,7 @@ class ncSileSiesta(SileCDFSiesta):
         # fix siesta specific notation
         _mat_spin_convert(EDM)
 
-        return EDM
+        return EDM.transpose(spin=False, sort=kwargs.get("sort", True))
 
     def read_force_constant(self):
         """ Reads the force-constant stored in the nc file
@@ -417,12 +418,15 @@ class ncSileSiesta(SileCDFSiesta):
             if a.tag in bs.groups:
                 # Assert the file sizes
                 if bs.groups[a.tag].Number_of_orbitals != a.no:
-                    raise ValueError('File {} has erroneous data '
-                                     'in regards of the already stored dimensions.'.format(self.file))
+                    raise ValueError(f'File {self.file} has erroneous data '
+                                     'in regards of the already stored dimensions.')
             else:
                 ba = bs.createGroup(a.tag)
                 ba.ID = np.int32(isp + 1)
-                ba.Atomic_number = np.int32(a.Z)
+                if isinstance(a, AtomGhost):
+                    ba.Atomic_number = -np.int32(a.Z)
+                else:
+                    ba.Atomic_number = np.int32(a.Z)
                 ba.Mass = a.mass
                 ba.Label = a.tag
                 ba.Element = a.symbol
@@ -539,7 +543,7 @@ class ncSileSiesta(SileCDFSiesta):
 
     def write_overlap(self, S, **kwargs):
         """ Write the overlap matrix to the NetCDF file """
-        csr = S._csr.copy()
+        csr = S.transpose(sort=False)._csr
         if csr.nnz == 0:
             raise SileError(str(self) + '.write_overlap cannot write a zero element sparse matrix!')
 
@@ -565,7 +569,7 @@ class ncSileSiesta(SileCDFSiesta):
         Ef : float, optional
            the Fermi level of the electronic structure (in eV), default to 0.
         """
-        csr = H._csr.copy()
+        csr = H.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
             raise SileError(str(self) + '.write_hamiltonian cannot write a zero element sparse matrix!')
 
@@ -613,7 +617,7 @@ class ncSileSiesta(SileCDFSiesta):
         DM : DensityMatrix
            the model to be saved in the NC file
         """
-        csr = DM._csr.copy()
+        csr = DM.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
             raise SileError(str(self) + '.write_density_matrix cannot write a zero element sparse matrix!')
 
@@ -660,7 +664,7 @@ class ncSileSiesta(SileCDFSiesta):
         EDM : EnergyDensityMatrix
            the model to be saved in the NC file
         """
-        csr = EDM._csr.copy()
+        csr = EDM.transpose(spin=False, sort=False)._csr
         if csr.nnz == 0:
             raise SileError(str(self) + '.write_energy_density_matrix cannot write a zero element sparse matrix!')
 
@@ -712,7 +716,7 @@ class ncSileSiesta(SileCDFSiesta):
         D : DynamicalMatrix
            the model to be saved in the NC file
         """
-        csr = D._csr.copy()
+        csr = D.transpose(sort=False)._csr
         if csr.nnz == 0:
             raise SileError(str(self) + '.write_dynamical_matrix cannot write a zero element sparse matrix!')
 
