@@ -16,7 +16,9 @@ __all__ = [
     'add_sile',
     'get_sile_class',
     'get_sile',
-    'get_siles']
+    'get_siles',
+    'get_sile_rules'
+]
 
 __all__ += [
     'BaseSile',
@@ -26,18 +28,20 @@ __all__ += [
     'SileError',
     'SileWarning',
     'SileInfo',
-    ]
+]
 
 # Decorators or sile-specific functions
-__all__ += ['sile_fh_open',
+__all__ += [
+    'sile_fh_open',
     'sile_raise_write',
-    'sile_raise_read']
+    'sile_raise_read'
+]
 
 # Global container of all Sile rules
 # This list of tuples is formed as
 #  [('fdf', fdfSileSiesta, fdfSileSiesta),
 #   ('fdf', ncSileSiesta, fdfSileSiesta)]
-# [0] is the file-endding
+# [0] is the file suffix
 # [1] is the base class that may be queried
 # [2] is the actual class the file represents
 # This enables one to add several files with the
@@ -54,10 +58,10 @@ class _sile_rule:
     def __init__(self, cls, suffix, case=True, gzip=False):
         self.cls = cls
         self.case = case
-        if not case:
-            self.suffix = suffix.lower()
-        else:
+        if case:
             self.suffix = suffix
+        else:
+            self.suffix = suffix.lower()
         self.gzip = gzip
         self.bases = self.build_bases()
         self.base_names = [c.__name__.lower() for c in self.bases]
@@ -318,6 +322,36 @@ def get_siles(attrs=None):
     return siles
 
 
+@set_module("sisl.io")
+def get_sile_rules(attrs=None, cls=None):
+    """
+    Retrieve all sile rules of siles with specific attributes or methods
+
+    Parameters
+    ----------
+    attrs : list of attribute names
+       limits the returned objects to those that have
+       the given attributes ``hasattr(sile, attrs)``, default ``[None]``
+    """
+    global __sile_rules
+
+    if attrs is None and cls is None:
+        return list(__sile_rules)
+
+    sile_rules = []
+    for rule in __sile_rules:
+        sile = rule.cls
+        if sile is cls:
+            sile_rules.append(rule)
+        elif attrs:
+            for attr in attrs:
+                if hasattr(sile, attr):
+                    sile_rules.append(rule)
+                    break
+
+    return sile_rules
+
+
 @set_module("sisl.io")
 class BaseSile:
     """ Base class for all sisl files """
@@ -332,11 +366,11 @@ class BaseSile:
         """ File of the current `Sile` """
         return basename(self._file)
 
-    def dir_file(self, filename=None):
+    def dir_file(self, filename=None, filename_base=""):
         """ File of the current `Sile` """
         if filename is None:
             filename = Path(self._file).name
-        return Path(self._directory) / filename
+        return Path(self._directory) / filename_base / filename
 
     def exist(self):
         """ Query whether the file exists """
@@ -783,7 +817,7 @@ class SileCDF(BaseSile):
 
     @staticmethod
     def _crt_grp(n, name):
-        if '/' in name:
+        if '/' in name: # this is NetCDF, so / is fixed as seperator!
             groups = name.split('/')
             grp = n
             for group in groups:
@@ -806,14 +840,15 @@ class SileCDF(BaseSile):
         if name in n.variables:
             return n.variables[name]
 
-        attr = None
-        if 'attr' in kwargs:
-            attr = kwargs.pop('attr')
-        v = n.createVariable(name, *args, **kwargs)
-        if attr is not None:
-            for name in attr:
-                setattr(v, name, attr[name])
-        return v
+        if 'attrs' in kwargs:
+            attrs = kwargs.pop('attrs')
+        else:
+            attrs = None
+        var = n.createVariable(name, *args, **kwargs)
+        if attrs is not None:
+            for name, value in attrs.items():
+                setattr(var, name, value)
+        return var
 
     @classmethod
     def isDimension(cls, obj):
@@ -940,21 +975,6 @@ class SileBin(BaseSile):
         return False
 
 
-@set_module("sisl.io")
-class SileError(IOError):
-    """ Define an error object related to the Sile objects """
-
-    def __init__(self, value, obj=None):
-        self.value = value
-        self.obj = obj
-
-    def __str__(self):
-        if self.obj:
-            return self.value + ' in ' + str(self.obj)
-        else:
-            return self.value
-
-
 def sile_raise_write(self, ok=('w', 'a')):
     is_ok = False
     for O in ok:
@@ -969,9 +989,23 @@ def sile_raise_read(self, ok=('r', 'a')):
     for O in ok:
         is_ok = is_ok or (O in self._mode)
     if not is_ok:
-        raise SileError('Reading file not possible; allowed '
-                        'modes={}, used mode={}'.format(
-                            ok, self._mode), self)
+        raise SileError(f"Reading file not possible; allowed "
+                        f"modes={ok}, used mode={self._mode}", self)
+
+
+@set_module("sisl.io")
+class SileError(IOError):
+    """ Define an error object related to the Sile objects """
+
+    def __init__(self, value, obj=None):
+        self.value = value
+        self.obj = obj
+
+    def __str__(self):
+        if self.obj:
+            return self.value + ' in ' + str(self.obj)
+        else:
+            return self.value
 
 
 @set_module("sisl.io")
