@@ -10,6 +10,7 @@ import sisl._array as _a
 # Import the geometry object
 from sisl import Geometry, Atom, SuperCell
 from sisl import SparseOrbitalBZSpin
+from sisl.sparse import _ncol_to_indptr
 from sisl.messages import warn
 from sisl.unit.siesta import unit_convert
 from ..siesta._help import _csr_to_siesta, _csr_from_sc_off, _mat_spin_convert
@@ -60,7 +61,7 @@ class deltancSileTBtrans(SileCDFTBtrans):
 
     def read_supercell(self):
         """ Returns the `SuperCell` object from this file """
-        cell = _a.arrayd(np.copy(self._value('cell')))
+        cell = _a.arrayd(np.copy(self._value('cell'))) * Bohr2Ang
         cell.shape = (3, 3)
 
         nsc = self._value('nsc')
@@ -77,7 +78,7 @@ class deltancSileTBtrans(SileCDFTBtrans):
         """ Returns the `Geometry` object from this file """
         sc = self.read_supercell()
 
-        xyz = _a.arrayd(np.copy(self._value('xa')))
+        xyz = _a.arrayd(np.copy(self._value('xa'))) * Bohr2Ang
         xyz.shape = (-1, 3)
 
         # Create list with correct number of orbitals
@@ -241,13 +242,13 @@ class deltancSileTBtrans(SileCDFTBtrans):
             if ilvl in [2, 4]:
                 self._crt_dim(lvl, 'nkpt', None)
                 self._crt_var(lvl, 'kpt', 'f8', ('nkpt', 'xyz'),
-                              attr = {'info': 'k-points for delta values',
-                                      'unit': 'b**-1'})
+                              attrs={'info': 'k-points for delta values',
+                                     'unit': 'b**-1'})
             if ilvl in [3, 4]:
                 self._crt_dim(lvl, 'ne', None)
                 self._crt_var(lvl, 'E', 'f8', ('ne',),
-                              attr = {'info': 'Energy points for delta values',
-                                      'unit': 'Ry'})
+                              attrs={'info': 'Energy points for delta values',
+                                     'unit': 'Ry'})
 
         return lvl
 
@@ -379,12 +380,12 @@ class deltancSileTBtrans(SileCDFTBtrans):
         if delta.dtype.kind == 'c':
             v1 = self._crt_var(lvl, 'Redelta', 'f8', dim,
                                chunksizes=csize,
-                               attr = {'info': "Real part of delta",
-                                       'unit': "Ry"}, **self._cmp_args)
+                               attrs={'info': "Real part of delta",
+                                     'unit': "Ry"}, **self._cmp_args)
             v2 = self._crt_var(lvl, 'Imdelta', 'f8', dim,
                                chunksizes=csize,
-                               attr = {'info': "Imaginary part of delta",
-                                       'unit': "Ry"}, **self._cmp_args)
+                               attrs={'info': "Imaginary part of delta",
+                                      'unit': "Ry"}, **self._cmp_args)
             for i in range(len(delta.spin)):
                 sl[-2] = i
                 v1[sl] = csr._D[:, i].real * eV2Ry
@@ -393,8 +394,8 @@ class deltancSileTBtrans(SileCDFTBtrans):
         else:
             v = self._crt_var(lvl, 'delta', 'f8', dim,
                               chunksizes=csize,
-                              attr = {'info': "delta",
-                                      'unit': "Ry"},  **self._cmp_args)
+                              attrs={'info': "delta",
+                                     'unit': "Ry"},  **self._cmp_args)
             for i in range(len(delta.spin)):
                 sl[-2] = i
                 v[sl] = csr._D[:, i] * eV2Ry
@@ -449,7 +450,7 @@ class deltancSileTBtrans(SileCDFTBtrans):
 
         C._csr.ncol = _a.arrayi(lvl.variables['n_col'][:])
         # Update maximum number of connections (in case future stuff happens)
-        C._csr.ptr = np.insert(_a.cumsumi(C._csr.ncol), 0, 0)
+        C._csr.ptr = _ncol_to_indptr(C._csr.ncol)
         C._csr.col = _a.arrayi(lvl.variables['list_col'][:]) - 1
 
         # Copy information over
