@@ -3,6 +3,7 @@ import numpy as np
 from .sile import SileVASP
 from ..sile import add_sile, sile_fh_open
 
+from sisl._array import arrayf
 from sisl._internal import set_module
 
 
@@ -13,6 +14,22 @@ __all__ = ['doscarSileVASP']
 class doscarSileVASP(SileVASP):
     """ Density of states output """
 
+    @sile_fh_open(True)
+    def read_fermi_level(self):
+        r""" Query the Fermi-level contained in the file
+
+        Returns
+        -------
+        Ef : fermi-level of the system
+        """
+        self.readline()  # NIONS, NIONS, JOBPAR_, WDES%INCDIJ
+        self.readline()  # AOMEGA, LATT_CUR%ANORM(1:3) *1e-10, POTIM * 1e-15
+        self.readline()  # TEMP
+        self.readline()  # ' CAR '
+        self.readline()  # name
+        line = self.readline().split()
+        return float(line[3])
+
     @sile_fh_open()
     def read_data(self):
         r""" Read DOS, as calculated and written by VASP
@@ -25,11 +42,11 @@ class doscarSileVASP(SileVASP):
             DOS points (in 1/eV)
         """
         # read first line
-        line = self.readline()  # NIONS, NIONS, JOBPAR_, WDES%INCDIJ
-        line = self.readline()  # AOMEGA, LATT_CUR%ANORM(1:3) *1e-10, POTIM * 1e-15
-        line = self.readline()  # TEMP
-        line = self.readline()  # ' CAR '
-        line = self.readline()  # name
+        self.readline()  # NIONS, NIONS, JOBPAR_, WDES%INCDIJ
+        self.readline()  # AOMEGA, LATT_CUR%ANORM(1:3) *1e-10, POTIM * 1e-15
+        self.readline()  # TEMP
+        self.readline()  # ' CAR '
+        self.readline()  # name
         line = self.readline().split()
         Emax = float(line[0])
         Emin = float(line[1])
@@ -38,13 +55,13 @@ class doscarSileVASP(SileVASP):
 
         E = np.empty([NE], np.float32)
         # Determine output
-        line = list(map(float, self.readline().split()))
+        line = arrayf(self.readline().split())
         ns = (len(line) - 1) // 2
         DOS = np.empty([ns, NE], np.float32)
         E[0] = line[0]
         DOS[:, 0] = line[1:ns+1]
         for ie in range(1, NE):
-            line = list(map(float, self.readline().split()))
+            line = arrayf(self.readline().split())
             E[ie] = line[0]
             DOS[:, ie] = line[1:ns+1]
         return E - Ef, DOS
