@@ -1,4 +1,3 @@
-# To check for integers
 from functools import partial
 from numbers import Integral
 from math import pi
@@ -14,6 +13,7 @@ from scipy.interpolate import UnivariateSpline
 from ._internal import set_module
 from . import _plot as plt
 from . import _array as _a
+from .messages import deprecate, deprecate_method
 from .shape import Sphere
 from .utils.mathematics import cart2spher
 
@@ -92,15 +92,6 @@ class Orbital:
     tag : str, optional
         user defined tag
 
-    Attributes
-    ----------
-    R : float
-        maximum radius (in Ang)
-    q0 : float
-        initial electronic charge
-    tag : str
-        user defined tag
-
     Examples
     --------
     >>> orb = Orbital(1)
@@ -111,22 +102,34 @@ class Orbital:
     >>> orbq.q0
     1.
     """
-    __slots__ = ['R', 'tag', 'q0']
+    __slots__ = ['_R', '_tag', '_q0']
 
     def __init__(self, R, q0=0., tag=''):
         """ Initialize orbital object """
-        if R < 0:
-            self.R = int(R)
-        else:
-            self.R = float(R)
-        self.q0 = float(q0)
-        self.tag = tag
+        self._R = float(R)
+        self._q0 = float(q0)
+        self._tag = tag
+
+    @property
+    def R(self):
+        """ Maxmimum radius of orbital """
+        return self._R
+
+    @property
+    def q0(self):
+        """ Initial charge """
+        return self._q0
+
+    @property
+    def tag(self):
+        """ Named tag of orbital """
+        return self._tag
 
     def __str__(self):
         """ A string representation of the object """
         if len(self.tag) > 0:
-            return self.__class__.__name__ + f'{{R: {self.R:.5f}, q0: {self.q0}, tag: {self.tag}}}'
-        return self.__class__.__name__ + f'{{R: {self.R:.5f}, q0: {self.q0}}}'
+            return f'{self.__class__.__name__}{{R: {self.R:.5f}, q0: {self.q0}, tag: {self.tag}}}'
+        return f'{self.__class__.__name__}{{R: {self.R:.5f}, q0: {self.q0}}}'
 
     def __repr__(self):
         if self.tag:
@@ -235,8 +238,8 @@ class Orbital:
 
             # Plot data
             cax = axes.contourf(theta, phi, s, *args, **kwargs)
+            cax.set_clim(s.min(), s.max())
             cab = axes.get_figure().colorbar(cax)
-            cab.set_clim(s.min(), s.max())
             axes.set_title(r'${}$'.format(self.name(True)))
             # I don't know how exactly to handle this...
             #axes.set_xlabel(r'Azimuthal angle $\theta$')
@@ -253,7 +256,7 @@ class Orbital:
 
         return axes
 
-    def toGrid(self, precision=0.05, c=1., R=None, dtype=np.float64, Z=1):
+    def toGrid(self, precision=0.05, c=1., R=None, dtype=np.float64, atom=1):
         """ Create a Grid with *only* this orbital wavefunction on it
 
         Parameters
@@ -266,13 +269,14 @@ class Orbital:
             box size of the grid (default to the orbital range)
         dtype : numpy.dtype, optional
             the used separation in the `Grid` between voxels
-        Z : int, optional
-            atomic number associated with the grid
+        atom : optional
+            atom associated with the grid; either an atom instance or
+            something that ``Atom(atom)`` would convert to a proper atom.
         """
         if R is None:
             R = self.R
         if R < 0:
-            raise ValueError(self.__class__.__name__ + ".toGrid was unable to create "
+            raise ValueError(f"{self.__class__.__name__}.toGrid was unable to create "
                              "the orbital grid for plotting, the box size is negative.")
 
         # Since all these things depend on other elements
@@ -283,7 +287,11 @@ class Orbital:
         from .atom import Atom
         from .physics.electron import wavefunction
         sc = SuperCell(R*2, origo=[-R] * 3)
-        g = Geometry([0] * 3, Atom(Z, self), sc=sc)
+        if isinstance(atom, Atom):
+            atom = atom.copy(orbitals=self)
+        else:
+            atom = Atom(atom, self)
+        g = Geometry([0] * 3, atom, sc=sc)
         G = Grid(precision, dtype=dtype, geometry=g)
         wavefunction(np.full(1, c), G, geometry=g)
         return G
@@ -333,16 +341,8 @@ class SphericalOrbital(Orbital):
 
     Attributes
     ----------
-    R : float
-        maximum radius (in Ang)
-    q0 : float
-        initial electronic charge
-    l : int
-        azimuthal quantum number
     f : func
         interpolation function that returns `f(r)` for a given radius
-    tag : str
-       user defined tag
 
     Examples
     --------
@@ -353,15 +353,19 @@ class SphericalOrbital(Orbital):
     True
     """
     # Additional slots (inherited classes retain the same slots)
-    __slots__ = ['l', 'f']
+    __slots__ = ['_l', 'f']
 
     def __init__(self, l, rf_or_func, q0=0., tag='', **kwargs):
         """ Initialize spherical orbital object """
-        self.l = l
+        self._l = l
 
         # Set the internal function
         if callable(rf_or_func):
             self.set_radial(rf_or_func, **kwargs)
+        elif rf_or_func is None:
+            # We don't do anything
+            self.f = NotImplemented
+            self._R = -1.
         else:
             # it must be two arguments
             self.set_radial(rf_or_func[0], rf_or_func[1], **kwargs)
@@ -371,6 +375,11 @@ class SphericalOrbital(Orbital):
         # maximum value in r.
         super().__init__(self.R, q0, tag)
 
+    @property
+    def l(self):
+        r""" :math:`l` quantum number """
+        return self._l
+
     def copy(self):
         """ Create an exact copy of this object """
         return self.__class__(self.l, self.f, self.q0, self.tag)
@@ -457,7 +466,7 @@ class SphericalOrbital(Orbital):
                 return R * 0.
             self.set_radial(f0)
             if 'R' in kwargs:
-                self.R = kwargs['R']
+                self._R = kwargs['R']
         elif len(args) == 1 and callable(args[0]):
             self.f = args[0]
             # Determine the maximum R
@@ -474,7 +483,7 @@ class SphericalOrbital(Orbital):
                 if idx < len(r) - 1:
                     idx += 1
                 # Preset R
-                self.R = r[idx]
+                self._R = r[idx]
                 # This should give us a precision of 0.0001 A
                 r = np.linspace(r[idx]-0.055+0.0001, r[idx]+0.055, 1100)
                 f = square(self.f(r))
@@ -484,13 +493,13 @@ class SphericalOrbital(Orbital):
                     idx = idx.max()
                     if idx < len(r) - 1:
                         idx += 1
-                    self.R = r[idx]
+                    self._R = r[idx]
 
             else:
                 # The orbital radius
                 # Is undefined, no values are above 0 in a range
                 # of 50 A
-                self.R = -1
+                self._R = -1
 
         elif len(args) > 1:
 
@@ -512,15 +521,15 @@ class SphericalOrbital(Orbital):
 
             self.set_radial(interp(r, f))
         elif 'R' in kwargs:
-            self.R = kwargs.get('R')
+            self._R = kwargs.get('R')
         else:
             raise ValueError('Arguments for set_radial are in-correct, please see the documentation of SphericalOrbital.set_radial')
 
     def __str__(self):
         """ A string representation of the object """
         if len(self.tag) > 0:
-            return self.__class__.__name__ + f'{{l: {self.l}, R: {self.R}, q0: {self.q0}, tag: {self.tag}}}'
-        return self.__class__.__name__ + f'{{l: {self.l}, R: {self.R}, q0: {self.q0}}}'
+            return f'{self.__class__.__name__}{{l: {self.l}, R: {self.R}, q0: {self.q0}, tag: {self.tag}}}'
+        return f'{self.__class__.__name__}{{l: {self.l}, R: {self.R}, q0: {self.q0}}}'
 
     def __repr__(self):
         if self.tag:
@@ -641,7 +650,7 @@ class SphericalOrbital(Orbital):
         """
         return self.f(r) * self.spher(theta, phi, m, cos_phi)
 
-    def toAtomicOrbital(self, m=None, n=None, Z=1, P=False, q0=None):
+    def toAtomicOrbital(self, m=None, n=None, zeta=1, P=False, q0=None):
         r""" Create a list of `AtomicOrbital` objects
 
         This defaults to create a list of `AtomicOrbital` objects for every `m` (for m in -l:l).
@@ -651,7 +660,7 @@ class SphericalOrbital(Orbital):
         ----------
         m : int or list or None
            if ``None`` it defaults to ``-l:l``, else only for the requested `m`
-        Z : int, optional
+        zeta : int, optional
            the specified zeta-shell
         P : bool, optional
            whether the orbitals are polarized.
@@ -669,8 +678,8 @@ class SphericalOrbital(Orbital):
         if m is None:
             m = range(-self.l, self.l + 1)
         elif isinstance(m, Integral):
-            return AtomicOrbital(n=n, l=self.l, m=m, Z=Z, P=P, spherical=self, q0=q0)
-        return [AtomicOrbital(n=n, l=self.l, m=mm, Z=Z, P=P, spherical=self, q0=q0) for mm in m]
+            return AtomicOrbital(n=n, l=self.l, m=m, zeta=zeta, P=P, spherical=self, q0=q0)
+        return [AtomicOrbital(n=n, l=self.l, m=mm, zeta=zeta, P=P, spherical=self, q0=q0) for mm in m]
 
     def __getstate__(self):
         """ Return the state of this object """
@@ -704,34 +713,13 @@ class AtomicOrbital(Orbital):
     tag : str, optional
         user defined tag
 
-    Attributes
-    ----------
-    R : float
-        maximum radius (in Ang)
-    n : int
-        principal quantum number
-    l : int
-        azimuthal quantum number
-    m : int
-        magnetic quantum number
-    Z : int
-        zeta shell
-    P : bool
-        whether this corresponds to a polarized shell (``True``)
-    f : func
-        interpolation function that returns `f(r)` for a given radius
-    q0 : float
-        initial electronic charge
-    tag : str
-        user defined tag
-
     Examples
     --------
     >>> r = np.linspace(0, 5, 50)
     >>> f = np.exp(-r)
-    >>> #                    n, l, m, [Z, [P]]
+    >>> #                    n, l, m, [zeta, [P]]
     >>> orb1 = AtomicOrbital(2, 1, 0, 1, (r, f))
-    >>> orb2 = AtomicOrbital(n=2, l=1, m=0, Z=1, (r, f))
+    >>> orb2 = AtomicOrbital(n=2, l=1, m=0, zeta=1, (r, f))
     >>> orb3 = AtomicOrbital('2pzZ', (r, f))
     >>> orb4 = AtomicOrbital('2pzZ1', (r, f))
     >>> orb5 = AtomicOrbital('pz', (r, f))
@@ -751,7 +739,7 @@ class AtomicOrbital(Orbital):
     #   P = polarization shell or not
     # orb is the SphericalOrbital class that retains the radial
     # grid and enables to calculate psi(r)
-    __slots__ = ['n', 'l', 'm', 'Z', 'P', 'orb']
+    __slots__ = ['_n', '_l', '_m', '_zeta', '_P', '_orb']
 
     def __init__(self, *args, **kwargs):
         """ Initialize atomic orbital object """
@@ -759,13 +747,15 @@ class AtomicOrbital(Orbital):
 
         # Ensure args is a list (to be able to pop)
         args = list(args)
-        self.orb = None
+        self._orb = None
 
         # Extract shell information
         n = kwargs.get('n', None)
         l = kwargs.get('l', None)
         m = kwargs.get('m', None)
-        Z = kwargs.get('Z', 1)
+        if 'Z' in kwargs:
+            deprecate(f"{self.__class__.__name__}(Z=) is deprecated, please use (zeta=) instead")
+        zeta = kwargs.get('zeta', kwargs.get('Z', 1))
         P = kwargs.get('P', False)
 
         if len(args) > 0:
@@ -813,11 +803,11 @@ class AtomicOrbital(Orbital):
                     # However, for now we assume this is enough (could easily
                     # be extended by a reg-exp)
                     try:
-                        Z = int(s[iZ+1])
+                        zeta = int(s[iZ+1])
                         # Remove Z + int
                         s = s[:iZ] + s[iZ+2:]
                     except:
-                        Z = 1
+                        zeta = 1
                         s = s[:iZ] + s[iZ+1:]
 
                 # We should be left with m specification
@@ -832,13 +822,13 @@ class AtomicOrbital(Orbital):
                 # Figure out if it is a sphericalorbital
                 if len(args) > 0:
                     if isinstance(args[0], SphericalOrbital):
-                        self.orb = args.pop(0)
+                        self._orb = args.pop(0)
                     else:
-                        self.orb = SphericalOrbital(l, args.pop(0))
+                        self._orb = SphericalOrbital(l, args.pop(0))
             else:
 
                 # Arguments *have* to be
-                # n, l, [m (only for l>0)] [, Z [, P]]
+                # n, l, [m (only for l>0)] [, zeta [, P]]
                 if n is None and len(args) > 0:
                     n = args.pop(0)
                 if l is None and len(args) > 0:
@@ -853,7 +843,7 @@ class AtomicOrbital(Orbital):
                 # information or radial functions
                 if len(args) > 0:
                     if isinstance(args[0], Integral):
-                        Z = args.pop(0)
+                        zeta = args.pop(0)
                 if len(args) > 0:
                     if isinstance(args[0], bool):
                         P = args.pop(0)
@@ -861,25 +851,25 @@ class AtomicOrbital(Orbital):
                 # Figure out if it is a sphericalorbital
                 if len(args) > 0:
                     if isinstance(args[0], SphericalOrbital):
-                        self.orb = args.pop(0)
+                        self._orb = args.pop(0)
                     else:
-                        self.orb = SphericalOrbital(l, args.pop(0))
+                        self._orb = SphericalOrbital(l, args.pop(0))
 
         # Still if n is None, we assign the default (lowest) quantum number
         if n is None:
             n = l + 1
 
         # Copy over information
-        self.n = n
-        self.l = l
-        self.m = m
-        self.Z = Z
-        self.P = P
+        self._n = n
+        self._l = l
+        self._m = m
+        self._zeta = zeta
+        self._P = P
 
         if self.l > 4:
-            raise ValueError(self.__class__.__name__ + ' does not implement shell h and above!')
+            raise ValueError(f'{self.__class__.__name__} does not implement shell h and above!')
         if abs(self.m) > self.l:
-            raise ValueError(self.__class__.__name__ + ' requires |m| <= l.')
+            raise ValueError(f'{self.__class__.__name__} requires |m| <= l.')
 
         # Retrieve user-passed spherical orbital
         s = kwargs.get('spherical', None)
@@ -888,24 +878,61 @@ class AtomicOrbital(Orbital):
             # Expect the orbital to already be set
             pass
         elif isinstance(s, Orbital):
-            self.orb = s
+            self._orb = s
         else:
-            self.orb = SphericalOrbital(l, s)
+            self._orb = SphericalOrbital(l, s)
 
-        if self.orb is None:
+        if self._orb is None:
             # Default orbital to none, this will not create any radial functions
             # But any use of the orbital will still work
-            self.orb = Orbital(self.R)
+            self._orb = Orbital(self.R)
 
-        self.R = self.orb.R
+        self._R = self._orb.R
+
+    @property
+    def n(self):
+        r""" :math:`n` shell """
+        return self._n
+
+    @property
+    def l(self):
+        r""" :math:`l` quantum number """
+        return self._l
+
+    @property
+    def m(self):
+        r""" :math:`m` quantum number """
+        return self._m
+
+    @property
+    def zeta(self):
+        r""" :math:`\zeta` shell """
+        return self._zeta
+
+    @property
+    @deprecate_method("AtomicOrbital.Z is deprecated, please use .zeta instead")
+    def Z(self):
+        r""" :math:`\zeta` shell """
+        return self._zeta
+
+    @property
+    def P(self):
+        r""" Whether this is polarized shell or not """
+        return self._P
+
+    @property
+    def orb(self):
+        r""" Orbital with radial part """
+        return self._orb
 
     @property
     def f(self):
+        r""" Radial function """
         return self.orb.f
 
     def copy(self):
         """ Create an exact copy of this object """
-        return self.__class__(n=self.n, l=self.l, m=self.m, Z=self.Z, P=self.P, spherical=self.orb.copy(), q0=self.q0, tag=self.tag)
+        return self.__class__(n=self.n, l=self.l, m=self.m, zeta=self.zeta, P=self.P, spherical=self.orb.copy(), q0=self.q0, tag=self.tag)
 
     def equal(self, other, psi=False, radial=False):
         """ Compare two orbitals by comparing their radius, and possibly the radial and psi functions
@@ -924,7 +951,7 @@ class AtomicOrbital(Orbital):
             same &= self.n == other.n
             same &= self.l == other.l
             same &= self.m == other.m
-            same &= self.Z == other.Z
+            same &= self.zeta == other.zeta
             same &= self.P == other.P
         elif isinstance(other, Orbital):
             same = self.orb.equal(other)
@@ -935,40 +962,38 @@ class AtomicOrbital(Orbital):
     def name(self, tex=False):
         """ Return named specification of the atomic orbital """
         if tex:
-            name = '{}{}'.format(self.n, {0: 's', 1: 'p', 2: 'd', 3: 'f', 4: 'g'}.get(self.l))
+            name = '{}{}'.format(self.n, 'spdfg'[self.l])
             if self.l == 1:
-                name += {0: '_z', 1: '_x', -1: '_y'}.get(self.m)
+                name += ['_y', '_z', '_x'][self.m+1]
             elif self.l == 2:
-                name += {-2: '_{xy}', -1: '_{yz}', 0: '_{z^2}', 1: '_{xz}', 2: '_{x^2-y^2}'}.get(self.m)
+                name += ['_{xy}', '_{yz}', '_{z^2}', '_{xz}', '_{x^2-y^2}'][self.m+2]
             elif self.l == 3:
-                name += {-3: '_{y(3x^2-y^2)}', -2: '_{xyz}', -1: '_{z^2y}', 0: '_{z^3}',
-                         1: '_{z^2x}', 2: '_{z(x^2-y^2)}', 3: '_{x(x^2-3y^2)}'}.get(self.m)
+                name += ['_{y(3x^2-y^2)}', '_{xyz}', '_{z^2y}', '_{z^3}', '_{z^2x}', '_{z(x^2-y^2)}', '_{x(x^2-3y^2)}'][self.m+3]
             elif self.l == 4:
-                name += {-4: '_{_{xy(x^2-y^2)}}', -3: '_{zy(3x^2-y^2)}', -2: '_{z^2xy}', -1: '_{z^3y}', 0: '_{z^4}',
-                         1: '_{z^3x}', 2: '_{z^2(x^2-y^2)}', 3: '_{zx(x^2-3y^2)}', 4: '_{x^4+y^4}'}.get(self.m)
+                name += ['_{_{xy(x^2-y^2)}}', '_{zy(3x^2-y^2)}', '_{z^2xy}', '_{z^3y}', '_{z^4}',
+                         '_{z^3x}', '_{z^2(x^2-y^2)}', '_{zx(x^2-3y^2)}', '_{x^4+y^4}'][self.m+4]
             if self.P:
-                return name + fr'\zeta^{self.Z}\mathrm{{P}}'
-            return name + fr'\zeta^{self.Z}'
-        name = '{}{}'.format(self.n, {0: 's', 1: 'p', 2: 'd', 3: 'f', 4: 'g'}.get(self.l))
+                return name + fr'\zeta^{self.zeta}\mathrm{{P}}'
+            return name + fr'\zeta^{self.zeta}'
+        name = '{}{}'.format(self.n, 'spdfg'[self.l])
         if self.l == 1:
-            name += {0: 'z', 1: 'x', -1: 'y'}.get(self.m)
+            name += ['y', 'z', 'x'][self.m+1]
         elif self.l == 2:
-            name += {-2: 'xy', -1: 'yz', 0: 'z2', 1: 'xz', 2: 'x2-y2'}.get(self.m)
+            name += ['xy', 'yz', 'z2', 'xz', 'x2-y2'][self.m+2]
         elif self.l == 3:
-            name += {-3: 'y(3x2-y2)', -2: 'xyz', -1: 'z2y', 0: 'z3',
-                     1: 'z2x', 2: 'z(x2-y2)', 3: 'x(x2-3y2)'}.get(self.m)
+            name += ['y(3x2-y2)', 'xyz', 'z2y', 'z3', 'z2x', 'z(x2-y2)', 'x(x2-3y2)'][self.m+3]
         elif self.l == 4:
-            name += {-4: 'xy(x2-y2)', -3: 'zy(3x2-y2)', -2: 'z2xy', -1: 'z3y', 0: 'z4',
-                     1: 'z3x', 2: 'z2(x2-y2)', 3: 'zx(x2-3y2)', 4: 'x4+y4'}.get(self.m)
+            name += ['xy(x2-y2)', 'zy(3x2-y2)', 'z2xy', 'z3y', 'z4',
+                     'z3x', 'z2(x2-y2)', 'zx(x2-3y2)', 'x4+y4'][self.m+4]
         if self.P:
-            return name + f'Z{self.Z}P'
-        return name + f'Z{self.Z}'
+            return name + f'Z{self.zeta}P'
+        return name + f'Z{self.zeta}'
 
     def __str__(self):
         """ A string representation of the object """
         if len(self.tag) > 0:
-            return self.__class__.__name__ + '{{{0}, q0: {1}, tag: {2}, {3}}}'.format(self.name(), self.q0, self.tag, str(self.orb))
-        return self.__class__.__name__ + '{{{0}, q0: {1}, {2}}}'.format(self.name(), self.q0, str(self.orb))
+            return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, tag: {self.tag}, {str(self.orb)}}}'
+        return f'{self.__class__.__name__}{{{self.name()}, q0: {self.q0}, {str(self.orb)}}}'
 
     def __repr__(self):
         if self.tag:
@@ -981,7 +1006,7 @@ class AtomicOrbital(Orbital):
         See `SphericalOrbital.set_radial` where these arguments are passed to.
         """
         self.orb.set_radial(*args)
-        self.R = self.orb.R
+        self._R = self.orb.R
 
     def radial(self, r, is_radius=True):
         r""" Calculate the radial part of the wavefunction :math:`f(\mathbf R)`
