@@ -1,9 +1,6 @@
 """Brillouin zone classes
 =========================
 
-.. module:: sisl.physics.brillouinzone
-   :noindex:
-
 The Brillouin zone objects are all special classes enabling easy manipulation
 of an underlying physical quantity.
 
@@ -26,7 +23,7 @@ This may be extremely convenient when calculating band-structures:
 
 >>> H = Hamiltonian(...)
 >>> bs = BandStructure(H, [[0, 0, 0], [0.5, 0, 0]], 100)
->>> bs_eig = bs.apply.array.eigh().T
+>>> bs_eig = bs.apply.array.eigh()
 >>> plt.plot(bs.lineark(), bs_eig)
 
 and then you have all eigenvalues for all the k-points along the path.
@@ -81,8 +78,54 @@ to use `~sisl.oplist` to handle cases of `BrillouinZone.apply.average` and `Bril
 
 Which does mathematical operations (averaging/summing) using `~sisl.oplist`.
 
-.. autosummary::
-   :toctree:
+
+Parallel calculations
+---------------------
+
+The ``apply`` method looping k-points may be explicitly parallelized.
+To run parallel do:
+
+>>> H = Hamiltonian(...)
+>>> mp = MonkhorstPack(H, [10, 10, 10])
+>>> with mp.apply.renew(pool=True) as par:
+...     par.eigh()
+
+This requires you also have the package ``pathos`` available.
+The above will run in parallel using a default number of processors
+in priority:
+
+1. Environment variable ``SISL_NPROCS``
+2. Return value of ``os.cpu_count()``.
+
+Note that this may interfere with BLAS implementation which defaults
+to use all CPU's for threading. The total processors/threads that will
+be created is ``SISL_NPROCS * OMP_NUM_THREADS``. Try and ensure this is below
+or equal to the actual core-count of your machine (or the number of requested
+cores in a HPC environment).
+
+
+Alternatively one can control the number of processors locally by doing:
+
+>>> H = Hamiltonian(...)
+>>> mp = MonkhorstPack(H, [10, 10, 10])
+>>> with mp.apply.renew(pool=2) as par:
+...     par.eigh()
+
+which will request 2 processors (regardless of core-count).
+As a last resort you can pass your own ``Pool`` of workers that
+will be used for the parallel processing.
+
+>>> from multiprocessing import Pool
+>>> pool = Pool(4)
+>>> H = Hamiltonian(...)
+>>> mp = MonkhorstPack(H, [10, 10, 10])
+>>> with mp.apply.renew(pool=pool) as par:
+...     par.eigh()
+
+The ``Pool`` should implement some standard methods that are
+existing in the ``pathos`` enviroment such as ``Pool.restart`` and ``Pool.terminate``
+and ``imap`` and ``uimap`` methods. See the ``pathos`` documentation for detalis.
+
 
    BrillouinZone
    MonkhorstPack
@@ -92,15 +135,12 @@ Which does mathematical operations (averaging/summing) using `~sisl.oplist`.
 
 import types
 from numbers import Integral, Real
-from functools import wraps, reduce
-import operator as op
 import warnings
 
 from numpy import pi
 import numpy as np
 from numpy import sum, dot, cross, argsort
 
-from sisl._dispatcher import ClassDispatcher, AbstractDispatch
 from sisl._internal import set_module
 from sisl.oplist import oplist
 from sisl.unit import units
@@ -119,22 +159,7 @@ except ImportError:
     _has_xarray = False
 
 
-__all__ = ['BrillouinZone', 'MonkhorstPack', 'BandStructure']
-__all__ += ["BrillouinZoneApply", "BrillouinZoneParentApply"]
-
-
-def _asoplist(arg):
-    if isinstance(arg, tuple):
-        return oplist(arg)
-    elif isinstance(arg, list) and not isinstance(arg, oplist):
-        return oplist(arg)
-    return arg
-
-
-def _apply_str(s):
-    def __str__(self):
-        return f"Apply{{{s}}}"
-    return __str__
+__all__ = ["BrillouinZone", "MonkhorstPack", "BandStructure"]
 
 
 @set_module("sisl.physics")
@@ -180,13 +205,49 @@ class BrillouinZone:
             else:
                 self._w = _a.arrayd(weight).ravel()
         if len(self.k) != len(self.weight):
-            raise ValueError(self.__class__.__name__ + '.__init__ requires input k-points and weights to be of equal length.')
+            raise ValueError(f'{self.__class__.__name__}.__init__ requires input k-points and weights to be of equal length.')
 
         # Instantiate the array call
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore')
             self.asarray()
 
+    def apply(self, **attrs):
+        r""" Loop over all k-points by applying `parent` methods for all k.
+
+        This allows great potential for running and collecting various computationally
+        heavy methods from a single point on all k-points.
+
+        The `apply` method will *dispatch* the parent methods through all k-points
+        and passing `k` as arguments to the parent methods in a straight-forward manner.
+
+        For instance to iterate over all eigenvalues of a Hamiltonian
+
+        >>> H = Hamiltonian(...)
+        >>> bz = BrillouinZone(H)
+        >>> for ik, eigh in enumerate(bz.apply.eigh()):
+        ...    # do something with eigh which corresponds to bz.k[ik]
+
+        By default the `apply` method exposes a set of dispatch methods:
+
+        - `apply.iter`, the default iterator module
+        - `apply.average` reduced result by averaging (using `BrillouinZone.weight`
+           as the weight per k-point.
+        - `apply.sum` reduced result without weighing
+        - `apply.array` return a single array with all values; has `len` equal to
+           number of k-points
+        - `apply.none`, specialized method that is mainly useful when wrapping
+           methods
+        - `apply.list` same as `apply.array` but using Python list as return value
+        - `apply.oplist` using `sisl.oplist` allows greater flexibility for mathematical
+           operations element wise
+        - `apply.datarray` if `xarray` is available one can retrieve an `xarray.DataArray`
+           instance
+
+        Please see `sisl.physics.brillouinzone` for further examples.
+        """
+        pass
+
     def set_parent(self, parent):
         """ Update the parent associated to this object
 
@@ -356,9 +417,17 @@ class BrillouinZone:
 
         return BrillouinZone(sc, k, w)
 
-    def copy(self):
-        """ Create a copy of this object """
-        bz = self.__class__(self.parent, self._k, self.weight)
+    def copy(self, parent=None):
+        """ Create a copy of this object, optionally changing the parent
+
+        Parameters
+        ----------
+        parent : optional
+           change the parent
+        """
+        if parent is None:
+            parent = self.parent
+        bz = self.__class__(parent, self._k, self.weight)
         bz._k = self._k.copy()
         bz._w = self._w.copy()
         return bz
@@ -539,7 +608,7 @@ class BrillouinZone:
             has_wrap = 'wrap' in kwargs
             if has_wrap:
                 wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.asarray',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asarray',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -605,7 +674,7 @@ class BrillouinZone:
         def asnone(self, *args, **kwargs):
             func = self._bz_get_func()
             wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap', lambda x: x))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.asnone',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asnone',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -669,7 +738,7 @@ class BrillouinZone:
                 has_wrap = 'wrap' in kwargs
                 if has_wrap:
                     wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-                eta = tqdm_eta(len(self), self.__class__.__name__ + '.asarray',
+                eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asarray',
                                'k', kwargs.pop('eta', False))
                 parent = self.parent
                 k = self.k
@@ -755,7 +824,7 @@ class BrillouinZone:
             has_wrap = 'wrap' in kwargs
             if has_wrap:
                 wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.aslist',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.aslist',
                            'k', kwargs.pop('eta', False))
             a = [None] * len(self)
             parent = self.parent
@@ -813,7 +882,7 @@ class BrillouinZone:
             has_wrap = 'wrap' in kwargs
             if has_wrap:
                 wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.asyield',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asyield',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -880,7 +949,7 @@ class BrillouinZone:
             has_wrap = 'wrap' in kwargs
             if has_wrap:
                 wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.asaverage',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asaverage',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -956,7 +1025,7 @@ class BrillouinZone:
             has_wrap = 'wrap' in kwargs
             if has_wrap:
                 wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap'))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.assum',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.assum',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -1059,226 +1128,6 @@ class BrillouinZone:
                 fh.write_data(kw.T, *args, **kwargs)
 
 
-@set_module("sisl.physics")
-class BrillouinZoneApply(AbstractDispatch):
-    # this dispatch function will do stuff on the BrillouinZone object
-    pass
-
-
-@set_module("sisl.physics")
-class BrillouinZoneParentApply(BrillouinZoneApply):
-
-    __str__ = _apply_str("parent apply over k")
-
-    def _parse_kwargs(self, wrap, eta, eta_key):
-        """ Parse kwargs """
-        bz = self._obj
-        parent = bz.parent
-        if wrap is None:
-            # we always return a wrap
-            def wrap(v, parent=None, k=None, weight=None):
-                return v
-        else:
-            wrap = allow_kwargs("parent", "k", "weight")(wrap)
-        eta = tqdm_eta(len(bz), f"{bz.__class__.__name__}.{eta_key}", "k", eta)
-        return bz, parent, wrap, eta
-
-    def __getattr__(self, key):
-        # We need to offload the dispatcher to retrieve
-        # methods from the parent object
-        # This dispatch will _never_ do anything to the BrillouinZone
-        method = getattr(self._obj.parent, key)
-        return self.dispatch(method)
-
-
-@set_module("sisl.physics")
-class IteratorApply(BrillouinZoneParentApply):
-    __str__ = _apply_str("iter")
-
-    def dispatch(self, method, eta_key="iter"):
-        """ Dispatch the method by iterating values """
-        @wraps(method)
-        def func(*args, wrap=None, eta=False, **kwargs):
-            bz, parent, wrap, eta = self._parse_kwargs(wrap, eta, eta_key=eta_key)
-            k = bz.k
-            w = bz.weight
-            for i in range(len(k)):
-                yield wrap(method(*args, k=k[i], **kwargs), parent=parent, k=k[i], weight=w[i])
-                eta.update()
-            eta.close()
-
-        return func
-
-
-@set_module("sisl.physics")
-class SumApply(IteratorApply):
-    __str__ = _apply_str("sum over k")
-
-    def dispatch(self, method):
-        """ Dispatch the method by summing """
-        iter_func = super().dispatch(method, eta_key="sum")
-
-        @wraps(method)
-        def func(*args, **kwargs):
-            it = iter_func(*args, **kwargs)
-            # next will be called before anything else
-            return reduce(op.add, it, _asoplist(next(it)))
-
-        return func
-
-
-@set_module("sisl.physics")
-class NoneApply(IteratorApply):
-    __str__ = _apply_str("return None")
-
-    def dispatch(self, method):
-        """ Dispatch the method by doing nothing (mostly useful if wrapped) """
-        iter_func = super().dispatch(method, eta_key="none")
-
-        @wraps(method)
-        def func(*args, **kwargs):
-            for _ in iter_func(*args, **kwargs):
-                pass
-            return None
-
-        return func
-
-
-@set_module("sisl.physics")
-class ListApply(IteratorApply):
-    __str__ = _apply_str("return list")
-
-    def dispatch(self, method):
-        """ Dispatch the method by returning list of values """
-        iter_func = super().dispatch(method, eta_key="list")
-        @wraps(method)
-        def func(*args, **kwargs):
-            return [v for v in iter_func(*args, **kwargs)]
-        return func
-
-
-@set_module("sisl.physics")
-class OpListApply(IteratorApply):
-    __str__ = _apply_str("return oplist")
-
-    def dispatch(self, method):
-        """ Dispatch the method by returning oplist of values """
-        iter_func = super().dispatch(method, eta_key="oplist")
-        @wraps(method)
-        def func(*args, **kwargs):
-            return oplist(v for v in iter_func(*args, **kwargs))
-        return func
-
-
-@set_module("sisl.physics")
-class ArrayApply(BrillouinZoneParentApply):
-    __str__ = _apply_str("return numpy.ndarray")
-
-    def dispatch(self, method, eta_key="array"):
-        """ Dispatch the method by one array """
-        @wraps(method)
-        def func(*args, wrap=None, eta=False, **kwargs):
-            bz, parent, wrap, eta = self._parse_kwargs(wrap, eta, eta_key=eta_key)
-            k = bz.k
-            w = bz.weight
-
-            # Get first values
-            v = wrap(method(*args, k=k[0], **kwargs), parent=parent, k=k[0], weight=w[0])
-            eta.update()
-
-            # Create full array
-            if v.ndim == 0:
-                a = np.empty([len(k)], dtype=v.dtype)
-            else:
-                a = np.empty((len(k), ) + v.shape, dtype=v.dtype)
-            a[0] = v
-            del v
-
-            for i in range(1, len(k)):
-                a[i] = wrap(method(*args, k=k[i], **kwargs), parent=parent, k=k[i], weight=w[i])
-                eta.update()
-            eta.close()
-
-            return a
-        return func
-
-
-@set_module("sisl.physics")
-class AverageApply(BrillouinZoneParentApply):
-    __str__ = _apply_str("return average")
-
-    def dispatch(self, method):
-        """ Dispatch the method by averaging """
-        @wraps(method)
-        def func(*args, wrap=None, eta=False, **kwargs):
-            bz, parent, wrap, eta = self._parse_kwargs(wrap, eta, eta_key="average")
-            # Do actual average
-            k = bz.k
-            w = bz.weight
-            v = _asoplist(wrap(method(*args, k=k[0], **kwargs), parent=parent, k=k[0], weight=w[0])) * w[0]
-            eta.update()
-            for i in range(1, len(k)):
-                v += _asoplist(wrap(method(*args, k=k[i], **kwargs), parent=parent, k=k[i], weight=w[i])) * w[i]
-                eta.update()
-            eta.close()
-            return v
-
-        return func
-
-
-@set_module("sisl.physics")
-class DataArrayApply(ArrayApply):
-    __str__ = _apply_str("return xarray.DataArray")
-
-    def dispatch(self, method):
-        """ Dispatch the method by returning a DataArray """
-        # Get data as array
-        array_func = super().dispatch(method, eta_key="dataarray")
-
-        @wraps(method)
-        def func(*args, coords=None, name=method.__name__, **kwargs):
-            # xarray specific data (default to function name)
-            bz = self._obj
-
-            # retrieve ALL data
-            array = array_func(*args, **kwargs)
-
-            # Create coords
-            if coords is None:
-                coords = [('k', _a.arangei(len(bz)))]
-                for i, v in enumerate(array.shape[1:]):
-                    coords.append((f"v{i+1}", _a.arangei(v)))
-            else:
-                coords = list(coords)
-                coords.insert(0, ('k', _a.arangei(len(bz))))
-                for i in range(1, len(coords)):
-                    if isinstance(coords[i], str):
-                        coords[i] = (coords[i], _a.arangei(array.shape[i]))
-            attrs = {'bz': bz, 'parent': bz.parent}
-
-            return xarray.DataArray(array, coords=coords, name=name, attrs=attrs)
-
-        return func
-
-# Add apply functions
-# Since apply is a built-in, we cannot assign as a class variable. :(
-setattr(BrillouinZone, "apply",
-        ClassDispatcher("apply",
-                        obj_getattr=lambda obj, key: getattr(obj.parent, key)
-        )
-)
-# Register dispatched functions
-BrillouinZone.apply.register("average", AverageApply)
-BrillouinZone.apply.register("sum", SumApply)
-BrillouinZone.apply.register("array", ArrayApply)
-BrillouinZone.apply.register("none", NoneApply)
-BrillouinZone.apply.register("iter", IteratorApply, default=True)
-BrillouinZone.apply.register("list", ListApply)
-BrillouinZone.apply.register("oplist", OpListApply)
-if _has_xarray:
-    BrillouinZone.apply.register("dataarray", DataArrayApply)
-
-
 @set_module("sisl.physics")
 class MonkhorstPack(BrillouinZone):
     r""" Create a Monkhorst-Pack grid for the Brillouin zone
@@ -1288,7 +1137,7 @@ class MonkhorstPack(BrillouinZone):
     parent : object or array_like
        An object with associated `parent.cell` and `parent.rcell` or
        an array of floats which may be turned into a `SuperCell`
-    nktp : array_like of ints
+    nkpt : array_like of ints
        a list of number of k-points along each cell direction
     displacement : float or array_like of float, optional
        the displacement of the evenly spaced grid, a single floating
@@ -1340,7 +1189,7 @@ class MonkhorstPack(BrillouinZone):
         # Retrieve the diagonal number of values
         Dn = np.diag(nkpt).astype(np.int32)
         if np.any(Dn) == 0:
-            raise ValueError(self.__class__.__name__ + ' *must* be initialized with '
+            raise ValueError(f'{self.__class__.__name__} *must* be initialized with '
                              'diagonal elements different from 0.')
 
         i_trs = -1
@@ -1466,9 +1315,17 @@ class MonkhorstPack(BrillouinZone):
         self._centered = state['centered']
         self._trs = state['trs']
 
-    def copy(self):
-        """ Create a copy of this object """
-        bz = self.__class__(self.parent, self._diag, self._displ, self._size, self._centered, self._trs >= 0)
+    def copy(self, parent=None):
+        """ Create a copy of this object, optionally changing the parent
+
+        Parameters
+        ----------
+        parent : optional
+           change the parent
+        """
+        if parent is None:
+            parent = self.parent
+        bz = self.__class__(parent, self._diag, self._displ, self._size, self._centered, self._trs >= 0)
         bz._k = self._k.copy()
         bz._w = self._w.copy()
         return bz
@@ -1517,7 +1374,7 @@ class MonkhorstPack(BrillouinZone):
 
             func = self._bz_get_func()
             wrap = allow_kwargs('parent', 'k', 'weight')(kwargs.pop('wrap', lambda x: x))
-            eta = tqdm_eta(len(self), self.__class__.__name__ + '.asgrid',
+            eta = tqdm_eta(len(self), f'{self.__class__.__name__}.asgrid',
                            'k', kwargs.pop('eta', False))
             parent = self.parent
             k = self.k
@@ -1777,7 +1634,7 @@ class MonkhorstPack(BrillouinZone):
         # k-point volumes.
         k_int = mp._size / k_vol
         if not np.allclose(np.rint(k_int), k_int):
-            raise SislError(self.__class__.__name__ + '.reduce could not replace k-point, BZ '
+            raise SislError(f'{self.__class__.__name__}.reduce could not replace k-point, BZ '
                             'volume replaced is not equivalent to the inherent k-point volume.')
         k_int = np.rint(k_int)
 
@@ -1792,7 +1649,7 @@ class MonkhorstPack(BrillouinZone):
         diff_k = self.in_primitive(self.k % 1. - k % 1.)
         idx = np.logical_and.reduce(np.abs(diff_k) <= dk, axis=1).nonzero()[0]
         if len(idx) == 0:
-            raise SislError(self.__class__.__name__ + '.reduce could not find any points to replace.')
+            raise SislError(f'{self.__class__.__name__}.reduce could not find any points to replace.')
 
         # Now we have the k-points we need to remove
         # Figure out if the total weight is consistent
@@ -1803,7 +1660,7 @@ class MonkhorstPack(BrillouinZone):
         elif abs(total_weight - replace_weight * 2) < 1e-8:
             weight_factor = 2.
             if self._trs < 0:
-                info(self.__class__.__name__ + '.reduce assumes that the replaced k-point has double weights.')
+                info(f'{self.__class__.__name__}.reduce assumes that the replaced k-point has double weights.')
         else:
             print('k-point to replace:')
             print(' ', k.ravel())
@@ -1813,7 +1670,7 @@ class MonkhorstPack(BrillouinZone):
             print(' ', idx)
             print('k-points replaced:')
             print(self.k[idx, :])
-            raise SislError(self.__class__.__name__ + '.reduce could not assert the weights are consistent during replacement.')
+            raise SislError(f'{self.__class__.__name__}.reduce could not assert the weights are consistent during replacement.')
 
         self._k = np.delete(self._k, idx, axis=0)
         self._w = np.delete(self._w, idx)
@@ -1897,8 +1754,7 @@ class BandStructure(BrillouinZone):
 
             division = div[:]
 
-        self.division = _a.arrayi(division)
-        self.division.shape = (-1,)
+        self.division = _a.arrayi(division).ravel()
 
         if name is None:
             self.name = 'ABCDEFGHIJKLMNOPQRSTUVXYZ'[:len(self.point)]
@@ -1908,6 +1764,19 @@ class BandStructure(BrillouinZone):
         self._k = _a.arrayd([k for k in self])
         self._w = _a.fulld(len(self.k), 1 / len(self.k))
 
+    def copy(self, parent=None):
+        """ Create a copy of this object, optionally changing the parent
+
+        Parameters
+        ----------
+        parent : optional
+           change the parent
+        """
+        if parent is None:
+            parent = self.parent
+        bz = self.__class__(parent, self.point, self.division, self.name)
+        return bz
+
     def __getstate__(self):
         """ Return dictionary with the current state """
         state = super().__getstate__()
