@@ -3,6 +3,7 @@ from scipy.interpolate import CubicSpline
 
 from sisl._internal import set_module
 import sisl._array as _a
+from ..messages import deprecate_method
 from .distribution import get_distribution
 from .electron import EigenvalueElectron, EigenstateElectron, spin_squared
 from .sparse import SparseOrbitalBZSpin
@@ -97,6 +98,9 @@ class Hamiltonian(SparseOrbitalBZSpin):
            the returned format of the matrix, defaulting to the ``scipy.sparse.csr_matrix``,
            however if one always requires operations on dense matrices, one can always
            return in `numpy.ndarray` (`'array'`/`'dense'`/`'matrix'`).
+           Prefixing with 'sc:', or simply 'sc' returns the matrix in supercell format
+           with phases. This is useful for e.g. bond-current calculations where individual
+           hopping + phases are required.
         spin : int, optional
            if the Hamiltonian is a spin polarized one can extract the specific spin direction
            matrix by passing an integer (0 or 1). If the Hamiltonian is not `Spin.POLARIZED`
@@ -219,8 +223,8 @@ class Hamiltonian(SparseOrbitalBZSpin):
 
         Returns
         -------
-        tuple of tuples
-            for each of the Cartesian directions
+        list of matrices
+            for each of the Cartesian directions (in Voigt representation); xx, yy, zz, zy, xz, xy
         """
         pass
 
@@ -271,6 +275,9 @@ class Hamiltonian(SparseOrbitalBZSpin):
         sparse : bool, optional
             if ``True``, `eigsh` will be called, else `eigh` will be
             called (default).
+        format : str, optional
+            see `eigh` for details, this will be passed to the EigenstateElectron
+            instance to be used in subsequent calls, may speed up post-processing.
         **kwargs : dict, optional
             passed arguments to the `eigh` routine
 
@@ -283,6 +290,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         -------
         EigenvalueElectron
         """
+        format = kwargs.pop("format", None)
         if kwargs.pop('sparse', False):
             e = self.eigsh(k, gauge=gauge, eigvals_only=True, **kwargs)
         else:
@@ -291,6 +299,8 @@ class Hamiltonian(SparseOrbitalBZSpin):
         for name in ["spin"]:
             if name in kwargs:
                 info[name] = kwargs[name]
+        if not format is None:
+            info["format"] = format
         return EigenvalueElectron(e, self, **info)
 
     def eigenstate(self, k=(0, 0, 0), gauge='R', **kwargs):
@@ -305,6 +315,9 @@ class Hamiltonian(SparseOrbitalBZSpin):
         sparse : bool, optional
             if ``True``, `eigsh` will be called, else `eigh` will be
             called (default).
+        format : str, optional
+            see `eigh` for details, this will be passed to the EigenstateElectron
+            instance to be used in subsequent calls, may speed up post-processing.
         **kwargs : dict, optional
             passed arguments to the `eigh`/`eighs` routine
 
@@ -317,6 +330,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         -------
         EigenstateElectron
         """
+        format = kwargs.pop("format", None)
         if kwargs.pop('sparse', False):
             e, v = self.eigsh(k, gauge=gauge, eigvals_only=False, **kwargs)
         else:
@@ -325,6 +339,8 @@ class Hamiltonian(SparseOrbitalBZSpin):
         for name in ["spin"]:
             if name in kwargs:
                 info[name] = kwargs[name]
+        if not format is None:
+            info["format"] = format
         # Since eigh returns the eigenvectors [:, i] we have to transpose
         return EigenstateElectron(v.T, e, self, **info)
 
@@ -360,6 +376,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
             with get_sile(sile, 'w') as fh:
                 fh.write_hamiltonian(self, *args, **kwargs)
 
+    @deprecate_method("Please use Hamiltonian.eigenstate(...).spin_squared() instead")
     def spin_squared(self, k=(0, 0, 0), n_up=None, n_down=None, **kwargs):
         r""" Calculate spin-squared expectation value, see `~sisl.physics.electron.spin_squared` for details
 
@@ -386,13 +403,16 @@ class Hamiltonian(SparseOrbitalBZSpin):
         # es_alpha.Sk should equal es_beta.Sk, so just pass one of them
         return spin_squared(es_alpha.state, es_beta.state, es_alpha.Sk())
 
-    def velocity(self, k=(0, 0, 0), **kwargs):
+    @deprecate_method("Please use Hamiltonian.eigenstate(...).velocity() instead")
+    def velocity(self, k=(0, 0, 0), project=False, **kwargs):
         r""" Calculate the velocity for the eigenstates for a given `k` point
 
         Parameters
         ----------
         k : array_like, optional
             k-point at which the velocities are calculated
+        project : {}
+            see `sisl.physics.electron.velocity` for details and possible arguments
         **kwargs : optional
             additional parameters passed to the `eigenstate` routine
 
@@ -401,15 +421,18 @@ class Hamiltonian(SparseOrbitalBZSpin):
         eigenstate : method used to calculate the eigenstates
         EigenvalueElectron.velocity : Underlying method used to calculate the velocity
         """
-        return self.eigenstate(k, **kwargs).velocity()
+        return self.eigenstate(k, **kwargs).velocity(project=project)
 
-    def spin_moment(self, k=(0, 0, 0), **kwargs):
+    @deprecate_method("Please use Hamiltonian.eigenstate(...).spin_moment() instead")
+    def spin_moment(self, k=(0, 0, 0), project=False, **kwargs):
         r""" Calculate the spin moment for the eigenstates for a given `k` point
 
         Parameters
         ----------
         k : array_like, optional
             k-point at which the spin moments are calculated
+        project : bool, optional
+           whether the moments are orbitally resolved or not
         **kwargs : optional
             additional parameters passed to the `eigenstate` routine
 
@@ -420,6 +443,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         """
         return self.eigenstate(k, **kwargs).spin_moment()
 
+    @deprecate_method("Please use Hamiltonian.eigenstate(...).DOS() instead")
     def DOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the DOS at the given energies for a specific `k` point
 
@@ -444,6 +468,7 @@ class Hamiltonian(SparseOrbitalBZSpin):
         """
         return self.eigenvalue(k, **kwargs).DOS(E, distribution)
 
+    @deprecate_method("Please use Hamiltonian.eigenstate(...).PDOS() instead")
     def PDOS(self, E, k=(0, 0, 0), distribution='gaussian', **kwargs):
         r""" Calculate the projected DOS at the given energies for a specific `k` point
 
