@@ -50,9 +50,9 @@ class TestBrillouinZone:
         assert len(bz) == 2
         assert len(bz.copy()) == 2
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_bz_fail(self, setup):
-        BrillouinZone(setup.s1, [0] * 3, [.5] * 2)
+        with pytest.raises(ValueError):
+            BrillouinZone(setup.s1, [0] * 3, [.5] * 2)
 
     def test_to_reduced(self, setup):
         bz = BrillouinZone(setup.s2)
@@ -142,7 +142,6 @@ class TestBrillouinZone:
             shape[i] = 3
             assert np.allclose(grid.shape, shape)
 
-    @pytest.mark.xfail(raises=SislError)
     def test_mp_asgrid_fail(self, setup):
         class Test(SuperCellChild):
             def __init__(self, sc):
@@ -150,7 +149,8 @@ class TestBrillouinZone:
             def eigh(self, k, *args, **kwargs):
                 return np.arange(3)
         bz = MonkhorstPack(Test(setup.s1), [2] * 3, displacement=[0.1] * 3).asgrid()
-        bz.eigh(wrap=lambda eig: eig[0])
+        with pytest.raises(SislError):
+            bz.eigh(wrap=lambda eig: eig[0])
 
     def test_mp1(self, setup):
         bz = MonkhorstPack(setup.s1, [2] * 3, trs=False)
@@ -253,10 +253,7 @@ class TestBrillouinZone:
         apply.none.eigh()
 
     def test_as_dataarray(self):
-        try:
-            import xarray
-        except ImportError:
-            pytest.skip('xarray not available')
+        pytest.importorskip("xarray", reason="xarray not available")
 
         from sisl import geom, Hamiltonian
         g = geom.graphene()
@@ -277,6 +274,76 @@ class TestBrillouinZone:
         asdarray = bz_da.eigh(coords=['orb'])
         assert asdarray.dims == ('k', 'orb')
 
+    def test_as_dataarray_unzip(self):
+        pytest.importorskip("xarray", reason="xarray not available")
+
+        from sisl import geom, Hamiltonian
+        g = geom.graphene()
+        H = Hamiltonian(g)
+        H.construct([[0.1, 1.44], [0, -2.7]])
+
+        E = np.linspace(-2, 2, 20)
+        bz = MonkhorstPack(H, [2, 2, 1], trs=False)
+        def wrap(es):
+            return es.eig, es.DOS(E), es.PDOS(E)
+
+        with bz.apply.renew(unzip=True) as unzip:
+            eig, DOS, PDOS = unzip.ndarray.eigenstate(wrap=wrap)
+            ds0 = unzip.dataarray.eigenstate(wrap=wrap, name=["eig", "DOS", "PDOS"])
+            # explicitly create dimensions
+            ds1 = unzip.dataarray.eigenstate(wrap=wrap,
+                                             coords=[
+                                                 {"orb": np.arange(len(H))},
+                                                 {"E": E},
+                                                 {"orb": np.arange(len(H)),
+                                                  "E": E},
+                                             ],
+                                             dims=(['orb'], ['E'], ['orb', 'E']),
+                                             name=["eig", "DOS", "PDOS"])
+
+        for var, data in zip(["eig", "DOS", "PDOS"], [eig, DOS, PDOS]):
+            assert np.allclose(ds0.data_vars[var].values, data)
+            assert np.allclose(ds1.data_vars[var].values, data)
+        assert len(ds1.coords) < len(ds0.coords)
+
+    def test_pathos(self):
+        pytest.importorskip("pathos", reason="pathos not available")
+
+        from sisl import geom, Hamiltonian
+        g = geom.graphene()
+        H = Hamiltonian(g)
+        H.construct([[0.1, 1.44], [0, -2.7]])
+
+        bz = MonkhorstPack(H, [2, 2, 2], trs=False)
+
+        # Check that the ObjectDispatcher works
+        apply = bz.apply
+        papply = bz.apply.renew(pool=True)
+        assert str(apply) != str(papply)
+
+        for method in ["iter", "average", "sum", "array", "list", "oplist"]:
+            # TODO One should be careful with zip
+            # zip will stop when it hits the final element in the first
+            # list.
+            # So if a generator has some clean-up code one has to use zip_longest
+            # regardless of method
+            for v1, v2 in zip(papply[method].eigh(), apply[method].eigh()):
+                assert np.allclose(v1, v2)
+
+        # Check that the MethodDispatcher works
+        apply = bz.apply.eigh
+        papply = bz.apply.renew(pool=True).eigh
+        assert str(apply) != str(papply)
+
+        for method in ["iter", "average", "sum", "array", "list", "oplist"]:
+            # TODO One should be careful with zip
+            # zip will stop when it hits the final element in the first
+            # list.
+            # So if a generator has some clean-up code one has to use zip_longest
+            # regardless of method
+            for v1, v2 in zip(papply[method](), apply[method]()):
+                assert np.allclose(v1, v2)
+
     def test_as_single(self):
         from sisl import geom, Hamiltonian
         g = geom.graphene()
@@ -332,7 +399,7 @@ class TestBrillouinZone:
         assert len(bz) == 2 ** 3
 
         # Check with a wrap function
-        E = np.linspace(-2, 2, 100)
+        E = np.linspace(-2, 2, 20)
         def wrap_sum(es, weight):
             PDOS = es.PDOS(E) * weight
             return PDOS.sum(0), PDOS
@@ -342,6 +409,34 @@ class TestBrillouinZone:
         assert np.allclose(bz_arr.DOS(E), DOS)
         assert np.allclose(bz_arr.PDOS(E), PDOS)
 
+    def test_wrap_unzip(self):
+        from sisl import geom, Hamiltonian
+        g = geom.graphene()
+        H = Hamiltonian(g)
+        H.construct([[0.1, 1.44], [0, -2.7]])
+
+        bz = MonkhorstPack(H, [2, 2, 2], trs=False)
+
+        # Check with a wrap function
+        E = np.linspace(-2, 2, 20)
+        def wrap(es):
+            return es.eig, es.DOS(E)
+
+        eig0, DOS0 = zip(*bz.apply.list.eigenstate(wrap=wrap))
+        with bz.apply.renew(unzip=True) as k_unzip:
+            eig1, DOS1 = k_unzip.list.eigenstate(wrap=wrap)
+            eig2, DOS2 = k_unzip.array.eigenstate(wrap=wrap)
+
+        #eig0 and DOS0 are generators, and not list's
+        #eig1 and DOS1 are generators, and not list's
+        assert isinstance(eig2, np.ndarray)
+        assert isinstance(DOS2, np.ndarray)
+
+        assert np.allclose(eig0, eig1)
+        assert np.allclose(DOS0, DOS1)
+        assert np.allclose(eig0, eig2)
+        assert np.allclose(DOS0, DOS2)
+
     # Check with a wrap function and the weight argument
     def test_wrap_kwargs(arg):
         from sisl import geom, Hamiltonian
@@ -357,7 +452,7 @@ class TestBrillouinZone:
         def wrap_kwargs(arg, parent, k, weight):
             return arg * weight
 
-        E = np.linspace(-2, 2, 100)
+        E = np.linspace(-2, 2, 20)
         bz_array = bz.apply.array
         asarray1 = (bz_array.DOS(E, wrap=wrap_none) * bz.weight.reshape(-1, 1)).sum(0)
         asarray2 = bz_array.DOS(E, wrap=wrap_kwargs).sum(0)
