@@ -1,5 +1,6 @@
 import pytest
 
+from functools import partial
 import warnings
 import numpy as np
 from scipy.linalg import block_diag
@@ -221,11 +222,53 @@ class TestHamiltonian:
         assert np.allclose(csr, arr)
         assert np.allclose(csr, coo)
 
-    @pytest.mark.xfail(raises=ValueError)
-    def test_construct_raise(self, setup):
+    @pytest.mark.parametrize("orthogonal", [True, False])
+    @pytest.mark.parametrize("gauge", ["R", "r"])
+    @pytest.mark.parametrize("spin", ["unpolarized", "polarized", "non-collinear", "spin-orbit"])
+    @pytest.mark.parametrize("dtype", [np.complex64, np.complex128])
+    def test_format_sc(self, orthogonal, gauge, spin, dtype):
+        g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell([10, 1, 5.], nsc=[3, 3, 1]))
+        H = Hamiltonian(g, dtype=np.float64, orthogonal=orthogonal, spin=Spin(spin))
+        nd = H._csr._D.shape[-1]
+        # this will correctly account for the double size for NC/SOC
+        no = len(H)
+        no_s = H.geometry.no_s
+        for ia in g:
+            idx = g.close(ia, R=(0.1, 1.01))[1]
+            H[ia, ia] = 1.
+            H[ia, idx] = np.random.rand(nd)
+        if dtype == np.complex64:
+            atol = 1e-6
+            rtol = 1e-12
+        else:
+            atol = 1e-9
+            rtol = 1e-15
+        allclose = partial(np.allclose, atol=atol, rtol=rtol)
+
+        H = (H + H.transpose(hermitian=True)) / 2
+        n_s = H.geometry.sc.n_s
+
+        for k in [[0, 0, 0], [0.15, 0.1, 0.05]]:
+            for attr, kwargs in [("Hk", {"gauge": gauge}), ("Sk", {})]:
+                Mk = getattr(H, attr)
+                csr = Mk(k, format='csr', **kwargs, dtype=dtype)
+                sc_csr1 = Mk(k, format='sc:csr', **kwargs, dtype=dtype)
+                sc_csr2 = Mk(k, format='sc', **kwargs, dtype=dtype)
+                sc_mat = Mk(k, format='sc:array', **kwargs, dtype=dtype)
+                mat = sc_mat.reshape(no, n_s, no).sum(1)
+
+                assert sc_mat.shape == sc_csr1.shape
+                assert allclose(csr.toarray(), mat)
+                assert allclose(sc_csr1.toarray(), sc_csr2.toarray())
+                for isc in range(n_s):
+                    csr -= sc_csr1[:, isc * no: (isc + 1) * no]
+                assert allclose(csr.toarray(), 0.)
+
+    def test_construct_raise_default(self, setup):
         # Test that construct fails with more than one
         # orbital
-        setup.H2.construct([(0.1, 1.5), (1., 0.1)])
+        with pytest.raises(ValueError):
+            setup.H2.construct([(0.1, 1.5), (1., 0.1)])
 
     def test_getitem1(self, setup):
         H = setup.H
@@ -261,12 +304,12 @@ class TestHamiltonian:
         assert H.spsame(setup.H)
         setup.H.empty()
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_fromsp2(self, setup):
         H = setup.H.copy()
         H.construct([(0.1, 1.5), (1., 0.1)])
         csr = H.tocsr(0)
-        Hamiltonian.fromsp(setup.H.geometry.tile(2, 0), csr)
+        with pytest.raises(ValueError):
+            Hamiltonian.fromsp(setup.H.geometry.tile(2, 0), csr)
 
     def test_fromsp3(self, setup):
         H = setup.HS.copy()
@@ -588,7 +631,20 @@ class TestHamiltonian:
 
         es2.change_gauge('r')
         es1.change_gauge('r')
-        assert np.allclose(es1.velocity(), es2.velocity())
+        v1 = es1.velocity()
+        v2 = es2.velocity()
+        assert np.allclose(v1, v2)
+
+        # Projected velocity
+        pv1 = es1.velocity(project=True)
+        pv2 = es2.velocity(project=True)
+        assert np.allclose(pv1.sum(1), v2)
+        assert np.allclose(pv2.sum(1), v1)
+        # since degenerate states *could* swap states
+        # we can't for sure compare states
+        # This test is one of those cases
+        # hence the below is disabled
+        #assert np.allclose(pv1, pv2)
 
     def test_berry_phase(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
@@ -600,12 +656,12 @@ class TestHamiltonian:
         berry_phase(bz, sub=0)
         berry_phase(bz, eigvals=True, sub=0)
 
-    @pytest.mark.xfail(raises=SislError)
     def test_berry_phase_fail_sc(self, setup):
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
         H = Hamiltonian(g)
         bz = BandStructure.param_circle(H.geometry.sc, 20, 0.01, [0, 0, 1], [1/3] * 3)
-        berry_phase(bz)
+        with pytest.raises(SislError):
+            berry_phase(bz)
 
     def test_berry_phase_loop(self, setup):
         g = setup.g.tile(2, 0).tile(2, 1).tile(2, 2)
@@ -629,7 +685,6 @@ class TestHamiltonian:
         # Just to do the other branch
         berry_phase(bz, method='zak')
 
-    @pytest.mark.xfail(raises=SislError)
     def test_berry_phase_method_fail(self):
         # wrong method keyword
         g = Geometry([[-.6, 0, 0], [0.6, 0, 0]], Atom(1, 1.001), sc=[2, 10, 10])
@@ -641,7 +696,8 @@ class TestHamiltonian:
         K = np.zeros([k.size, 3])
         K[:, 0] = k
         bz = BrillouinZone(H, K)
-        berry_phase(bz, method='unknown')
+        with pytest.raises(ValueError):
+            berry_phase(bz, method='unknown')
 
     def test_berry_curvature(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
@@ -931,7 +987,7 @@ class TestHamiltonian:
         assert len(sup) == 2
         assert len(sdn) == 1
 
-    def test_non_colinear1(self, setup):
+    def test_non_colinear_orthogonal(self, setup):
         g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
         H = Hamiltonian(g, dtype=np.float64, spin=Spin.NONCOLINEAR)
         for i in range(10):
@@ -984,7 +1040,7 @@ class TestHamiltonian:
             assert np.allclose(sm[:, 2], sm2)
             assert np.allclose(sm[:, 2], sm3)
 
-            om = es.spin_orbital_moment()
+            om = es.spin_moment(project=True)
             assert np.allclose(sm, om.sum(1))
 
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
@@ -993,6 +1049,12 @@ class TestHamiltonian:
             es.velocity_matrix()
             es.inv_eff_mass_tensor()
 
+        # Check the velocities
+        # But only compare for np.float64, we need the precision
+        v = es.velocity()
+        pv = es.velocity(project=True)
+        assert np.allclose(pv.sum(1), v)
+
         # Ensure we can change gauge for NC stuff
         es.change_gauge('R')
         es.change_gauge('r')
@@ -1042,7 +1104,7 @@ class TestHamiltonian:
 
             sm = es.spin_moment()
 
-            om = es.spin_orbital_moment()
+            om = es.spin_moment(project=True)
             assert np.allclose(sm, om.sum(1))
 
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
@@ -1051,11 +1113,17 @@ class TestHamiltonian:
             es.velocity_matrix()
             es.inv_eff_mass_tensor()
 
+        # Check the velocities
+        # But only compare for np.float64, we need the precision
+        v = es.velocity()
+        pv = es.velocity(project=True)
+        assert np.allclose(pv.sum(1), v)
+
         # Ensure we can change gauge for NC stuff
         es.change_gauge('R')
         es.change_gauge('r')
 
-    def test_so1(self):
+    def test_spin_orbit_orthogonal(self):
         g = Geometry([[i, 0, 0] for i in range(10)], Atom(6, R=1.01), sc=SuperCell(100, nsc=[3, 3, 1]))
         H = Hamiltonian(g, dtype=np.float64, spin=Spin.SPINORBIT)
         for i in range(10):
@@ -1085,6 +1153,10 @@ class TestHamiltonian:
             H1[i, i, 1] = 0.05
             H1[i, i, 2] = 0.1
             H1[i, i, 3] = 0.1
+            H1[i, i, 4] = 0.1
+            H1[i, i, 5] = 0.1
+            H1[i, i, 6] = 0.1
+            H1[i, i, 7] = 0.1
             if i > 0:
                 H1[i, i-1, 0] = 1.
                 H1[i, i-1, 1] = 1.
@@ -1093,8 +1165,8 @@ class TestHamiltonian:
                 H1[i, i+1, 1] = 1.
         assert H1.spsame(H)
         eig1 = H1.eigh(dtype=np.complex64)
-        assert np.allclose(H1.eigh(dtype=np.complex128), eig1)
-        assert np.allclose(H.eigh(dtype=np.complex64), H1.eigh(dtype=np.complex128))
+        assert np.allclose(H1.eigh(dtype=np.complex128), eig1, atol=1e-5)
+        assert np.allclose(H.eigh(dtype=np.complex64), H1.eigh(dtype=np.complex128), atol=1e-5)
 
         # Create the block matrix for expectation
         SZ = block_diag(*([H1.spin.Z] * H1.no))
@@ -1109,7 +1181,7 @@ class TestHamiltonian:
             assert np.allclose(sm[:, 2], sm2)
             assert np.allclose(sm[:, 2], sm3)
 
-            om = es.spin_orbital_moment()
+            om = es.spin_moment(project=True)
             assert np.allclose(sm, om.sum(1))
 
             PDOS = es.PDOS(np.linspace(-1, 1, 100))
@@ -1118,6 +1190,12 @@ class TestHamiltonian:
             es.velocity_matrix()
             es.inv_eff_mass_tensor()
 
+        # Check the velocities
+        # But only compare for np.float64, we need the precision
+        v = es.velocity()
+        pv = es.velocity(project=True)
+        assert np.allclose(pv.sum(1), v)
+
         # Ensure we can change gauge for SO stuff
         es.change_gauge('R')
         es.change_gauge('r')
@@ -1138,36 +1216,25 @@ class TestHamiltonian:
         setup.HS.empty()
 
     @pytest.mark.slow
-    def test_tile1(self, setup):
+    @pytest.mark.parametrize("nx", [1, 4])
+    @pytest.mark.parametrize("ny", [1, 5])
+    @pytest.mark.parametrize("nz", [1, 6])
+    def test_tile_same(self, setup, nx, ny, nz):
         R, param = [0.1, 1.5], [1., 0.1]
 
         # Create reference
-        Hg = Hamiltonian(setup.g.tile(2, 0).tile(2, 1).tile(2, 2))
+        Hg = Hamiltonian(setup.g.tile(nx, 0).tile(ny, 1).tile(nz, 2))
         Hg.construct([R, param])
         Hg.finalize()
         H = Hamiltonian(setup.g)
         H.construct([R, param])
-        H = H.tile(2, 0).tile(2, 1).tile(2, 2)
-        assert Hg.spsame(H)
-        H.finalize()
-        Hg.finalize()
-        assert np.allclose(H._csr._D, Hg._csr._D)
-
-    @pytest.mark.slow
-    def test_tile2(self, setup):
-        R, param = [0.1, 1.5], [1., 0.1]
-
-        # Create reference
-        Hg = Hamiltonian(setup.g.tile(2, 0))
-        Hg.construct([R, param])
-        Hg.finalize()
-        H = Hamiltonian(setup.g)
-        H.construct([R, param])
-        H = H.tile(2, 0)
+        H = H.tile(nx, 0).tile(ny, 1).tile(nz, 2)
         assert Hg.spsame(H)
         H.finalize()
         Hg.finalize()
         assert np.allclose(H._csr._D, Hg._csr._D)
+        assert np.allclose(Hg.Hk([0.1, 0.2, 0.3], format='array'),
+                           H.Hk([0.1, 0.2, 0.3], format='array'))
 
     @pytest.mark.slow
     def test_tile3(self, setup):
@@ -1428,12 +1495,12 @@ class TestHamiltonian:
         H.construct([R, param])
         assert len(H.edges(0)) == 4
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_edges2(self, setup):
         R, param = [0.1, 1.5], [1., 0.1]
         H = Hamiltonian(setup.g)
         H.construct([R, param])
-        H.edges()
+        with pytest.raises(ValueError):
+            H.edges()
 
     def test_edges3(self, setup):
         def func(self, ia, atoms, atoms_xyz=None):
