@@ -7,13 +7,14 @@ import numpy as np
 # To speed up the _extend algorithm we limit lookups
 from numpy import (
     ndarray, int32,
-    empty, zeros, full, arange,
+    empty, zeros, full, arange, repeat,
     asarray, atleast_1d,
-    take, delete, insert, split,
+    take, delete, insert, split, concatenate,
     copyto,
+    argsort, lexsort,
     intersect1d, setdiff1d, unique, in1d,
     diff, count_nonzero, allclose,
-    isnan, broadcast, argsort, isscalar,
+    isnan, isscalar, broadcast,
     any as np_any, all as np_all
 )
 from numpy.lib.mixins import NDArrayOperatorsMixin
@@ -31,6 +32,7 @@ from ._indices import indices, indices_only, sorted_unique
 from .messages import warn, SislError
 from ._help import array_fill_repeat, get_dtype, isiterable
 from .utils.ranges import array_arange
+from .utils.mathematics import intersect_and_diff_sets
 from ._sparse import sparse_dense
 
 # Although this re-implements the CSR in scipy.sparse.csr_matrix
@@ -39,6 +41,14 @@ from ._sparse import sparse_dense
 __all__ = ['SparseCSR', 'ispmatrix', 'ispmatrixd']
 
 
+def _ncol_to_indptr(ncol):
+    """ Convert the ncol array into a pointer array """
+    ptr = _a.emptyi(ncol.size + 1)
+    ptr[0] = 0
+    _a.cumsumi(ncol, out=ptr[1:])
+    return ptr
+
+
 @set_module("sisl")
 class SparseCSR(NDArrayOperatorsMixin):
     """
@@ -63,7 +73,7 @@ class SparseCSR(NDArrayOperatorsMixin):
     This class may be instantiated by verious means.
 
     - ``SparseCSR(S)``
-      where ``S`` is a :module:`scipy.sparse` matrix
+      where ``S`` is a `scipy.sparse` matrix
     - ``SparseCSR((M,N)[, dtype])``
       the shape of the sparse matrix (equivalent
       to ``SparseCSR((M,N,1)[, dtype])``.
@@ -100,17 +110,6 @@ class SparseCSR(NDArrayOperatorsMixin):
        pointer index in the 1D column indices of the corresponding row
     col : int-array
        column indices of the sparse elements
-    data:
-       the data in the sparse matrix
-    dim : int
-       the extra dimension of the sparse matrix
-    nnz : int
-       number of contained sparse elements
-    shape : tuple, 3*(,)
-       size of contained matrix, M, N, K
-    finalized : boolean
-       whether the sparse matrix is finalized and non-set elements
-       are removed
     """
     # We don't really need slots, but it is useful
     # to keep a good overview of which variables are present
@@ -237,6 +236,60 @@ class SparseCSR(NDArrayOperatorsMixin):
         # Denote that this sparsity pattern hasn't been finalized
         self._finalized = False
 
+    @classmethod
+    def sparsity_union(cls, *spmats, dtype=None, dim=None, value=0):
+        """Create a SparseCSR with constant fill value in all places that `spmats` have nonzeros
+
+        Parameters
+        ----------
+        spmats : SparseCSR or csr_matrix
+            SparseCSRs to find the sparsity pattern union of.
+        dtype : dtype, optional
+            Output dtype. If not given, use the result dtype of the spmats.
+        dim : int, optional
+            If given, the returned SparseCSR will have this as dim.
+            By default the first given spmat decides the dimension.
+        value : scalar, default 0
+            The used fill value.
+        """
+        shape2 = spmats[0].shape[:2]
+        if not all(shape2 == m.shape[:2] for m in spmats):
+            raise ValueError(
+                f"Cannot find sparsity union of differently shaped csrs: "
+                + " & ".join(str(m.shape) for m in spmats)
+            )
+        if dim is not None:
+            shape = shape2 + (dim,)
+        elif len(spmats[0].shape) == 3:
+            shape = shape2 + (spmats[0].shape[2],)
+        else: # csr_matrix
+            shape = shape2 + (1,)
+
+        if dtype is None:
+            dtype = np.result_type(*(m.dtype for m in spmats))
+
+        out = cls(shape, dtype=dtype, nnzpr=1, nnz=2)
+
+        # Create sparsity union (columns/indices array)
+        out_col = []
+        for row in range(shape[0]):
+            row_cols = []
+            for mat in spmats:
+                if isinstance(mat, SparseCSR):
+                    row_cols.append(mat.col[mat.ptr[row]:mat.ptr[row] + mat.ncol[row]])
+                else:
+                    # we have to ensure it is a csr matrix
+                    mat = mat.tocsr()
+                    row_cols.append(mat.indices[mat.indptr[row]:mat.indptr[row+1]])
+            out_col.append(np.unique(concatenate(row_cols)))
+        # Put into the output
+        out.ncol = _a.arrayi([len(cols) for cols in out_col])
+        out.ptr = _ncol_to_indptr(out.ncol)
+        out.col = concatenate(out_col)
+        out._nnz = len(out.col)
+        out._D = full([out._nnz, out.dim], value, dtype=dtype)
+        return out
+
     def diagonal(self):
         r""" Return the diagonal elements from the matrix """
         return np.array([self[i, i] for i in range(self.shape[0])], dtype=self.dtype)
@@ -374,19 +427,24 @@ class SparseCSR(NDArrayOperatorsMixin):
         D = self._D
 
         # We truncate all the connections
+        # to get correct rows we need to remove those which have 0 elements
+        row_nonzero = (ncol > 0).nonzero()[0]
+        # Now we have [0 0 0 0 1 1 1 1 2 2 ... no-1 no-1]
+        row = repeat(row_nonzero, ncol[row_nonzero])
         if sort:
-            for r in range(self.shape[0]):
-                # Sort and check whether there are double entries
-                sl = slice(ptr[r], ptr[r+1])
-                ccol = col[sl]
-                DD = D[sl, :]
-                idx = argsort(ccol)
-                # Do in-place sorting
-                ccol[:] = ccol[idx]
-                if not sorted_unique(ccol):
-                    raise SislError('You cannot have two elements between the same ' +
-                                    f'i,j index (i={r}), something has went terribly wrong.')
-                DD[:, :] = DD[idx, :]
+            # first sort according to row (which is already sorted)
+            # then sort by col
+            idx = lexsort((col, row))
+            # direct re-order
+            col[:] = col[idx]
+            D[:, :] = D[idx, :]
+
+            # check that we don't have any duplicate values
+            duplicates = np.logical_and(diff(col) == 0, diff(row) == 0).nonzero()[0]
+
+            if len(duplicates) > 0:
+                raise SislError('You cannot have two elements between the same ' +
+                                f'i,j index (i={row[duplicates]}, j={col[duplicates]})')
 
         else:
             for r in range(self.shape[0]):
@@ -774,7 +832,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         if asarray(i).size == 0:
             return arrayi([])
         if i < 0 or i >= self.shape[0]:
-            raise IndexError('row index is out-of-bounds')
+            raise IndexError(f"row index is out-of-bounds {i} : {self.shape[0]}")
         i1 = int(i) + 1
         # We skip this check and let sisl die if wrong input is given...
         #if not isinstance(i, Integral):
@@ -786,7 +844,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         if len(j) == 0:
             return arrayi([])
         if np_any(j < 0) or np_any(j >= self.shape[1]):
-            raise IndexError('column index is out-of-bounds')
+            raise IndexError(f"column index is out-of-bounds {j} : {self.shape[1]}")
 
         # fast reference
         ptr = self.ptr
@@ -821,7 +879,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         new_nnz = new_n - int(ptr[i1]) + ncol_ptr_i
 
         if new_nnz > 0:
-
+            #print(f"new_nnz {i} : {new_nnz}")
             # Ensure that it is not-set as finalized
             # There is no need to set it all the time.
             # Simply because the first call to finalize
@@ -875,7 +933,7 @@ class SparseCSR(NDArrayOperatorsMixin):
             return indices(col[ptr_i:ncol_ptr_i], j, ptr_i)
 
     def _extend_empty(self, i, n):
-        """ Extends the sparsity pattern to retain elements `j` in row `i`
+        """ Extends the sparsity pattern with `n` elements in row `i`
 
         Parameters
         ----------
@@ -1024,7 +1082,7 @@ class SparseCSR(NDArrayOperatorsMixin):
             n = len(get_idx)
 
         # Indices of existing values in return array
-        ret_idx = (get_idx >= 0).nonzero()[0]
+        ret_idx = atleast_1d(get_idx >= 0).nonzero()[0]
         # Indices of existing values in get array
         get_idx = get_idx.ravel()[ret_idx]
 
@@ -1314,33 +1372,50 @@ class SparseCSR(NDArrayOperatorsMixin):
         # Use array_arange
         idx = array_arange(self.ptr[:-1], n=self.ncol)
         # create new pointer
-        ptr = insert(_a.cumsumi(self.ncol), 0, 0)
+        ptr = _ncol_to_indptr(self.ncol)
 
         return csr_matrix((self._D[idx, dim].copy(), self.col[idx], ptr.astype(int32, copy=False)),
                           shape=shape, **kwargs)
 
     @classmethod
-    def fromsp(cls, *sps, **kwargs):
-        shape = sps[0].shape
-        dtype = kwargs.get("dtype", np.result_type(*tuple(sp.dtype for sp in sps)))
-        out = cls(shape + (len(sps), ), dtype=dtype)
+    def fromsp(cls, *sps, dtype=None):
+        """ Combine multiple single-dimension sparse matrices into one SparseCSR matrix
 
-        ptr = out.ptr
-        ncol = out.ncol
-        col = out.col
+        The different sparse matrices need not have the same sparsity pattern.
 
-        # Now we need to add things to the sparsity pattern
-        for iD, sp in enumerate(sps):
-            sp = sp.tocsr()
-            if sp.shape != shape:
-                raise ValueError(f"{cls.__name__}.fromsp found non compatible shapes")
+        Parameters
+        ----------
+        *sps : sparse-matrix
+            any sparse matrix which can convert to a `scipy.sparse.csr_matrix` matrix
+        dtype : numpy.dtype, optional
+            data-type to store in the matrix, default to largest ``dtype`` for the
+            passed sparse matrices
+        """
+        if dtype is None:
+            dtype = np.result_type(*[sp.dtype for sp in sps])
 
-            # Loop stuff
-            for r in range(shape[0]):
-                sl = slice(sp.indptr[r], sp.indptr[r+1])
-                cols = sp.indices[sl]
-                idx = out._extend(r, cols)
-                out._D[idx, iD] += sp.data[sl]
+        if len(sps) == 1:
+            m = sps[0].tocsr()
+            out = cls(m.shape + (1,), nnzpr=1, nnz=1, dtype=dtype)
+            out.col = m.indices.copy()
+            out.ptr = m.indptr.copy()
+            out.ncol = np.diff(out.ptr)
+            out._nnz = len(out.col)
+            out._D = m.data.reshape(-1, 1).astype(dtype, copy=True)
+            return out
+
+        # Pre-allocate by finding sparsity pattern union
+        out = cls.sparsity_union(*sps, dim=len(sps), dtype=dtype)
+
+        # Now transfer the data
+        for im, m in enumerate(sps):
+            m = m.tocsr()
+            m.sort_indices()
+            for r in range(out.shape[0]):
+                msl = slice(m.indptr[r], m.indptr[r+1])
+                osl = slice(out.ptr[r], out.ptr[r] + out.ncol[r])
+                oidx = indices(out.col[osl], m.indices[msl], osl.start, both_sorted=True)
+                out._D[oidx, im] = m.data[msl]
 
         return out
 
@@ -1558,7 +1633,7 @@ class SparseCSR(NDArrayOperatorsMixin):
         self._nnz = self.ncol.sum()
         self._finalized = state['finalized']
         if self.finalized:
-            self.ptr = insert(_a.cumsumi(self.ncol), 0, 0)
+            self.ptr = _ncol_to_indptr(self.ncol)
         else:
             self.ptr = state['ptr']
 
@@ -1639,37 +1714,59 @@ def _ufunc_sp_sp(ufunc, a, b, **kwargs):
     """ Calculate ufunc on sparse matrices """
     if isinstance(a, tuple):
         a = SparseCSR.fromsp(*a)
-    elif not isinstance(a, SparseCSR):
-        a = SparseCSR.fromsp(a)
     if isinstance(b, tuple):
         b = SparseCSR.fromsp(*b)
-    elif not isinstance(b, SparseCSR):
-        b = SparseCSR.fromsp(b)
 
-    if not (reduce(op_eq, zip(a.shape[:2], b.shape[:2])) and
-            a.shape[2] == 1 or b.shape[2] == 1 or a.shape[2] == b.shape[2]):
+    def accessors(mat):
+        if isinstance(mat, SparseCSR):
+            def rowslice(r):
+                return slice(mat.ptr[r], mat.ptr[r] + mat.ncol[r])
+            accessors = mat.dim, mat.col, mat._D, rowslice
+            issorted = mat.finalized
+        else:
+            # makes this work for all matrices
+            # and csr_matrix.tocsr is a no-op
+            mat = mat.tocsr()
+            mat.sort_indices()
+            def rowslice(r):
+                return slice(mat.indptr[r], mat.indptr[r + 1])
+            accessors = 1, mat.indices, mat.data.reshape(-1, 1), rowslice
+            issorted = mat.has_sorted_indices
+        if issorted:
+            indexfunc = lambda ocol, matcol, offset: indices(ocol, matcol, offset, both_sorted=True)
+        else:
+            indexfunc = lambda ocol, matcol, offset: np.searchsorted(ocol, matcol) + offset
+        return accessors + (indexfunc,)
+
+    adim, acol, adata, arow, afindidx = accessors(a)
+    bdim, bcol, bdata, brow, bfindidx = accessors(b)
+
+    if a.shape[:2] != b.shape[:2] or (adim != bdim and not (adim == 1 or bdim == 1)):
         raise ValueError(f"could not broadcast sparse matrices {a.shape} and {b.shape}")
 
-    shape = a.shape
-    dtype = kwargs.get("dtype", np.result_type(a.dtype, b.dtype))
-    out = SparseCSR(shape, dtype=dtype, nnzpr=1, nnz=a.shape[0])
+    # create union of the sparsity pattern
+    out = SparseCSR.sparsity_union(a, b, dim=max(adim, bdim))
 
-    # Now we need to add things to the sparsity pattern
-    for r in range(shape[0]):
-        asl = np.arange(a.ptr[r], a.ptr[r] + a.ncol[r])
-        aidx = out._extend(r, a.col[asl])
-        bsl = np.arange(b.ptr[r], b.ptr[r] + b.ncol[r])
-        bidx = out._extend(r, b.col[bsl])
+    for r in range(out.shape[0]):
+        offset = out.ptr[r]
+        ocol = out.col[offset:offset + out.ncol[r]]
 
-        # Find common indices
-        idx, aover, bover = np.intersect1d(aidx, bidx, return_indices=True)
-        out._D[idx, :] = ufunc(a._D[asl[aover], :],
-                               b._D[bsl[bover], :], **kwargs)
+        asl = arow(r)
+        aidx = afindidx(ocol, acol[asl], offset)
+        asl = arange(asl.start, asl.stop)
 
-        aonly = np.delete(aidx, aover)
-        out._D[aonly, :] = ufunc(a._D[np.delete(asl, aover), :], 0, **kwargs)
-        bonly = np.delete(bidx, bover)
-        out._D[bonly, :] = ufunc(0, b._D[np.delete(bsl, bover), :], **kwargs)
+        bsl = brow(r)
+        bidx = bfindidx(ocol, bcol[bsl], offset)
+        bsl = arange(bsl.start, bsl.stop)
+
+        # Common indices
+        idx, aover, bover, iaonly, ibonly = intersect_and_diff_sets(aidx, bidx)
+        out._D[idx, :] = ufunc(adata[asl[aover], :],
+                               bdata[bsl[bover], :], **kwargs)
+
+        # Remaining indices
+        out._D[aidx[iaonly]] = ufunc(adata[asl[iaonly], :], 0, **kwargs)
+        out._D[bidx[ibonly]] = ufunc(0, bdata[bsl[ibonly], :], **kwargs)
 
     return out
 
