@@ -1,11 +1,13 @@
 from numbers import Integral
 import warnings
 import functools as ftool
+import itertools
+import operator
 import numpy as np
 from numpy.lib.mixins import NDArrayOperatorsMixin
 from numpy import (
     int32,
-    insert, take, delete, argsort,
+    take, delete, argsort, lexsort,
     unique, diff, allclose,
     tile, repeat, concatenate
 )
@@ -19,7 +21,7 @@ from .messages import warn, SislError, SislWarning, tqdm_eta, deprecate_method
 from ._indices import indices_only
 from ._help import get_dtype
 from .utils.ranges import array_arange, list2str
-from .sparse import SparseCSR, isspmatrix
+from .sparse import SparseCSR, isspmatrix, _ncol_to_indptr
 
 
 __all__ = ['SparseAtom', 'SparseOrbital']
@@ -326,7 +328,7 @@ class _SparseGeometry(NDArrayOperatorsMixin):
 
         self.geometry.set_nsc(*args, **kwargs)
 
-    def transpose(self):
+    def transpose(self, sort=True):
         """ Create the transposed sparse geometry by interchanging supercell indices
 
         Sparse geometries are (typically) relying on symmetry in the supercell picture.
@@ -337,10 +339,15 @@ class _SparseGeometry(NDArrayOperatorsMixin):
         row, `r`, and column `c` in a given supercell `(i,j,k)` will be transposed
         into row `c`, column `r` in the supercell `(-i,-j,-k)`.
 
+        Parameters
+        ----------
+        sort : bool, optional
+           the returned columns for the transposed structure will be sorted
+           if this is true, default
+
         Notes
         -----
-        For Hamiltonians with non-collinear or spin-orbit there is no transposing of the
-        sub-spin matrix box. This needs to be done *manually*.
+        The components for each sparse element are not changed in this method.
 
         Examples
         --------
@@ -381,7 +388,7 @@ class _SparseGeometry(NDArrayOperatorsMixin):
             D = csr._D.copy()
         else:
             idx = array_arange(csr.ptr[:-1], n=ncol, dtype=int32)
-            ptr = insert(_a.cumsumi(ncol), 0, 0)
+            ptr = _ncol_to_indptr(ncol)
             col = csr.col[idx]
             D = csr._D[idx, :].copy()
             del idx
@@ -389,8 +396,7 @@ class _SparseGeometry(NDArrayOperatorsMixin):
         # figure out rows where ncol is > 0
         # we skip the first column
         row_nonzero = (ncol > 0).nonzero()[0]
-        row = repeat(row_nonzero.astype(np.int32, copy=False),
-                     ncol[row_nonzero]).astype(np.int32, copy=False)
+        row = repeat(row_nonzero.astype(np.int32, copy=False), ncol[row_nonzero])
 
         # Now we have the DOK format
         #  row, col, _D
@@ -411,21 +417,24 @@ class _SparseGeometry(NDArrayOperatorsMixin):
         T._csr.ncol[rows] = nrow
         del rows
 
-        # Now we have everything ready...
-        # Simply figure out how to sort the columns
-        # such that we have them unified.
-        idx = argsort(col)
+        if sort:
+            # also sort individual rows for each column
+            idx = lexsort((row, col))
+        else:
+            # sort columns to get transposed values.
+            # This will randomize the rows
+            idx = argsort(col)
 
         # Our new data will then be
         T._csr.col = row[idx]
         del row
         T._csr._D = D[idx]
         del D
-        T._csr.ptr = insert(_a.cumsumi(T._csr.ncol), 0, 0)
+        T._csr.ptr = _ncol_to_indptr(T._csr.ncol)
 
-        # For-sure we haven't sorted the columns.
-        # We haven't changed the number of non-zeros
-        T._csr._finalized = False
+        # If `sort` we have everything sorted, otherwise it
+        # is not ensured
+        T._csr._finalized = sort
 
         return T
 
@@ -496,6 +505,8 @@ class _SparseGeometry(NDArrayOperatorsMixin):
         --------
         construct : routine to create the sparse matrix from a generic function (as returned from `create_construct`)
         """
+        if len(R) != len(param):
+            raise ValueError(f"{self.__class__.__name__}.create_construct got different lengths of `R` and `param`")
 
         def func(self, ia, atoms, atoms_xyz=None):
             idx = self.geometry.close(ia, R=R, atoms=atoms, atoms_xyz=atoms_xyz)
@@ -708,35 +719,12 @@ class _SparseGeometry(NDArrayOperatorsMixin):
         if isinstance(P, tuple):
             P = list(P)
 
-        # Number of dimensions
-        dim = len(P)
-        nnzpr = 1
-        # Sort all indices for the passed sparse matrices
-        for i in range(dim):
-            P[i] = P[i].tocsr()
-            P[i].sort_indices()
-            P[i].sum_duplicates()
-
-            # Figure out the maximum connections per
-            # row to reduce number of re-allocations to 0
-            nnzpr = max(nnzpr, P[i].nnz // P[i].shape[0])
-
-        # Create the sparse object
-        p = cls(geometry, dim, P[0].dtype, nnzpr, **kwargs)
+        p = cls(geometry, len(P), P[0].dtype, 1, **kwargs)
+        p._csr = p._csr.fromsp(*P, dtype=kwargs.get("dtype"))
 
         if p._size != P[0].shape[0]:
-            raise ValueError(cls.__name__ + '.fromsp cannot create a new class, the geometry ' + \
-                             'and sparse matrices does not have coinciding dimensions size != P[0].shape[0]')
-
-        for i in range(dim):
-            ptr = P[i].indptr
-            col = P[i].indices
-            D = P[i].data
-
-            # loop and add elements
-            for r in range(p.shape[0]):
-                sl = slice(ptr[r], ptr[r+1], None)
-                p[r, col[sl], i] = D[sl]
+            raise ValueError(f"{cls.__name__}.fromsp cannot create a new class, the geometry "
+                             "and sparse matrices does not have coinciding dimensions size != P[0].shape[0]")
 
         return p
 
@@ -1085,7 +1073,7 @@ class SparseAtom(_SparseGeometry):
         # Create new indptr, indices and D
         ncol = tile(ncol, reps)
         # Now indptr is complete
-        indptr = insert(_a.cumsumi(ncol), 0, 0)
+        indptr = _ncol_to_indptr(ncol)
         del ncol
         indices = _a.emptyi([indptr[-1]])
         indices.shape = (reps, -1)
@@ -1099,9 +1087,9 @@ class SparseAtom(_SparseGeometry):
         # Create repetitions
         for rep in range(reps):
             # Correct the supercell information
-            isc[:, axis] = JA // na_n
+            isc[:, axis], mod = np.divmod(JA, na_n)
 
-            indices[rep, :] = JA % na_n + sc_index(isc) * na_n
+            indices[rep, :] = mod + sc_index(isc) * na_n
 
             # Step atoms
             JA += na
@@ -1165,7 +1153,7 @@ class SparseAtom(_SparseGeometry):
         # Create new indptr, indices and D
         ncol = repeat(ncol, reps)
         # Now indptr is complete
-        indptr = insert(_a.cumsumi(ncol), 0, 0)
+        indptr = _ncol_to_indptr(ncol)
         del ncol
         indices = _a.emptyi([indptr[-1]])
 
@@ -1182,11 +1170,11 @@ class SparseAtom(_SparseGeometry):
             # Update the offset
             A += 1
             # Correct supercell information
-            isc[:, axis] = A // reps
+            isc[:, axis], mod = np.divmod(A, reps)
 
             # Create the indices for the repetition
             idx = array_arange(indptr[rep:-1:reps], n=csr.ncol)
-            indices[idx] = JA + A % reps + sc_index(isc) * na_n
+            indices[idx] = JA + mod + sc_index(isc) * na_n
 
         # Clean-up
         del isc, JA, A, idx
@@ -1669,7 +1657,7 @@ class SparseOrbital(_SparseGeometry):
 
         new_atom = old_atom.sub(orbitals)
         # Rename the new-atom to <>_1_2 for orbital == [1, 2]
-        new_atom.tag += '_' + '_'.join(map(str, orbitals))
+        new_atom._tag += '_' + '_'.join(map(str, orbitals))
 
         # We catch the warning about reducing the number of orbitals!
         with warnings.catch_warnings():
@@ -1744,7 +1732,7 @@ class SparseOrbital(_SparseGeometry):
         # Create new indptr, indices and D
         ncol = tile(ncol, reps)
         # Now indptr is complete
-        indptr = insert(_a.cumsumi(ncol), 0, 0)
+        indptr = _ncol_to_indptr(ncol)
         del ncol
         indices = _a.emptyi([indptr[-1]])
         indices.shape = (reps, -1)
@@ -1758,9 +1746,9 @@ class SparseOrbital(_SparseGeometry):
         # Create repetitions
         for rep in range(reps):
             # Correct the supercell information
-            isc[:, axis] = JO // no_n
+            isc[:, axis], mod = np.divmod(JO, no_n)
 
-            indices[rep, :] = JO % no_n + sc_index(isc) * no_n
+            indices[rep, :] = mod + sc_index(isc) * no_n
 
             # Step orbitals
             JO += no
@@ -1824,11 +1812,11 @@ class SparseOrbital(_SparseGeometry):
                            repeat(geom.firsto[1:], reps))
         ncol = csr.ncol[idx]
         # Now indptr is complete
-        indptr = insert(_a.cumsumi(ncol), 0, 0)
+        indptr = _ncol_to_indptr(ncol)
         # Note that D above is already reduced to a *finalized* state
         # So we have to re-create the reduced index pointer
         # Then we take repeat the data by smart indexing
-        D = D[array_arange(insert(_a.cumsumi(csr.ncol), 0, 0)[idx], n=ncol), :]
+        D = D[array_arange(_ncol_to_indptr(csr.ncol)[idx], n=ncol), :]
         del ncol, idx
         indices = _a.emptyi([indptr[-1]])
 
@@ -1871,11 +1859,11 @@ class SparseOrbital(_SparseGeometry):
             # Update the offset
             O += 1
             # Correct supercell information
-            isc[:, axis] = O // reps
+            isc[:, axis], mod = np.divmod(O, reps)
 
             # Create the indices for the repetition
             idx = array_arange(indptr[array_arange(OA, n=AO)], n=ncol)
-            indices[idx] = JO + oA * (O % reps) + sc_index(isc) * no_n
+            indices[idx] = JO + oA * mod + sc_index(isc) * no_n
 
         # Clean-up
         del isc, JO, O, OA, AO, idx
@@ -2009,7 +1997,7 @@ class SparseOrbital(_SparseGeometry):
             geom = self.geometry.add(other.geometry, offset=offset)
         else:
             # Same effect but also adds the lattice vectors
-            geom = self.geometry.append(other, axis, offset=offset)
+            geom = self.geometry.append(other.geometry, axis, offset=offset)
 
         # Now we have the correct geometry, then create the correct
         # class
@@ -2184,24 +2172,24 @@ class SparseOrbital(_SparseGeometry):
             a new instance with two sparse matrices joined and appended together
         """
         if not (type(self) is type(other)):
-            raise ValueError(self.__class__.__name__ + f'.append requires other to be of same type: {other.__class__.__name__}')
+            raise ValueError(f"{self.__class__.__name__}.append requires other to be of same type: {other.__class__.__name__}")
 
         if self.geometry.nsc[axis] > 3 or other.geometry.nsc[axis] > 3:
-            raise ValueError(self.__class__.__name__ + '.append requires sparse-geometries to maximally '
-                             'have 3 supercells along appending axis.')
+            raise ValueError(f"{self.__class__.__name__}.append requires sparse-geometries to maximally "
+                             "have 3 supercells along appending axis.")
 
         if not allclose(self.geometry.nsc, other.geometry.nsc):
-            raise ValueError(self.__class__.__name__ + '.append requires sparse-geometries to have the same '
-                             'number of supercells along all directions.')
+            raise ValueError(f"{self.__class__.__name__}.append requires sparse-geometries to have the same "
+                             "number of supercells along all directions.")
 
         if not allclose(self.geometry.sc._isc_off, other.geometry.sc._isc_off):
-            raise ValueError(self.__class__.__name__ + '.append requires supercell offsets to be the same.')
+            raise ValueError(f"{self.__class__.__name__}.append requires supercell offsets to be the same.")
 
         if self.dtype != other.dtype:
-            raise ValueError(self.__class__.__name__ + '.append requires the same datatypes in the two matrices.')
+            raise ValueError(f"{self.__class__.__name__}.append requires the same datatypes in the two matrices.")
 
         if self.dim != other.dim:
-            raise ValueError(self.__class__.__name__ + '.append requires the same number of dimensions in the matrix.')
+            raise ValueError(f"{self.__class__.__name__}.append requires the same number of dimensions in the matrix.")
 
         if np.asarray(scale).size == 1:
             scale = np.array([scale, scale])
@@ -2259,40 +2247,68 @@ class SparseOrbital(_SparseGeometry):
         # allowed
         def _test(diff):
             if diff.size != diff.nonzero()[0].size:
-                raise ValueError(self.__class__.__name__ + '.append requires that there is maximally one '
-                                 'atom overlapping one other atom in the other structure.')
+                raise ValueError(f"{self.__class__.__name__}.append requires that there is maximally one "
+                                 "atom overlapping one other atom in the other structure.")
         _test(diff(idx_s_first))
         _test(diff(idx_s_last))
         # Also ensure that atoms have the same number of orbitals in the two cases
         if (not allclose(self.geometry.orbitals[idx_s_first], other.geometry.orbitals[idx_o_first])) or \
            (not allclose(self.geometry.orbitals[idx_s_last], other.geometry.orbitals[idx_o_last])):
-            raise ValueError(self.__class__.__name__ + '.append requires the overlapping geometries '
-                             'to have the same number of orbitals per atom that is to be replaced.')
+            raise ValueError(f"{self.__class__.__name__}.append requires the overlapping geometries "
+                             "to have the same number of orbitals per atom that is to be replaced.")
 
-        def _check_edges_and_coordinates(spgeom, array, isc):
+        def _check_edges_and_coordinates(spgeom, atoms, isc, err_help):
             # Figure out if we have found all couplings
             geom = spgeom.geometry
             # Find orbitals that we wish to exclude from the orbital connections
             # This ensures that we only find couplings crossing the supercell boundaries
-            exclude = _a.arangei(geom.no_s).reshape(geom.n_s, -1)
-            exclude = exclude[geom.sc.sc_index(isc), :].ravel()
+            irrelevant_sc = delete(_a.arangei(geom.sc.n_s), geom.sc.sc_index(isc))
+            sc_orbitals = _a.arangei(geom.no_s).reshape(geom.sc.n_s, -1)
+            exclude = sc_orbitals[irrelevant_sc, :].ravel()
             # get connections and transfer them to the unit-cell
-            edges = geom.sc2uc(geom.o2a(spgeom.edges(orbitals=_a.arangei(geom.no), exclude=exclude), True), True)
-            if not np.all(np.isin(edges, array, assume_unique=True)):
-                raise ValueError(self.__class__.__name__ + '.append requires that the coupling coordinates '
-                                 'coincide between the two geometries:\n   '
-                                 'overlapping atoms: {}'.format(list2str(idx_s_first)) + '\n   '
-                                 'connected atoms: {}'.format(list2str(edges)))
-        isc = [None] * 3
+            edges_sc = geom.o2a(spgeom.edges(orbitals=_a.arangei(geom.no), exclude=exclude), True)
+            edges_uc = geom.sc2uc(edges_sc, True)
+            edges_valid = np.isin(edges_uc, atoms, assume_unique=True)
+            if not np.all(edges_valid):
+                edges_uc = edges_sc % geom.na
+                # Reduce edges to those that are faulty
+                edges_valid = np.isin(edges_uc, atoms, assume_unique=False)
+                errors = edges_sc[~edges_valid]
+                # Get supercell offset and unit-cell atom
+                isc_off, uca = np.divmod(errors, geom.na)
+                # group atoms for each supercell index
+                # find unique supercell offsets
+                sc_off_atoms = []
+                # This will be much faster
+                for isc in unique(isc_off):
+                    idx = (isc_off == isc).nonzero()[0]
+                    sc_off_atoms.append("{k}: {v}".format(
+                        k=str(geom.sc.sc_off[isc]),
+                        v=list2str(np.sort(uca[idx]))))
+                sc_off_atoms = "\n   ".join(sc_off_atoms)
+                raise ValueError(f"{self.__class__.__name__}.append requires matching coupling elements.\n\n"
+                                 f"The following atoms in a {err_help[1]} connection of `{err_help[0]}` super-cell "
+                                 "are connected from the unit cell, but are not found in matches:\n\n"
+                                 f"[sc-offset]: atoms\n   {sc_off_atoms}")
+
+        # setup supercells to look up
+        isc_inplace = [None] * 3
+        isc_inplace[axis] = 0
+        isc_forward = isc_inplace.copy()
+        isc_forward[axis] = 1
+        isc_back = isc_inplace.copy()
+        isc_back[axis] = -1
 
         # Check that edges and overlapping atoms are the same (or at least that the
         # edges are all in the overlapping region)
-        isc[axis] = 1
-        _check_edges_and_coordinates(self, idx_s_first, isc)
-        _check_edges_and_coordinates(other, idx_o_first, isc)
-        isc[axis] = -1
-        _check_edges_and_coordinates(self, idx_s_last, isc)
-        _check_edges_and_coordinates(other, idx_o_last, isc)
+        # [self|other]: self sc-connections forward must be on left-aligned matching atoms
+        _check_edges_and_coordinates(self, idx_s_first, isc_forward, err_help=("self", "forward"))
+        # [other|self]: other sc-connections forward must be on left-aligned matching atoms
+        _check_edges_and_coordinates(other, idx_o_first, isc_forward, err_help=("other", "forward"))
+        # [other|self]: self sc-connections backward must be on right-aligned matching atoms
+        _check_edges_and_coordinates(self, idx_s_last, isc_back, err_help=("self", "backward"))
+        # [self|other]: other sc-connections backward must be on right-aligned matching atoms
+        _check_edges_and_coordinates(other, idx_o_last, isc_back, ("other", "backward"))
 
         # Now we have ensured that the overlapping coordinates and the connectivity graph
         # co-incide and that we can actually perform the merge.
@@ -2308,14 +2324,11 @@ class SparseOrbital(_SparseGeometry):
             off = delete(geom.sc.sc_off[idx].T, axis, axis=0)
             return idx[np.lexsort(off)]
 
-        isc[axis] = 1
-        idx_iscP = idx[_sc_index_sort(isc)]
-        isc[axis] = 0
-        idx_isc0 = idx[_sc_index_sort(isc)]
-        isc[axis] = -1
-        idx_iscM = idx[_sc_index_sort(isc)]
+        idx_iscP = idx[_sc_index_sort(isc_forward)]
+        idx_isc0 = idx[_sc_index_sort(isc_inplace)]
+        idx_iscM = idx[_sc_index_sort(isc_back)]
         # Clean (for me to know what to do in this code)
-        del idx, isc, _sc_index_sort
+        del idx, _sc_index_sort
 
         # First scale all values
         idx_s_first = self.geometry.a2o(idx_s_first, all=True).reshape(1, -1)
