@@ -88,9 +88,9 @@ class TestGeometry:
             i += 1
         assert i == 2
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_tile0(self, setup):
-        t = setup.g.tile(0, 0)
+        with pytest.raises(ValueError):
+            t = setup.g.tile(0, 0)
 
     def test_tile1(self, setup):
         cell = np.copy(setup.g.sc.cell)
@@ -158,9 +158,9 @@ class TestGeometry:
         t = setup.g.tile(2, 0).tile(2, 2)
         assert np.allclose(t[:len(setup.g), :], setup.g.xyz)
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_repeat0(self, setup):
-        t = setup.g.repeat(0, 0)
+        with pytest.raises(ValueError):
+            t = setup.g.repeat(0, 0)
 
     def test_repeat1(self, setup):
         cell = np.copy(setup.g.sc.cell)
@@ -515,13 +515,13 @@ class TestGeometry:
             assert np.allclose(s.cell[axis, :], setup.g.cell[axis, :]* 2)
             assert np.allclose(s.cell[axis, :], setup.g.cell[axis, :]* 2)
 
-    @pytest.mark.xfail(raises=ValueError)
-    def test_append_xfail(self, setup):
-        s = setup.g.append(setup.g, 0, offset='not')
+    def test_append_raise_valueerror(self, setup):
+        with pytest.raises(ValueError):
+            s = setup.g.append(setup.g, 0, offset='not')
 
-    @pytest.mark.xfail(raises=ValueError)
-    def test_prepend_xfail(self, setup):
-        s = setup.g.prepend(setup.g, 0, offset='not')
+    def test_prepend_raise_valueerror(self, setup):
+        with pytest.raises(ValueError):
+            s = setup.g.prepend(setup.g, 0, offset='not')
 
     def test_append_prepend_offset(self, setup):
         for axis in [0, 1, 2]:
@@ -557,9 +557,9 @@ class TestGeometry:
         al = setup.g.center(what='mass')
         al = setup.g.center(what='mm(xyz)')
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_center_raise(self, setup):
-        al = setup.g.center(what='unknown')
+        with pytest.raises(ValueError):
+            al = setup.g.center(what='unknown')
 
     def test___add1__(self, setup):
         n = len(setup.g)
@@ -745,16 +745,17 @@ class TestGeometry:
 
     def test_close_within3(self, setup):
         g = setup.g.repeat(6, 0).repeat(6, 1)
-        args = {'ret_xyz': True, 'ret_rij': True}
+        args = {'ret_xyz': True, 'ret_rij': True, 'ret_isc': True}
         for ia in g:
             shapes = [Sphere(0.1, g[ia]),
                       Sphere(1.5, g[ia])]
-            i, xa, d = g.close(ia, R=(0.1, 1.5), **args)
-            ii, xai, di = g.within(shapes, **args)
+            i, xa, d, isc = g.close(ia, R=(0.1, 1.5), **args)
+            ii, xai, di, isci = g.within(shapes, **args)
             for j in [0, 1]:
                 assert np.all(i[j] == ii[j])
                 assert np.allclose(xa[j], xai[j])
                 assert np.allclose(d[j], di[j])
+                assert np.allclose(isc[j], isci[j])
 
     def test_within_inf1(self, setup):
         g = setup.g.translate([0.05] * 3)
@@ -818,11 +819,15 @@ class TestGeometry:
         assert idx[1][1].shape[1] == 3
 
         # Return index of two things
-        idx = setup.mol.close(point, R=(.1, 1.1), ret_xyz=True, ret_rij=True)
-        # [[idx-1, idx-2], [coord-1, coord-2], [dist-1, dist-2]]
-        assert len(idx) == 3
+        idx = setup.mol.close(point, R=(.1, 1.1), ret_xyz=True, ret_rij=True, ret_isc=True)
+        # [[idx-1, idx-2], [coord-1, coord-2], [dist-1, dist-2], [isc-1, isc-2]]
+        assert len(idx) == 4
         assert len(idx[0]) == 2
         assert len(idx[1]) == 2
+        assert len(idx[2]) == 2
+        assert len(idx[3]) == 2
+        assert all(len(idx[0][0]) == len(t[0]) for t in idx)
+        assert all(len(idx[0][1]) == len(t[1]) for t in idx)
         # idx-1
         assert len(idx[0][0].shape) == 1
         assert idx[0][0].shape[0] == 1
@@ -838,6 +843,11 @@ class TestGeometry:
         assert idx[2][0].shape[0] == 1
         # dist-2
         assert idx[2][1].shape[0] == 1
+        # isc-1
+        assert len(idx[3][0].shape) == 2
+        assert idx[3][0].shape[1] == 3
+        # isc-2
+        assert idx[3][1].shape[1] == 3
 
         # Return index of two things
         idx = setup.mol.close(point, R=(.1, 1.1), ret_rij=True)
@@ -984,16 +994,16 @@ class TestGeometry:
             assert np.allclose(g1.cell[i, :], g2.cell[i, :])
         assert not np.allclose(g1.cell[2, :], g2.cell[2, :])
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_distance1(self, setup):
         geom = Geometry(setup.g.xyz, Atom[6])
         # maxR is undefined
-        d = geom.distance()
+        with pytest.raises(ValueError):
+            d = geom.distance()
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_distance2(self, setup):
         geom = Geometry(setup.g.xyz, Atom[6])
-        d = geom.distance(R=1.42, method='unknown_numpy_function')
+        with pytest.raises(ValueError):
+            d = geom.distance(R=1.42, method='unknown_numpy_function')
 
     def test_distance3(self, setup):
         geom = setup.g.copy()
@@ -1207,31 +1217,26 @@ class TestGeometry:
         del g.names['B']
         assert len(g.names) == 1
 
-    @pytest.mark.xfail(raises=SislError)
     def test_geometry_groups_raise(self):
         g = sisl_geom.graphene()
         g['A'] = 1
-        g['A'] = [1, 2]
+        with pytest.raises(SislError):
+            g['A'] = [1, 2]
 
-    @pytest.mark.xfail(raises=ValueError)
-    def test_geometry_sanitize_raise(self):
-        g = sisl_geom.graphene()
-        xyz = g.axyz([[[0, 1]]])
-
-    @pytest.mark.xfail(raises=ValueError)
     def test_geometry_as_primary_raise_nondivisable(self):
         g = sisl_geom.graphene()
-        g.as_primary(3)
+        with pytest.raises(ValueError):
+            g.as_primary(3)
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_geometry_cut_raise_nondivisable(self):
         g = sisl_geom.graphene()
-        g.cut(3, 0)
+        with pytest.raises(ValueError):
+            g.cut(3, 0)
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_geometry_iR_negative_R(self):
         g = sisl_geom.graphene()
-        g.iR(-1.)
+        with pytest.raises(ValueError):
+            g.iR(R=-1.)
 
     @pytest.mark.parametrize("geometry", [sisl_geom.graphene(),
                                           sisl_geom.diamond(),
@@ -1270,11 +1275,11 @@ class TestGeometry:
     # Test ASE (but only fail if present)
 
     def test_geometry_toASE(self):
-        ase = pytest.importorskip("ase")
+        pytest.importorskip("ase", reason="ase not available")
         sisl_geom.graphene().toASE()
 
     def test_geometry_fromASE(self):
-        ase = pytest.importorskip("ase")
+        pytest.importorskip("ase", reason="ase not available")
         gr = sisl_geom.graphene()
         ase_geom = gr.toASE()
         from_ase = Geometry.fromASE(ase_geom)
@@ -1282,6 +1287,21 @@ class TestGeometry:
         assert np.allclose(gr.cell, from_ase.cell)
         assert np.allclose(gr.atoms.Z, from_ase.atoms.Z)
 
+    def test_geometry_ase_new_to(self):
+        pytest.importorskip("ase", reason="ase not available")
+        gr = sisl_geom.graphene()
+        to_ase = gr.to.ase()
+        from_ase = gr.new(to_ase)
+        assert gr.equal(from_ase, R=False)
+
+    def test_geometry_pandas_to(self):
+        pytest.importorskip("pandas", reason="pandas not available")
+        gr = sisl_geom.graphene()
+        df = gr.to.df()
+        assert np.allclose(df["x"], gr.xyz[:, 0])
+        assert np.allclose(df["y"], gr.xyz[:, 1])
+        assert np.allclose(df["z"], gr.xyz[:, 2])
+
     def test_geometry_overlapping_atoms(self):
         gr22 = sisl_geom.graphene().tile(2, 0).tile(2, 1)
         gr44 = gr22.tile(2, 0).tile(2, 1)
@@ -1423,11 +1443,13 @@ def test_geometry_sort_group():
     assert np.allclose(BN.atoms.Z, BN3.atoms.Z)
 
 
-@pytest.mark.xfail(raises=ValueError)
 def test_geometry_sort_fail_keyword():
-    sisl_geom.bilayer().sort(not_found_keyword=True)
+    with pytest.raises(ValueError):
+        sisl_geom.bilayer().sort(not_found_keyword=True)
 
 
+@pytest.mark.category
+@pytest.mark.geom_category
 def test_geometry_sanitize_atom():
     bi = sisl_geom.bilayer(bottom_atoms=Atom[6], top_atoms=(Atom[5], Atom[7])).tile(2, 0).repeat(2, 1)
     C_idx = (bi.atoms.Z == 6).nonzero()[0]
@@ -1436,3 +1458,20 @@ def test_geometry_sanitize_atom():
     assert np.allclose(only_C, check_C)
     only_C = bi.axyz(bi.atoms.Z == 6)
     assert np.allclose(only_C, check_C)
+    # with dict redirect
+    only_C = bi.axyz({'Z': 6})
+    assert np.allclose(only_C, check_C)
+    # Using a dict that has multiple keys. This basically checks
+    # that it accepts generic categories such as the AndCategory
+    bi2 = bi.copy()
+    bi2.atoms["C"] = Atom("C", R=1.9)
+    only_C = bi2.axyz({'Z': 6, "neighbours": 3})
+    assert np.allclose(only_C, check_C)
+
+    tup_01 = (0, 2)
+    list_01 = [0, 2]
+    ndarray_01 = np.array(list_01)
+    assert np.allclose(bi._sanitize_atoms(tup_01),
+                       bi._sanitize_atoms(list_01))
+    assert np.allclose(bi._sanitize_atoms(ndarray_01),
+                       bi._sanitize_atoms(list_01))
