@@ -23,11 +23,11 @@ def setup():
 @pytest.mark.sparse_geometry
 class TestSparseAtom:
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_fail_align1(self, setup):
         s = SparseAtom(setup.g * 2)
         str(s)
-        setup.s1.spalign(s)
+        with pytest.raises(ValueError):
+            setup.s1.spalign(s)
 
     def test_align1(self, setup):
         s = SparseAtom(setup.g)
@@ -101,6 +101,11 @@ class TestSparseAtom:
         c = s2.nonzero(atoms=0, only_col=True)
         assert np.allclose(c, [0, 2, 3])
 
+    def test_create_construct_different_length(self, setup):
+        s1 = SparseAtom(setup.g)
+        with pytest.raises(ValueError):
+            s1.construct([[0.1, 1.5], [1]])
+
     def test_cut1(self, setup):
         s1 = SparseAtom(setup.g)
         s1.construct([[0.1, 1.5], [1, 2]])
@@ -131,11 +136,11 @@ class TestSparseAtom:
             i += 1
         assert i == s1.nnz
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_rij_fail1(self, setup):
         s = SparseOrbital(setup.g.copy())
         s.construct([[0.1, 1.5], [1, 2]])
-        s.rij(what='none')
+        with pytest.raises(ValueError):
+            s.rij(what='none')
 
     def test_rij_atom(self, setup):
         s = SparseAtom(setup.g.copy())
@@ -388,7 +393,7 @@ class TestSparseAtom:
         lil = sc.sparse.lil_matrix((g.na, g.na_s), dtype=np.int32)
         lil[0, [1, 2, 3]] = 1
         lil[1, [2, 4, 1]] = 2
-        s1 = SparseAtom.fromsp(g, [lil])
+        s1 = SparseAtom.fromsp(g, [lil], unknown_key="hello")
         assert s1.nnz == 6
         assert np.allclose(s1.shape, [g.na, g.na_s, 1])
 
@@ -414,7 +419,6 @@ class TestSparseAtom:
         assert np.allclose(s1[1, [1, 2, 4], 0], np.zeros([3], np.int32))
         assert np.allclose(s1[1, [1, 2, 4], 1], np.ones([3], np.int32)*2)
 
-    @pytest.mark.xfail(raises=ValueError)
     def test_fromsp4(self, setup):
         g = setup.g.repeat(2, 0).tile(2, 1)
         lil1 = sc.sparse.lil_matrix((g.na, g.na_s), dtype=np.int32)
@@ -423,7 +427,8 @@ class TestSparseAtom:
         lil2[1, [2, 4, 1]] = 2
 
         # Ensure that one does not mix everything.
-        SparseAtom.fromsp(setup.g.copy(), [lil1, lil2])
+        with pytest.raises(ValueError):
+            SparseAtom.fromsp(setup.g.copy(), [lil1, lil2])
 
     def test_pickle(self, setup):
         import pickle as p
@@ -439,9 +444,9 @@ class TestSparseAtom:
         assert s.spsame(S)
 
 
-@pytest.mark.parametrize("n0", [1, 2, 4])
-@pytest.mark.parametrize("n1", [1, 2, 4])
-@pytest.mark.parametrize("n2", [1, 2, 4])
+@pytest.mark.parametrize("n0", [1, 2])
+@pytest.mark.parametrize("n1", [1, 3])
+@pytest.mark.parametrize("n2", [1, 4])
 def test_sparse_atom_symmetric(n0, n1, n2):
     g = fcc(1., Atom(1, R=1.5)) * 2
     s = SparseAtom(g)
