@@ -1,19 +1,12 @@
 """
-====================================
-Utility routines (:mod:`sisl.utils`)
-====================================
-
-.. module:: sisl.utils
-   :noindex:
+Utility routines
+================
 
 Several utility functions are used throughout sisl.
 
 Range routines
 ==============
 
-.. autosummary::
-   :toctree:
-
    array_arange - fast creation of sub-aranges
    strmap
    strseq
@@ -25,9 +18,6 @@ Range routines
 Miscellaneous routines
 ======================
 
-.. autosummary::
-   :toctree:
-
    str_spec
    direction - abc/012 -> 012
    angle - radian to degree
